#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Library/audit
#   Description: A set of helpers for audit testing.
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2019 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

rlJournalStart
    rlPhaseStartSetup
        rlRun "rlImport audit/testing" || rlDie
        rlFileBackup --clean "/var/log/audit/" 
        rlFileBackup --clean "$auditDaemonConf" 
        rlFileBackup --clean "$auditDispatcherConf" 
        rlServiceStop "auditd"
    rlPhaseEnd

    rlPhaseStartTest
    
        # Configuration updates.
        rlRun "auditConfig 'freq' '1'" 0
        rlAssertGrep "^freq = 1$" /etc/audit/auditd.conf
        rlRun "auditConfig 'freq' '2'" 0
        rlAssertGrep "^freq = 2$" /etc/audit/auditd.conf
        rlRun "auditConfig 'freq' '3' /etc/audit/auditd.conf" 0
        rlAssertGrep "^freq = 3$" /etc/audit/auditd.conf
        rlRun "auditConfig 'freq' '4' $auditDaemonConf" 0
        rlAssertGrep "^freq = 4$" /etc/audit/auditd.conf
        rlRun "auditConfig 'queue_depth' '300' $auditDispatcherConf" 0
        if !rlIsRHEL "<8"; then
            rlAssertGrep "^queue_depth = 300$" /etc/audit/audisp-remote.conf
        else
            rlAssertGrep "^queue_depth = 300$" /etc/audisp/audisp-remote.conf
        fi
        
        # Prepare for next testing.
        rlServiceStart "auditd"
        rlRun "auditConfig 'flush' 'INCREMENTAL'" 0
        rlRun "auditConfig 'freq' '1'" 0

        # Waiting for audit message.
        ts=$(date "+%T")
        rlRun "auditctl -m 'test1'" 0
        rlRun "auditWaitForEvent $ts 'USER' 'test1' '30'" 0
        rlRun "auditWaitForEvent $ts 'USER' 'test2' '30'" 1
        rlRun "auditWaitForEvent $ts 'USER' 'test1' '30' test.msg" 0
        rlAssertGrep "type=USER.*msg='test1 exe=" test.msg

        # Waiting for other type.
        rlRun "auditWaitForEvent 'recent' 'SERVICE_START' 'unit=auditd' '30'" 0

        # Using specific ausearch paramters (-i).
        rlRun "auditWaitForEvent 'recent' 'SERVICE_START' 'unit=auditd' '30' test.msg -i" 0
        rlAssertGrep "ses=unset" test.msg

        # Log filling.
        size_before=$(stat -c "%s" /var/log/audit/* | paste -sd+ | bc)
        rlRun "auditIncreaseLogSize 10 30" 0
        size_after=$(stat -c "%s" /var/log/audit/* | paste -sd+ | bc)
        rlRun "[ $[($size_after-$size_before)/1024] -gt 10000 ]" 0
        rlRun "auditIncreaseLogSize 5 60" 0
        size_after=$(stat -c "%s" /var/log/audit/* | paste -sd+ | bc)
        rlRun "[ $[($size_after-$size_before)/1024] -gt 15000 ]" 0

    rlPhaseStartTest
    
    rlPhaseStartCleanup
        rlFileRestore
        rlServiceRestore "auditd"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
