#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1019898-audit-auditd-calling-audit-log-start-causes-hang
#   Description: Test for BZ#1019898 (audit auditd calling audit_log_start causes hang)
#   Author: Karel Srot <ksrot@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2014 Red Hat, Inc.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
        rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlRun "uname -a"

        rlServiceStop auditd && sleep 3
        rlFileBackup --clean "/var/log/audit/audit.log"
        rlFileBackup --clean "/etc/audit/"
        rlServiceStart auditd

        rlRun "auditctl -b 320"
        rlRun "auditctl -a exit,always -F pid=\$(pgrep -x auditd)"
        rlRun "ping -q -f 127.0.0.1 &"
        PID=$!
        rlRun "auditctl -s"
        rlRun "auditctl -a exit,always -F pid=$PID"
        sleep 10
        rlPhaseEnd


        rlPhaseStartTest
        TIME_START=`date '+%s'`
        rlServiceStop auditd && sleep 3
        TIME_END=`date '+%s'`
        DURATION=$(( $TIME_END-$TIME_START ))
        rlLogInfo "auditd stop took $DURATION seconds"
        rlAssertGreater "audit stop should not take more than 30 seconds" 30 $DURATION
        rlPhaseEnd

        rlPhaseStartCleanup
        rlRun "kill -9 $PID"

        rlFileRestore
        rlRun "service auditd restart && sleep 3" 0 "Restarting auditd service"
        rlServiceRestore auditd
        rlPhaseEnd


rlJournalPrintText
rlJournalEnd
