#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1073063-seccomp-syscall-name-translation
#   Description: checks format of audit messages with type=SECCOMP
#   Author: Jiri Jaburek <jjaburek@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2014 Red Hat, Inc.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlFileBackup "/etc/audit/auditd.conf"	
        rlFileBackup "/var/log/audit/audit.log"
        rlServiceStop "auditd" && sleep 3
        rlRun "sed -i 's/flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
        rlRun "sed -i 's/freq.*/freq = 1/' /etc/audit/auditd.conf" 0
        rlRun "TmpDir=\$(mktemp -d)" 0 "Creating tmp directory"
        rlRun "cp -v repro.c $TmpDir/."
        rlRun "pushd $TmpDir"
        rlRun "gcc -o repro -Wall -Wextra repro.c -lseccomp"
    rlPhaseEnd

    rlPhaseStartTest "reproducer run [RAW]"
        rlRun "sed -i 's/log_format.*/log_format = RAW/' /etc/audit/auditd.conf" 0
        rlRun "service auditd start && sleep 3" 0
        ts=$(date +%H:%M:%S)
        rlRun "./repro" 159
    rlPhaseEnd

    rlPhaseStartTest "find entry using -m SECCOMP [RAW]"
        rlRun "ausearch -ts \"$ts\" -m SECCOMP &>log"
        rlAssertGrep "^type=SECCOMP" log || cat log
        cat log
    rlPhaseEnd
    rlPhaseStartTest "entry has to contain arch= (BZ#1053921) [RAW]"
        rlAssertGrep " arch=" log || cat log
    rlPhaseEnd

    rlPhaseStartTest "resolve syscall name using -i [RAW]"
        rlRun "ausearch -ts \"$ts\" -m SECCOMP -i &>log"
        rlAssertGrep " syscall=exit_group" log || cat log
    rlPhaseEnd

    rlPhaseStartTest "search for syscall using -sc [RAW]"
        rlRun "ausearch -ts \"$ts\" -m SECCOMP -i -sc exit_group &>log"
        rlAssertGrep " syscall=exit_group" log || cat log
        rlRun "service auditd stop && sleep 3" 0
    rlPhaseEnd

    rlPhaseStartTest "reproducer run [ENRICHED]"
        rlRun "sed -i 's/log_format.*/log_format = ENRICHED/' /etc/audit/auditd.conf" 0
        rlRun "service auditd start && sleep 3" 0
        ts=$(date +%H:%M:%S)
        rlRun "./repro" 159
    rlPhaseEnd

    rlPhaseStartTest "find entry using -m SECCOMP [ENRICHED]"
        rlRun "ausearch -ts \"$ts\" -m SECCOMP &>log"
        rlAssertGrep "^type=SECCOMP" log || cat log
        cat log
    rlPhaseEnd
    rlPhaseStartTest "entry has to contain arch= (BZ#1053921) [ENRICHED]"
        rlAssertGrep " arch=" log || cat log
    rlPhaseEnd

    rlPhaseStartTest "resolve syscall name using -i [ENRICHED]"
        rlRun "ausearch -ts \"$ts\" -m SECCOMP -i &>log"
        rlAssertGrep " syscall=exit_group" log || cat log
    rlPhaseEnd

    rlPhaseStartTest "search for syscall using -sc [ENRICHED]"
        rlRun "ausearch -ts \"$ts\" -m SECCOMP -i -sc exit_group &>log"
        rlAssertGrep " syscall=exit_group" log || cat log
        rlRun "service auditd stop && sleep 3" 0
    rlPhaseEnd

    rlPhaseStartCleanup
        rlRun "popd"
        rlRun "rm -r $TmpDir" 0 "Removing tmp directory"
        rlFileRestore
        rlServiceRestore "auditd"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
