
#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1155208-BUG-Audit-subsystem-not-resolving-path-name-on
#   Description: Test for BZ#1155208 (BUG Audit subsystem not resolving path name on)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2015 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlAssertRpm kernel
        rlRun "rlImport audit/testing" || rlDie
        rlFileBackup --clean "$auditDaemonConf" 
        rlFileBackup --clean "/var/log/audit/" 
        rlServiceStop "auditd" && sleep 3
        rlRun "auditConfig 'flush' 'INCREMENTAL'" 0
        rlRun "auditConfig 'freq' '1'" 0
        rlRun "service auditd start && sleep 3" 0
        rlRun "TmpDir=\$(mktemp -d)" 0 "Creating tmp directory"
        rlRun "touch $TmpDir/file" 0 "Creating tmp file"
        rlRun "auditctl -D"
        rlRun "auditctl -a always,exit -F dir=$TmpDir" 0 "Adding directory watch"
        rlRun "auditctl -l"
        rlRun "cat $auditDaemonConf"
        rlRun "service auditd status"
    rlPhaseEnd

    rlPhaseStartTest
    
        ls /dev/fd/*
        ps -aux | grep ssh
        tty

        T=$(date "+%T")
        rlRun "touch $TmpDir/file && sleep 1" 0 "Operation: CREATE (file)" 
        # The following two commands are running the same ausearch.
        # The one without unbuffer will fail incorrectly.
        # Both commands should pass.
        rlRun "unbuffer ausearch -ts $T -f file" 0
        rlRun "ausearch -ts $T -f file" 0

    rlPhaseEnd

    rlPhaseStartCleanup
        rlRun "rm -r $TmpDir" 0 "Removing tmp directory"
        rlRun "service auditd stop && sleep 3" 0
        rlFileRestore
        rlServiceRestore "auditd"
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
