#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1249813
#   Description: Test for BZ#1249813
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details. 
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "audit"
        rlAssertRpm "kernel"
        rlRun "uname -a"
        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "/var/log/audit/audit.log"
        rlServiceStop "auditd" && sleep 5
        rlRun "sed -i \"s/^flush.*/flush = incremental/\" /etc/audit/auditd.conf" 0
        rlRun "sed -i \"s/^freq.*/freq = 1/\" /etc/audit/auditd.conf" 0
        
    rlPhaseEnd

    i_max=10
    for i in $(seq 1 $i_max); do
        rlPhaseStartTest "Iteration $i/$i_max"
    
            # Setup.  
            mark=$(date "+%T")
            old_bl=$(auditctl -s | grep 'backlog_limit' | awk '{print $2}')
            rlRun "auditctl -D" 0
            rlRun "auditctl -b 1" 0

            # Trigger.
            ping -f 127.0.0.1 &
            ping_pid=$!
            rlRun "auditctl -a exit,always -S all -F pid=$ping_pid -k bz1249813" 0
            rlRun "sleep 5 && kill -s TERM $ping_pid" 0
            rlRun "auditctl -d exit,always -S all -F pid=$ping_pid -k bz1249813" 0
            rlRun "service auditd start && sleep 5" 0
            rlRun -s "auditctl --reset-lost" 0
            
            # Verify.        
            lost=$(cat $rlRun_LOG | grep 'lost' | awk -F ': ' '{print $2}')
            rlRun "test $lost -ge 0" 0
            rlRun -s "ausearch -ts $mark -i -m CONFIG_CHANGE" 0
            rlAssertGrep "lost=0" $rlRun_LOG
            rlAssertGrep "old=$lost" $rlRun_LOG
            rlRun "auditctl -s | grep 'lost 0'" 0

            # Clean-Up.
            rlRun "service auditd stop && sleep 5" 0
            rlRun "auditctl -b $old_bl" 0

        rlPhaseEnd
    done
        
    rlPhaseStartCleanup

        rlFileRestore
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
