#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1305103
#   Description: Regression test for BZ#1305103
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2010 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="kernel"

rlJournalStart

    rlPhaseStartSetup

        rlCheckMakefileRequires || rlDie
        rlFileBackup "/etc/audit/auditd.conf"
        rlServiceStop "auditd" && sleep 5
        rlRun "service auditd start" 0
        rlRun "auditctl -D" 0
        
    rlPhaseEnd

    rlPhaseStartTest

        # Setup.
        testdir=$(mktemp -d)
        
        rlRun "touch ${testdir}/foo" 0
        rlRun "auditctl -a always,exit -F dir=${testdir}" 0

        # Trigger.
        mark=$(date "+%T")
        
        rlRun "touch ${testdir}/foo" 0
        rlRun "sleep 3" 0
        
        # Verify.
        rlRun    "ausearch -ts $mark -f foo" 0
        rlRun -s "ausearch --raw -ts $mark -f foo -m PATH " 0

        rlRun "[[ $(cat $rlRun_LOG | grep PATH | wc -l) -eq 2 ]]" 0 "Checking the number of PATH events"
        inode=$(ls -di ${testdir} | awk '{print $1}')
        rlAssertGrep "type=PATH .*name=\"${testdir}/\".*inode=$inode.*(name|obj)type=PARENT" $rlRun_LOG -E
        inode=$(ls -i ${testdir}/foo | awk '{print $1}')
        rlAssertGrep "type=PATH .*name=\"${testdir}/foo\".*inode=${inode}.*(name|obj)type=NORMAL" $rlRun_LOG -E
        
    rlPhaseEnd

    rlPhaseStartCleanup

        rlRun "rm -rf $testdir" 0
        rlServiceRestore "audit"
    
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
