#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1465614
#   Description: Test for BZ#1465614
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2017 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="kernel"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "kernel" || rlDie
        rlAssertRpm "audit"  || rlDie
        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "/var/log/audit/audit.log"
        rlServiceStop "auditd" && sleep 5
        rlRun "sed -i \"s/^flush.*/flush = incremental/\" /etc/audit/auditd.conf" 0
        rlRun "sed -i \"s/^freq.*/freq = 1/\" /etc/audit/auditd.conf" 0
        rlRun "service auditd start && sleep 5" 0
        
    rlPhaseEnd

    rlPhaseStartTest

        # Setup.
        mark=$(date "+%T")
	rlRun "auditctl -D" 0
        rlRun "auditctl -a always,exit -F arch=b64 -S execve -F path=/usr/bin/ping" 0
        rlRun "auditctl -a always,exit -F arch=b64 -S capset" 0
        rlRun "auditctl -l" 0
            
        # Trigger.
        capsh --print
        rlRun "capsh --drop=cap_sys_boot -- -c 'ping -c 3 localhost'" 0
        
        # Verify.
        rlRun -s "ausearch -ts $mark" 0
        rlRun "grep -e \"type=BPRM_FCAPS .* old_pa=0\+ .* pa=0\+\" $rlRun_LOG" 0
        rlRun "grep -e \"type=CAPSET .* cap_pa=0\+\" $rlRun_LOG" 0

        # Clean-up.
        rlRun "auditctl -D" 0

    rlPhaseEnd

    rlPhaseStartCleanup

        rlFileRestore
        rlServiceRestore "auditd"
    
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
