#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1478533-RFE-Cleanup-audit-s-NETFILTER-CFG-event
#   Description: Test for BZ#1478533 (RFE Cleanup audit's NETFILTER_CFG event)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2020 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup

        rlAssertRpm "audit"
        rlAssertRpm "kernel"
        rlRun "TmpDir=\$(mktemp -d)"
        rlRun "pushd $TmpDir"

        # Let's try to make this test more stable by using a fixed
        # version of iptables that is known to work with this test.
        IPTABLES_TAG=v1.8.7
        rlRun "git clone git://git.netfilter.org/iptables"
        rlRun "pushd iptables && git reset --hard ${IPTABLES_TAG}"
        rlRun "./autogen.sh && ./configure --disable-nftables --disable-dependency-tracking && make && make install"
        rlRun "popd"
        rlServiceStart "auditd"

    rlPhaseEnd

    rlPhaseStartTest

        # Setup.
        if [ $(uname -m) == "s390x" ]; then
            syscall="socketcall"
        else
            syscall="setsockopt"
        fi
        rlRun "auditctl -a exit,always -S $syscall -k iptablesChange"

        # Trigger.
        rlRun "/usr/local/sbin/iptables -A INPUT -s 1.2.3.4 -j DROP"

        # Verify.
        rlRun -s "ausearch -k iptablesChange -i -m NETFILTER_CFG" 
        rlAssertGrep "table=filter" $rlRun_LOG
        rlAssertGrep "family=ipv4" $rlRun_LOG
        rlAssertGrep "entries=4" $rlRun_LOG
        if ! rlIsRHEL "<8.4"; then
            rlAssertGrep "op=xt_replace" $rlRun_LOG
        else
            rlAssertGrep "op=replace" $rlRun_LOG
        fi
        rlAssertGrep "entries=4" $rlRun_LOG
        rlAssertGrep "pid=" $rlRun_LOG
        rlAssertGrep "subj=" $rlRun_LOG
        rlAssertGrep "comm=iptables" $rlRun_LOG

        # Clean-up.
        rlRun "/usr/local/sbin/iptables -D INPUT -s 1.2.3.4 -j DROP"
        rlRun "auditctl -D" 

    rlPhaseEnd

    rlPhaseStartCleanup

        rlServiceRestore "auditd"
        rlRun "popd"
        rlRun "rm -r $TmpDir" 0 "Removing tmp directory"

    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
