#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1487352
#   Description: Test for BZ#1487352
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2017 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="kernel"

REBOOT_FLAG_SETUP="/var/tmp/bz1487352-reboot-setup"
REBOOT_FLAG_CLEANUP="/var/tmp/bz1487352-reboot-cleanup"

rlJournalStart
       
    if ! [ -e $REBOOT_FLAG_SETUP ] && ! [ -e $REBOOT_FLAG_CLEANUP ]; then
             
        # Setup.
        rlPhaseStartSetup

            default_kernel="$(grubby --default-kernel)"
            default_args="$(grubby --info $default_kernel | grep 'args=')"

            [[ "$default_args"  =~ audit ]] && \
                echo $default_args | sed 's/.*\(audit=[01]\).*/\1/' >/var/tmp/old_audit

            if [[ "$(uname -m)" =~ s390 ]]; then
                rlRun "grubby --zipl --args audit=1 --update-kernel $default_kernel" 0
                rlRun "zipl" 0
            else
                rlRun "grubby --args audit=1 --update-kernel $default_kernel" 0
            fi

            touch $REBOOT_FLAG_SETUP

        rlPhaseEnd

        # Trigger.
        rhts-reboot

    elif [ -e $REBOOT_FLAG_SETUP ]; then

        # Verify & Clean-up.
        rlPhaseStartTest 

            rlLog "Machine was rebooted (1/2)"
            rm -f $REBOOT_FLAG_SETUP

            rlAssertRpm "kernel" || rlDie
            rlAssertRpm "audit"  || rlDie
            
            rlRun -s "ausearch -i -ts boot -m MAC_STATUS" 0
            rlAssertGrep "audit=1" /proc/cmdline
	    if rlIsRHEL; then
		rlAssertGrep "type=PROCTITLE" $rlRun_LOG        
		rlAssertGrep "type=SYSCALL" $rlRun_LOG        
		rlAssertGrep "type=MAC_STATUS" $rlRun_LOG        
	    fi
            default_kernel="$(grubby --default-kernel)"
            default_args="$(grubby --info $default_kernel | grep 'args=')"

            old_audit="$(cat /var/tmp/old_audit)"
            
            grubb_args=""
            [ -n "$old_audit" ] || grubby_args+=" --remove-args audit"
            [ -n "$old_audit" ] && grubby_args+=" --args audit=$old_audit"            
            [[ "$(uname -m)" =~ s390 ]] && grubb_args+=" --zipl"
            rlRun "grubby $grubby_args --update-kernel $default_kernel" 0
            [[ "$(uname -m)" =~ s390 ]] && rlRun "zipl" 0

            touch $REBOOT_FLAG_CLEANUP

        rlPhaseEnd
            
        rhts-reboot

    elif [ -e $REBOOT_FLAG_CLEANUP ]; then
        
        # Clean-up.
        rlPhaseStartCleanup    

            rlLog "Machine was rebooted (2/2)"
            rm -f $REBOOT_FLAG_CLEANUP

        rlPhaseEnd
        
    fi

rlJournalPrintText

rlJournalEnd
