#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1642315-renameat2-does-not-generate-any-audit-events-for
#   Description: Test for BZ#1642315 (renameat2() does not generate any audit events for)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2019 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlRun "TmpDir=\$(mktemp -d)" 0 "Creating tmp directory"
        rlRun "pushd $TmpDir"

        rlRun "touch rename.old"
        rlServiceStart "auditd"
        [ $(uname -m) == "aarch64" ] && syscallNames="renameat,renameat2" || syscallNames="rename,renameat,renameat2"
        rlRun "auditctl -a always,exit -S ${syscallNames} -F dir=$PWD -F perm=w -F key=bz1642315" 0
    rlPhaseEnd

    rlPhaseStartTest
        ts="$(date +%T)"
        rlRun -s "strace mv rename.old rename.new 2>&1" 0
        syscall="renameat2"
        rlLog "Checking presence of ${syscall}"
        rlAssertGrep "${syscall}(" $rlRun_LOG
        rlRun -s "sleep 15 && ausearch -ts $ts -k bz1642315 -sc $syscall -i" 0
    rlPhaseEnd

    rlPhaseStartCleanup
        rlRun "auditctl -d always,exit -S ${syscallNames} -F dir=$PWD -F perm=w -F key=bz1642315" 0
        rlRun "popd"
        rlRun "rm -r $TmpDir" 0 "Removing tmp directory"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
