#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1663285-ausearch-does-not-record-device-inode-details-when
#   Description: Test for BZ#1663285 (ausearch does not record device/inode details when)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2019 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlRun "TmpDir=\$(mktemp -d)" 0
        rlRun "pushd $TmpDir"
    rlPhaseEnd

    rlPhaseStartTest
        # This test will fail if audit.log is empty, so let's wait a
        # little while before starting the test.
        rlRun "service auditd stop"
        rlFileSubmit "/var/log/audit/audit.log" "before-the-test-audit.log"
        rlRun "service auditd start && sleep 10"

        rlRun "cp /var/log/audit/audit.log ." 0
        rlRun "ausearch --input ./audit.log --checkpoint ./check.txt" 0
        rlRun -s "cat ./check.txt" 0
        device=$(stat -c "%D" ./audit.log)
        inode=$(stat -c "%i" ./audit.log)
        rlAssertGrep "dev=0x$device" $rlRun_LOG -i
        rlAssertGrep "inode=$inode" $rlRun_LOG -i
    rlPhaseEnd

    rlPhaseStartCleanup
        rlRun "popd"
        rlRun "rm -r $TmpDir" 0
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
