#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1696709-updating-auditd-is-enabling-disabled-service
#   Description: Test for BZ#1696709 (updating auditd is enabling disabled service)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2019 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup

        rlAssertRpm $PACKAGE
        rlRun "TmpDir=\$(mktemp -d)" 0 "Creating tmp directory"
        rlRun "pushd $TmpDir"
        
        # Download packages installed.
        PACKAGES+=" $(rpm -qa | egrep '^audit'  | grep -vi 'coreos')"
        PACKAGES+=" $(rpm -qa | egrep '^audisp' | grep -vi 'coreos')"
        for P in $PACKAGES; do
            rlRpmDownload $P
        done
        
        # Stop auditd service and disable it.
        rlServiceStop "auditd"
        rlRun "systemctl disable auditd" 0

    rlPhaseEnd

    rlPhaseStartTest

        # Sanity check.
        status="$(systemctl is-enabled auditd) / $(systemctl is-active auditd)"
        rlAssertEquals "Check that auditd is inactive and disabled" \
            "disabled / inactive" "$status"

        # Trigger.
        service auditd status
        rlRun "rpm --reinstall *.rpm" 0 "Update audit package"
        service auditd status

        # Verify.
        status="$(systemctl is-enabled auditd) / $(systemctl is-active auditd)"
        rlAssertEquals "Check that auditd remains inactive and disabled" \
            "disabled / inactive" "$status"

    rlPhaseEnd

    rlPhaseStartCleanup

        rlRun "popd"
        rlRun "rm -r $TmpDir" 0 "Removing tmp directory"
        rlServiceRestore "auditd"
        rlRun "systemctl enable auditd" 0

    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
