#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1738634-audit-add-kernel-boot-option-to-override-default
#   Description: Test for BZ#1738634 (audit add kernel boot option to override default)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2019 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

case $(uname -m) in
    "x86_64" | "aarch64" | "ppc64le" | "ppc64")
        bootloader="grub2"
        ;;
    "s390x")
        bootloader="zipl"
        ;;
esac

kernel=$(grubby --default-kernel)

rlJournalStart

    # Initial setup.
    if ! [ -f "/var/tmp/bz1738634-reboot-setup" ]; then
        rlPhaseStartSetup "Initial Setup"
            rlAssertRpm $PACKAGE
            rlRun "mkdir /var/tmp/rules.d-backup/" 0
            rlRun "cp -r /etc/audit/rules.d/* /var/tmp/rules.d-backup/" 0

            # Disable setting of audit backlog limit by auditd.
            rlRun "find /etc/audit/rules.d/ -name '*.rules' -exec sed -i 's/-b/# -b/g' {} \;" 0
            
            # Get current value.
            bl=$(auditctl -s | grep backlog_limit | cut -d ' ' -f 2)
            rlLog "backlog_limit is set to $bl"
            
            # Set new value to be two times bigger.
            bl_new=$[$bl*2]
            rlLog "backlog_limit will be set to $bl_new"

            # Update kernel arguments and prepare for reboot.
            rlRun "grubby --args audit_backlog_limit=${bl_new} --update-kernel=${kernel} --${bootloader}" 0
            test "$(uname -m)" == "s390x" && rlRun "zipl" 0
            echo "$bl_new" > /var/tmp/bz1738634-reboot-setup
        rlPhaseEnd

        rhts-reboot
    fi
    
    # Test.
    if ! [ -f "/var/tmp/bz1738634-reboot-cleanup" ]; then
        rlPhaseStartTest "Test"
            # Get expected value.
            bl_exp=$(cat /var/tmp/bz1738634-reboot-setup)

            # Get current value.
            bl=$(auditctl -s | grep backlog_limit | cut -d ' ' -f 2)
            
            # Compare and initiate clean-up.
            rlAssertEquals "backlog_limit should be set to $bl_exp" $bl_exp $bl            
            touch /var/tmp/bz1738634-reboot-cleanup
        rlPhaseEnd

        rhts-reboot
    fi

    if ! [ -f "/var/tmp/bz1738634-reboot-initial-cleanup" ]; then
        rlPhaseStartCleanup "Initial clean-up"
            # Restore kernel arguments.
            rlRun "grubby --remove-args audit_backlog_limit --update-kernel=${kernel} --${bootloader}" 0

            # Restore audit rules.
            rlRun "cp /var/tmp/rules.d-backup/* /etc/audit/rules.d/" 0
            rlRun "rm -rf /var/tmp/rules.d-backup/" 0
            rlRun "restorecon -vvRF /etc/audit/rules.d/" 0

            # Initiate final clean-up.
            touch /var/tmp/bz1738634-reboot-initial-cleanup
        rlPhaseEnd
    
        rhts-reboot
    fi

    rlPhaseStartCleanup "Final clean-up"
        rlRun "rm -f /var/tmp/bz1738634-*" 0
    rlPhaseEnd
    
rlJournalPrintText
rlJournalEnd
