#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz495711-problem-loading-some-rules
#   Description: Some audit rules will not load even though they are correct
#   Author: Eduard Benes <ebenes@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2010 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

BASE_ARCH=b32
SYSCALL=pread
[ ` uname -i | grep 64` ] && BASE_ARCH=b64
[ ` uname -i | grep -e i386 -e ia64` ] && SYSCALL=pread64
ARCH=`uname -i`

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlRun "uname -a"
        rlServiceStart auditd
    rlPhaseEnd
    rlPhaseStartTest
        # Test the actual bug / report, 
        rlRun "auditctl -D" 
        rlRun "auditctl -a exit,always -F arch=$BASE_ARCH -S $SYSCALL -F 'ppid=1'"
        rlRun "auditctl -l | grep ppid=1" 0 "Make sure the rule is there"
        rlRun "auditctl -d exit,always -F arch=$BASE_ARCH -S $SYSCALL -F 'ppid=1'"
        # Do some additional exercies for all available syscalls
        rlLog "Going to exercise all syscalls using their mapping names"
        for SYSCALL in `ausyscall --dump | grep -v Using | cut -f2` ; do
            # TODO Some syscalls are causing issues on ... systems
            if [ $ARCH == 'ia64' ] ; then
                [ $SYSCALL == 'pread' ] && continue
            elif [ $ARCH == 's390x' ] ; then
                [ $SYSCALL == 'select' ] && continue
                [ $SYSCALL == 'newfstatat' ] && continue
            fi
            rlRun "auditctl -a exit,always -F  arch=$BASE_ARCH -S $SYSCALL"
	    if rlIsRHEL "<6.5"; then
		rlRun "auditctl -l | grep syscall=$SYSCALL"
	    else
		rlRun "auditctl -l | grep \"\-S $SYSCALL\""
	    fi
            rlRun "auditctl -d exit,always -F  arch=$BASE_ARCH -S $SYSCALL"
        done
        rlLog "Going to exercise all syscalls using their mapping numbers"
        for SYSCALL in `ausyscall --dump | grep -v Using | cut -f1` ; do
            # TODO This are two calls causing issues on s390x systems
            if [ $ARCH == 's390x' ] ; then
                [ $SYSCALL == "142" ] && continue
                [ $SYSCALL == "293" ] && continue
            fi
            rlRun "auditctl -a exit,always -F  arch=$BASE_ARCH -S $SYSCALL"
            rlRun "auditctl -d exit,always -F  arch=$BASE_ARCH -S $SYSCALL"
        done
    rlPhaseEnd

    rlPhaseStartCleanup
        rlRun "auditctl -D"
        rlRun "service auditd restart"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
