#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz554555-audit-does-not-work-for-filtering-a-large-inode
#   Description: Test for bz554555 (audit does not work for filtering a large inode)
#   Author: Ludek Dolihal <dollson@localhost.localdomain>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2010 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlServiceStart "auditd"
        rlRun "cp /var/log/audit/audit.log /var/log/audit/audit.log.old" 
        OLD_PAGER="$PAGER"
        rlRun "export PAGER=''" 0
    rlPhaseEnd

    rlPhaseStartTest
        rlRun "service auditd status || service auditd start && sleep 5" 0
        rlRun "auditctl -a exit,always -S all -F inode=`ls -i /proc/cpuinfo | cut -f 1 -d ' '`" 0 "add audit rule"
        rlRun "ls /proc/cpuinfo" 0 "listing the cpuinfo"
        rlRun "diff /var/log/audit/audit.log /var/log/audit/audit.log.old" 1 "1 means difference" 
    rlPhaseEnd

    rlPhaseStartCleanup
        rlRun "rm -r /var/log/audit/audit.log.old" 0 "remove old copy of audit.log"
        rlRun "auditctl -D" 0
        rlServiceRestore "auditd"
        rlRun "export PAGER=$OLD_PAGER" 0
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
