#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   test.sh of /CoreOS/audit/Regression/default-systemd-service-blocks-watch-rules-in-some-dirs
#   Description: Test watch rules in a couple of directories, that may be
#   affected by auditd systemd service default settings.
#   Author: Sergio Correia <scorreia@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2022 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm "${PACKAGE}"
        rlFileBackup --clean "/etc/audit/rules.d/"

        rlServiceStop "auditd" && sleep 5
        rlRun "service auditd start && sleep 5" 0
    rlPhaseEnd

    rlPhaseStartTest "Testing rules added directly with auditctl"
        rlRun "auditctl -D"
        rlRun "auditctl -l"

        rlLogInfo "Adding rules"

        _ts=$(date "+%T")

        rlRun "auditctl -w /root/testfile -k test1"
        rlRun "auditctl -w /home/testfile -k test2"
        rlRun "auditctl -l"

        rlRun "echo 1 > /root/testfile"
        rlRun "echo 2 > /home/testfile"

        _te=$(date "+%T")

        rlRun "ausearch -ts ${_ts} -te ${_te}"
        rlRun -s "ausearch -ts ${_ts} -te ${_te} --format text"

        rlAssertGrep "successfully opened-file /root/testfile" $rlRun_LOG
        rlAssertGrep "successfully opened-file /home/testfile" $rlRun_LOG
    rlPhaseEnd

    rlPhaseStartTest "Testing rules added by auditd upon starting"
        rlRun "auditctl -D"
        rlRun "auditctl -l"

        rlServiceStop "auditd" && sleep 5

        rlLogInfo "Preparing rules file"
        rlRun "rm -f /etc/audit/rules.d/*"

        RULES="/etc/audit/rules.d/test.rules"

        rlRun "echo '-w /root/testfile -k test1' > ${RULES}"
        rlRun "echo '-w /home/testfile -k test2' >> ${RULES}"

        _ts=$(date "+%T")

        rlRun "service auditd start && sleep 5" 0
        rlRun "auditctl -l"

        rlRun "echo 1 > /root/testfile"
        rlRun "echo 2 > /home/testfile"

        _te=$(date "+%T")

        rlRun "ausearch -ts ${_ts} -te ${_te}"
        rlRun -s "ausearch -ts ${_ts} -te ${_te} --format text"

        rlAssertGrep "successfully opened-file /root/testfile" $rlRun_LOG
        rlAssertGrep "successfully opened-file /home/testfile" $rlRun_LOG
    rlPhaseEnd

    rlPhaseStartCleanup
        rlFileRestore
        rlServiceRestore "auditd"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
