 #!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/audisp-au-remote-plugin
#   Description: Sanity test of audisp audit-remote plugin
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

# Since RHEL8 there is no seprate dispatcher.
if rlIsRHEL "<8"; then
    DISPATCHER_DIR="/etc/audisp"
else
    DISPATCHER_DIR="/etc/audit"
fi

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "$PACKAGE"

        # Imports.
        rlRun "rlImport distribution/sync" || rlDie
        rlRun "rlImport krb5/basic" || rlDie
        rlRun "rlImport audit/testing" || rlDie

        # Backup.
        rlFileBackup "/etc/hosts"
        rlFileBackup "$auditDaemonConf"
        rlFileBackup "$auditLog"
        rlFileBackup "$auditDispatcherConf"
        rlFileBackup "$auditRemotePluginConf"

        # Setup.
        rlServiceStop "auditd" && sleep 3
        rlRun "auditConfig 'flush' 'INCREMENTAL'" 0
        rlRun "auditConfig 'freq' '1'" 0
        syncRun "SERVER" "auditConfig 'tcp_listen_port' '60'" 0
        syncRun "CLIENT" "auditConfig 'active' 'yes' $auditRemotePluginConf" 0
        syncRun "CLIENT" "auditConfig 'remote_server' '$syncSERVER' $auditDispatcherConf" 0
        syncRun "CLIENT" "auditConfig 'port' '60' $auditDispatcherConf" 0

    rlPhaseEnd

    rlPhaseStartTest "Sanity (IPv$V)"

        # Setup.
        syncRun "CLIENT" "echo \"$syncSERVER server\" >/etc/hosts" 0

        mark=$(date "+%T")
        syncRun "CLIENT" "syncExp SANITY_SERVER_READY" 0
        rlRun "service auditd start && sleep 5" 0
        rlRun "service auditd status" 0
        syncRun "SERVER" "syncSet SANITY_SERVER_READY" 0

        # Trigger.
        if syncIsClient; then
            rlRun "auditctl -m \"REMOTE-TEST-IPv$V\"" 0
            rlRun "syncSet SANITY_CLIENT_TEST_DONE $mark" 0
        fi

        # Verify.
        if syncIsClient; then
            rlRun -s "systemctl status auditd -n 1000 -l" 0
            rlAssertGrep "Connected to $syncSERVER" $rlRun_LOG
            rlAssertNotGrep "Unknown keyword" $rlRun_LOG
            rlAssertNotGrep "terminated unexpectedly" $rlRun_LOG
            rlRun "syncExp SANITY_SERVER_TEST_DONE" 0

            # BZ#1614833.
            rlRun "kill $(pidof audisp-remote)" 0
            rlRun "service auditd reload" 0
            audispd_pid=$(pidof audispd)
            rlRun "[ $(journalctl -xe | grep 'audispd\[$audispd_pid\]: Starting reconfigure' | wc -l) -le 1 ]" 0 \
                    "Check that there is not 'reconfigure' loop"
            rlRun "syncSet BZ1614833_CLIENT_TEST_DONE" 0

        elif syncIsServer; then
            rlRun "client_mark=$(syncExp SANITY_CLIENT_TEST_DONE)" 0
            rlRun -s "ausearch -ts $client_mark -m USER" 0
            rlAssertGrep "REMOTE-TEST-IPv$V" $rlRun_LOG
            rlRun -s "ss -putna | grep ':60'" 0

            dst=$(grep ESTAB $rlRun_LOG | awk '{print $5}')
            src=$(grep ESTAB $rlRun_LOG | awk '{print $6}')

            pattern_dst="[\[]?::ffff:${syncSERVER}[\]]?:60"
            pattern_src="[\[]?::ffff:${syncCLIENT}[\]]?:[0-9]+"

            rlRun "[[ $dst =~ $pattern_dst ]]" 0
            rlRun "[[ $src =~ $pattern_src ]]"

            # BZ#1534748.
            msg=$(mktemp)
            rlRun "auditWaitForEvent $mark DAEMON_ACCEPT 'audit' 30 $msg" 0
            rlAssertGrep "addr=::ffff:${syncCLIENT}" $msg

            rlRun "syncSet SANITY_SERVER_TEST_DONE" 0
            rlRun "syncExp BZ1614833_CLIENT_TEST_DONE" 0
        fi

        # Clean-up.
        rlRun "service auditd stop && sleep 5" 0
    rlPhaseEnd

    # Skip on RHEL-7 (BZ#1752577).
    if ! rlIsRHEL "<8"; then
    rlPhaseStartTest "Using Kerberos"

        # Setup.
        if rlIsRHEL "<8"; then
            syncRun "SERVER" "auditConfig 'enable_krb5' 'yes'" 0
        else
            syncRun "SERVER" "auditConfig 'transport' 'KRB5'" 0
        fi
        syncRun "SERVER" "auditConfig 'krb5_principal' 'auditd'" 0
        syncRun "SERVER" "auditConfig 'krb5_key_file' '/etc/auditd.keytab'" 0

        if rlIsRHEL "<8"; then
            syncRun "CLIENT" "auditConfig 'enable_krb5' 'yes' $auditDispatcherConf" 0
        else
            syncRun "CLIENT" "auditConfig 'transport' 'KRB5' $auditDispatcherConf" 0
        fi
        if syncIsClient; then            
            if [[ $SERVERS =~ ".com" ]]; then
                SERVER_FDQN="$SERVERS" 
            else
                SERVER_FDQN="$(nslookup $SERVERS | grep name | sed 's/^.*name = \(.*\).$/\1/')"
            fi 
        fi
        syncRun "CLIENT" "auditConfig 'krb5_principal' 'auditd/${SERVER_FDQN}' $auditDispatcherConf" 0
        syncRun "CLIENT" "auditConfig 'krb5_client_name' 'auditd' $auditDispatcherConf" 0
        syncRun "CLIENT" "auditConfig 'krb5_key_file' '/etc/auditd.keytab' $auditDispatcherConf" 0
        syncRun "CLIENT" "auditConfig 'remote_server' '$SERVER_FDQN' $auditDispatcherConf" 0
        krb5Services="auditd"
        syncRun "SERVER" "krb5Setup" 0
        syncRun "SERVER" "syncSet KRB5_SETUP_DONE" 0
        syncRun "CLIENT" "syncExp KRB5_SETUP_DONE" 0
        syncRun "CLIENT" "krb5ClientSetup $SERVERS" 0

        rlRun "chmod 0400 /etc/auditd.keytab" 0
        rlRun "restorecon -vvFR /etc/ /var/"

        # Trigger.
        mark=$(date "+%T")
        syncRun "CLIENT" "syncExp KERBEROS_SERVER_READY" 0
        rlRun "service auditd start && sleep 5" 0
        rlRun "PAGER='' service auditd status" 0
        syncRun "SERVER" "syncSet KERBEROS_SERVER_READY" 0

        # Trigger.
        if syncIsClient; then
            rlRun "auditctl -m 'KERBEROS-REMOTE-TEST'" 0
            rlRun "syncSet KERBEROS_CLIENT_TEST_DONE $mark" 0
        fi

        # Verify.
        if syncIsClient; then
            rlRun -s "systemctl status auditd -n 1000 -l" 0
            rlAssertGrep "Connected to $SERVER_FDQN" $rlRun_LOG
            rlAssertGrep "kerberos principal: auditd/$(hostname)@TEST.REDHAT.COM" $rlRun_LOG
            rlRun "syncExp KERBEROS_SERVER_TEST_DONE" 0
        elif syncIsServer; then
            rlRun "client_mark=$(syncExp KERBEROS_CLIENT_TEST_DONE)" 0
            rlRun "auditWaitForEvent $client_mark USER 'KERBEROS-REMOTE-TEST' 30" 0
            rlRun "syncSet KERBEROS_SERVER_TEST_DONE" 0
        fi

        # Clean-up.
        rlRun "service auditd stop && sleep 5" 0
        syncRun "SERVER" "krb5Cleanup" 0
        syncRun "CLIENT" "krb5ClientCleanup" 0

    rlPhaseEnd
    fi

    rlPhaseStartCleanup

        rlFileRestore
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
