#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/audisp-syslog-plugin
#   Description: Sanity of syslog audisp plugin
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh

PACKAGE="audit"

F=( "LOCAL0"       \
    "LOCAL1"       \
    "LOCAL2"       \
    "LOCAL3"       \
    "LOCAL4"       \
    "LOCAL5"       \
    "LOCAL6"       \
    "LOCAL7"       )

# On RHEL7 the list of facilities is extended (BZ#1241634).
rlIsRHEL ">=7" || rlIsFedora && \
    F=( ${F[@]} "USER"     \
                "DAEMON"   \
                "SYSLOG"   \
                "AUTH"     \
                "AUTHPRIV" )

# Since RHEL8 there is no separate dispatcher.
if rlIsRHEL "<8"; then
    DISPATCHER_DIR="/etc/audisp"
else
    DISPATCHER_DIR="/etc/audit"
fi

rlJournalStart

    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlAssertRpm "rsyslog"
        rlRun "rlImport audit/testing" || rlDie

        rlFileBackup "${DISPATCHER_DIR}/plugins.d/syslog.conf"
        rlFileBackup "/etc/rsyslog.conf"
        rlFileBackup "/var/log/audit/audit.log"
        rlFileBackup --clean "/etc/audit"

        if ! rlIsRHEL "<9"; then
            rlRun "systemctl status systemd-journald-audit.socket > socket_state_initial" 0-255
            rlFileSubmit "socket_state_initial"
        fi

        rlServiceStop "auditd" && sleep 3
        rlServiceStop "rsyslog" && sleep 3

        rlRun "mkdir /var/log/a-s-p-test/" 0
        for I in `seq 0 $[${#F[@]}-1]`; do
            rlRun "echo \"${F[$I]}.* /var/log/a-s-p-test/${F[$I]}\" >> /etc/rsyslog.d/test.conf" 0
            rlRun "touch /var/log/a-s-p-test/${F[$I]}" 0
            rlRun "chmod a+w /var/log/a-s-p-test/${F[$I]}" 0
        done

        rlRun "auditConfig 'log_format' 'RAW'" 0
        rlRun "auditConfig 'flush' 'INCREMENTAL'" 0
        rlRun "auditConfig 'freq' '1'" 0
        rlRun "auditConfig 'active' 'yes' $auditSyslogPluginConf" 0

        rlRun "restorecon -vvFR /var/log /etc/rsyslog.d" 0
        rlRun "service rsyslog start && sleep 5" 0
        rlRun "service auditd start && sleep 5" 0
        rlRun "echo '' > /var/log/audit/audit.log" 0

        if rlIsRHEL "9"; then
            # Since RHEL-9/Fedora we must stop journald picking up audit events
            # because otherwise there will be a copy of syslog message in AUTH.
            if systemctl -q is-active systemctl status systemd-journald-audit.socket; then
                systemd_journald_audit_active=1
                rlRun "systemctl mask systemd-journald-audit.socket" 0
                rlRun "systemctl stop systemd-journald-audit.socket" 0
                rlRun "service systemd-journald restart" 0
                rlRun "systemctl -q is-active systemctl status systemd-journald-audit.socket" 3 "Make sure systemd-journald-audit.socket is inactive"
            fi
        fi
    rlPhaseEnd

    if rlIsRHEL ">=8.2"; then
        rlPhaseStartTest "Interpretation"
            rlRun "auditConfig 'args' 'LOG_LOCAL0 interpret' $auditSyslogPluginConf" 0
            rlRun "service auditd restart && sleep 5" 0
            mark=$(date "+%T")
            rlRun "auditctl -m \"interpretation_test\"" 0
            rlRun -s "auditWaitForEvent $mark 'USER' 'interpretation_test' 30 '' -i" 0
            rlRun "sed -i \"s/\([0-9:]\+\)\.[0-9:]\+/\1/\" $rlRun_LOG" 0
            rlRun "sed -i \"s|\([0-9][0-9]/[0-9][0-9]/\)[0-9][0-9]\([0-9][0-9]\)|\1\2|\" $rlRun_LOG" 0
            exp_event=$rlRun_LOG
            rlRun -s "auditWaitForSyslogEvent 'interpretation_test' 60 /var/log/a-s-p-test/LOCAL0" 0
            rlRun "sed -i \"s/.*audispd\[[0-9]\+\]: //\" $rlRun_LOG" 0
            rlRun "sed -i \"s/text=/msg='text=/\" $rlRun_LOG" 0
            rlRun "sed -i \"s/\(res=.*\)/\1'/\" $rlRun_LOG" 0
            act_event=$rlRun_LOG
            rlRun "diff -Z $exp_event $act_event" 0
        rlPhaseEnd
    fi

    rlPhaseStartTest
        for I in `seq 0 $[${#F[@]}-1]`; do
            rlRun "auditConfig 'args' 'LOG_${F[$I]}' $auditSyslogPluginConf" 0
            rlRun "service auditd restart && sleep 5" 0
            rlRun "auditctl -m \"${F[$I]} test\" && sleep 5" 0
            rlRun "auditWaitForSyslogEvent '${F[$I]} test' 60 /var/log/a-s-p-test/${F[$I]}" 0
            for log in $(ls -1 /var/log/a-s-p-test/* | grep -v ${F[$I]} | tr '\n' ' '); do
                rlRun "auditWaitForSyslogEvent '${F[$I]} test' 1 $log" 1
            done
        done
    rlPhaseEnd

    rlPhaseStartCleanup

        rlRun "sleep 5 && service rsyslog stop && sleep 5" 0-255
        rlRun "rm -rf /var/log/a-s-p-test /etc/rsyslog.d/test.conf" 0

        rlFileRestore

        if [ "$systemd_journald_audit_active" == "1" ]; then
            rlRun "systemctl unmask systemd-journald-audit.socket" 0
            rlRun "service systemd-journald restart" 0
            rlRun "systemctl -q is-active systemctl status systemd-journald-audit.socket" 0 "Make sure systemd-journald-audit.socket is active"
        fi

        rlServiceRestore "rsyslog"
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
