#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/audit-log-rotate-test
#   Description: Test for max_log_file_action configuration parameter.
#   Author: Patrik Kis <pkis@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2014 Red Hat, Inc.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

function change_auditd.conf {
    # change_auditd.conf <parameter> <value>
    [[ $# -ne 2 ]] && return 1
    local PARAM=$1
    local VALUE=$2
    sed -i "s/^\s*$PARAM\s\+=.*/$PARAM = $VALUE/" /etc/audit/auditd.conf
    echo "+---------- /etc/audit/auditd.conf ----------+"
    grep -v ^\# /etc/audit/auditd.conf
    echo "+--------------------------------------------+"
    grep "^$PARAM = $VALUE$" /etc/audit/auditd.conf
    return $?
}

function fill_audit.log {
    # fill_audit.log [<size in MB>]
    local SIZE=${1:-1} # default: 1MB
    local ORIG_LOG_SIZE=`du -b /var/log/audit/ |awk '{print $1}'`
    echo "fill_audit.log: SIZE=$SIZE"
    echo "fill_audit.log: ORIG_LOG_SIZE=$ORIG_LOG_SIZE"
    echo "fill_audit.log: start filling audit.log with $SIZE MB"
    for i in `seq 1 $(($SIZE*4000))`; do
        auditctl -m _test_message__test_message__test_message__test_message_
        sleep 0.0004 # Give some time to the daemon to write the logs
    done
    sleep 5
    echo "fill_audit.log: expected size = $(($SIZE*1000000))"
    local NEW_LOG_SIZE=`du -b /var/log/audit/ |awk '{print $1}'`
    local NUM_LOGS=`grep '^\s*num_logs' /etc/audit/auditd.conf |sed 's/^\s*num_logs\s*=\s*//'`
    local MAX_LOG_FILE=`grep '^\s*max_log_file\s*=' /etc/audit/auditd.conf |sed 's/^\s*max_log_file\s*=\s*//'`
    echo "fill_audit.log: NEW_LOG_SIZE=$NEW_LOG_SIZE"
    echo "fill_audit.log: NUM_LOGS=$NUM_LOGS"
    echo "fill_audit.log: MAX_LOG_FILE=$MAX_LOG_FILE"
    # The total log size is either what was sent 
    if [[ $((NEW_LOG_SIZE-ORIG_LOG_SIZE)) -gt $(($SIZE*1000000)) ]]; then
        return 0
    # or what is set in auditd.conf as maximum
    elif [[ $NEW_LOG_SIZE -lt $((NUM_LOGS*MAX_LOG_FILE*1000000)) ]]; then
        return 0
    else
        return 1
    fi
}

function clean_audit.log {
    # cleans audit log and restart the daemon
    service auditd stop
    rm -f /var/log/audit/*
    sleep 1
    service auditd start
    echo "clean_audit.log: /var/log/audit/audit.log cleaned"
    service auditd status
    return $?
}

function log_record_start {
    # start recording the given log file to the local directory
    local LOG_FILE=${1:-/var/log/messages}
    local LOG_NAME=`basename $LOG_FILE`
    echo "log_record_start: LOG_FILE=$LOG_FILE"
    echo "log_record_start: LOG_NAME=$LOG_NAME"
    tail -n0 -f $LOG_FILE &> $LOG_NAME.log.record &
    echo $! > $LOG_NAME.record.pid
    echo "log_record_start: PID = `cat $LOG_NAME.record.pid`"
    sleep 3
    kill -0 `cat $LOG_NAME.record.pid` && return 0 || return 1
}

function log_record_stop {
    # stop recording the given log file to the local directory
    local LOG_FILE=${1:-/var/log/messages}
    local LOG_NAME=`basename $LOG_FILE`
    echo "log_record_start: LOG_FILE=$LOG_FILE"
    echo "log_record_start: LOG_NAME=$LOG_NAME"
    if [[ ! -f $LOG_NAME.record.pid ]]; then
        echo "log_record_start: No pid file found for $LOG_FILE"
        ls -l $LOG_NAME.record.pid
        return 1
    fi
    local LOG_PID=`cat $LOG_NAME.record.pid`
    echo "log_record_start: LOG_PID=$LOG_PID"
    rm -f $LOG_NAME.record.pid
    kill $LOG_PID
    sleep 5
    echo "+------------ messages.log.record ------------+"
    cat $LOG_NAME.log.record
    echo "+---------------------------------------------+"
    kill -0 $LOG_PID && return 1 || return 0
}

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlRun "TmpDir=\$(mktemp -d)" 0 "Creating tmp directory"
        rlRun "pushd $TmpDir"
        rlFileBackup /etc/audit/auditd.conf
        rlRun "change_auditd.conf max_log_file 1"
        rlRun "change_auditd.conf num_logs 3"
        rlRun "change_auditd.conf flush INCREMENTAL"
        rlRun "change_auditd.conf freq 1"
        rlRun "service auditd stop"
        rlFileBackup --clean /var/log/audit/
        rlRun "rm -f /var/log/audit/*"
        rlRun "service auditd start"
        rlServiceStart "rsyslog"
        OLD_PAGER="$PAGER"
        rlRun "export PAGER=''" 0
    rlPhaseEnd

    rlPhaseStartTest "Test max_log_file_action=ignore"
        # Change the config and clean the log
        rlRun "change_auditd.conf max_log_file_action ignore"
        rlRun "clean_audit.log"
        # Fill the log and check there is only one log file
        rlRun "fill_audit.log 1"
        rlAssertExists /var/log/audit/audit.log
        rlAssertNotExists /var/log/audit/audit.log.*
        ls -l /var/log/audit/
        # Add even more data to log file: still only one log file
        rlRun "fill_audit.log 3"
        rlAssertExists /var/log/audit/audit.log
        rlAssertNotExists /var/log/audit/audit.log.*
        ls -l /var/log/audit/
        rlRun "ausearch -m avc 2>&1 |grep 'no matches'"
    rlPhaseEnd

    rlPhaseStartTest "Test max_log_file_action=syslog"
        # Change the config and clean the log
        rlRun "change_auditd.conf max_log_file_action syslog"
        rlRun "clean_audit.log"
        rlRun "log_record_start"
        # Fill the log and check there is only one log file
        rlRun "fill_audit.log 1"
        rlAssertExists /var/log/audit/audit.log
        rlAssertNotExists /var/log/audit/audit.log.*
        ls -l /var/log/audit/
        # Check there is warning
        rlRun "log_record_stop"
        rlAssertGrep "Audit daemon log file is larger than max size" messages.log.record
        if [[ $? -ne 0 ]]; then
            echo "+---------- tail -100 /var/log/messages ----------+"
            tail -100 /var/log/messages
            echo "+-------------------------------------------------+"
        fi
        rlRun "ausearch -m avc 2>&1 |grep 'no matches'"
    rlPhaseEnd

    rlPhaseStartTest "Test max_log_file_action=suspend"
        # Change the config and clean the log
        rlRun "change_auditd.conf max_log_file_action suspend"
        rlRun "clean_audit.log"
        rlRun "log_record_start"
        # Fill the log and check there is only one log file
        rlRun "fill_audit.log 1"
        rlAssertExists /var/log/audit/audit.log
        rlAssertNotExists /var/log/audit/audit.log.*
        ls -l /var/log/audit/
        # Check there is warning and no new records appear
        rlRun "log_record_stop"
        rlAssertGrep "Audit daemon is suspending logging due to logfile size" messages.log.record
        if [[ $? -ne 0 ]]; then
            echo "+---------- tail -100 /var/log/messages ----------+"
            tail -100 /var/log/messages
            echo "+-------------------------------------------------+"
        fi
        rlRun "auditctl -m should_not_be_logged"
        sleep 1
        rlAssertNotGrep "should_not_be_logged" /var/log/audit/audit.log
        if [[ $? -ne 0 ]]; then
            echo "+---------- tail -100 /var/log/audit/audit.log ----------+"
            tail -100 /var/log/audit/audit.log
            echo "+--------------------------------------------------------+"
        fi
        rlRun "service auditd status"
        rlRun "ausearch -m avc 2>&1 |grep 'no matches'"
    rlPhaseEnd

    rlPhaseStartTest "Test max_log_file_action=rotate"
        # Change the config and clean the log
        rlRun "change_auditd.conf max_log_file_action rotate"
        rlRun "clean_audit.log"
        rlRun "log_record_start"
        # Fill the log and check there are 2 log files
        rlRun "fill_audit.log 1"
        rlAssertExists /var/log/audit/audit.log
        rlAssertExists /var/log/audit/audit.log.1
        rlAssertNotExists /var/log/audit/audit.log.2
        ls -l /var/log/audit/
        # Check there is warning and no new records appear
        rlRun "log_record_stop"
        rlAssertGrep "Audit daemon rotating log files" messages.log.record
        if [[ $? -ne 0 ]]; then
            echo "+---------- tail -100 /var/log/messages ----------+"
            tail -100 /var/log/messages
            echo "+-------------------------------------------------+"
        fi
        # Add even more data to log file: up to num_logs=3 log files should be created
        rlRun "fill_audit.log 1"
        rlAssertExists /var/log/audit/audit.log
        rlAssertExists /var/log/audit/audit.log.1
        rlAssertExists /var/log/audit/audit.log.2
        rlAssertNotExists /var/log/audit/audit.log.3
        ls -l /var/log/audit/
        # Add even more data to log file: up to num_logs=3 log files should be created
        rlRun "fill_audit.log 3"
        rlAssertExists /var/log/audit/audit.log
        rlAssertExists /var/log/audit/audit.log.1
        rlAssertExists /var/log/audit/audit.log.2
        rlAssertNotExists /var/log/audit/audit.log.3
        ls -l /var/log/audit/
        rlRun "ausearch -m avc 2>&1 |grep 'no matches'"
    rlPhaseEnd

    rlPhaseStartTest "Test max_log_file_action=keep_logs"
        # Change the config and clean the log
        rlRun "change_auditd.conf max_log_file_action keep_logs"
        rlRun "clean_audit.log"
        rlRun "log_record_start"
        # Fill the log and check there are 2 log files
        rlRun "fill_audit.log 1"
        rlAssertExists /var/log/audit/audit.log
        rlAssertExists /var/log/audit/audit.log.1
        rlAssertNotExists /var/log/audit/audit.log.2
        ls -l /var/log/audit/
        # Check there is warning and no new records appear
        rlRun "log_record_stop"
        rlAssertGrep "Audit daemon rotating log files" messages.log.record
        if [[ $? -ne 0 ]]; then
            echo "+---------- tail -100 /var/log/messages ----------+"
            tail -100 /var/log/messages
            echo "+-------------------------------------------------+"
        fi
        # Add even more data to log file: the number should not be limited
        rlRun "fill_audit.log 1"
        rlAssertExists /var/log/audit/audit.log
        rlAssertExists /var/log/audit/audit.log.1
        rlAssertExists /var/log/audit/audit.log.2
        rlAssertNotExists /var/log/audit/audit.log.3
        ls -l /var/log/audit/
        # Add even more data to log file: the number should not be limited
        rlRun "fill_audit.log 3"
        rlAssertExists /var/log/audit/audit.log
        rlAssertExists /var/log/audit/audit.log.1
        rlAssertExists /var/log/audit/audit.log.2
        rlAssertExists /var/log/audit/audit.log.3
        ls -l /var/log/audit/
        rlRun "ausearch -m avc 2>&1 |grep 'no matches'"
    rlPhaseEnd

    for num_logs in 3 1 0; do
    rlPhaseStartTest "Test max_log_file_action=keep_logs and num_logs=$num_logs"
        # Change the config and clean the log
        rlRun "change_auditd.conf num_logs $num_logs"
        rlRun "clean_audit.log"
        rlRun "log_record_start"
        # Fill the log and check there are 2 log files
        rlRun "fill_audit.log 1"
        rlAssertExists /var/log/audit/audit.log
        rlAssertExists /var/log/audit/audit.log.1
        rlAssertNotExists /var/log/audit/audit.log.2
        ls -l /var/log/audit/
        # Check there is warning and no new records appear
        rlRun "log_record_stop"
        rlAssertGrep "Audit daemon rotating log files" messages.log.record
        if [[ $? -ne 0 ]]; then
            echo "+---------- tail -100 /var/log/messages ----------+"
            tail -100 /var/log/messages
            echo "+-------------------------------------------------+"
        fi
        # Add even more data to log file: the number should not be limited
        rlRun "fill_audit.log 1"
        rlAssertExists /var/log/audit/audit.log
        rlAssertExists /var/log/audit/audit.log.1
        rlAssertExists /var/log/audit/audit.log.2
        rlAssertNotExists /var/log/audit/audit.log.3
        ls -l /var/log/audit/
        # Add even more data to log file: the number should not be limited
        rlRun "fill_audit.log 3"
        rlAssertExists /var/log/audit/audit.log
        rlAssertExists /var/log/audit/audit.log.1
        rlAssertExists /var/log/audit/audit.log.2
        rlAssertExists /var/log/audit/audit.log.3
        ls -l /var/log/audit/
        rlRun "ausearch -m avc 2>&1 |grep 'no matches'"
    rlPhaseEnd
    done

    rlPhaseStartCleanup
        rlRun "service auditd stop"
        rlFileRestore
        rlRun "service auditd start"
        rlServiceRestore "rsyslog"
        rlRun "popd"
        rlRun "rm -r $TmpDir" 0 "Removing tmp directory"
        rlRun "export PAGER=$OLD_PAGER" 0
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
