#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/audit-rules-built-from-directory
#   Description: Audit rules are built from directory
#   Author: Patrik Kis <pkis@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2014 Red Hat, Inc.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlRun "TmpDir=\$(mktemp -d)" 0 "Creating tmp directory"
        rlRun "cp bz967238.spec $TmpDir"
        rlRun "pushd $TmpDir"
        rlFileBackup --clean /etc/audit/ /etc/sysconfig/auditd
        rlRun "rpmbuild -bb bz967238.spec &>rpmbuild.log"
        cat rpmbuild.log
        RPM=`awk '/Wrote:.*\/RPMS\// {print $NF}' rpmbuild.log`
        echo RPM=$RPM
        if rlIsRHEL 6; then
            rlRun "sed -i 's/^USE_AUGENRULES.*/USE_AUGENRULES=yes/' /etc/sysconfig/auditd"
            cat /etc/sysconfig/auditd
            rlRun "service auditd restart"
        fi
        tail -n0 -f /var/log/audit/audit.log > audit.log &
        LOG_PID=$!; echo LOG_PID=$LOG_PID; sleep 1
    rlPhaseEnd

    rlPhaseStartTest
        rlRun "rpm -ivh $RPM"
        rlAssertExists /etc/audit/rules.d/bz967238.rules
        rlRun "service auditd restart"
        sleep 5
        rlRun "auditctl -l | grep watch-date"
        rlRun "date"
        rlAssertGrep "watch-date" audit.log
        cat audit.log
        rlRun "ausearch -if audit.log -k watch-date"
        rlRun "rpm -ev bz967238"
        rlAssertNotExists /etc/audit/rules.d/bz967238.rules
    rlPhaseEnd

    rlPhaseStartCleanup
        kill $LOG_PID
        rlFileRestore
        rlRun "service auditd restart"
        rlRun "popd"
        rlRun "rm -r $TmpDir" 0 "Removing tmp directory"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
