#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/audit-rules-smoke
#   Description: Smoke test for audit.rules
#   Author: Eduard Benes <ebenes@redhat.com>
#           Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2010 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rules_file=/etc/audit/rules.d/audit.rules

rlIsRHEL "<7" && rules_file=/etc/audit/audit.rules
VALID_OPEN_SYSCALLS="-S open -S openat"
[[ `arch` == aarch64 ]] && VALID_OPEN_SYSCALLS="-S openat"

function set_valid_rules {
cat > $rules_file  << EORULES
-D
-i
-a always,exit -F arch=b32 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k access
-a always,exit -F arch=b32 $VALID_OPEN_SYSCALLS -F exit=-EPERM -k access
-a always,exit -F arch=b64 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k access
-a always,exit -F arch=b64 $VALID_OPEN_SYSCALLS -F exit=-EPERM -k access
EORULES
}

function set_invalid_rules_A {
cat > $rules_file  << EORULES
-D
-a always,exitX -F arch=b32 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k invalid
-a always,exitX -F arch=b64 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k invalid
-a always,exit -F arch=b32 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k valid
-a always,exit -F arch=b64 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k valid
EORULES
}

function set_invalid_rules_B {
cat > $rules_file  << EORULES
-D
-i
-a always,exitX -F arch=b32 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k invalid
-a always,exitX -F arch=b64 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k invalid
-a always,exit -F arch=b32 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k valid
-a always,exit -F arch=b64 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k valid
EORULES
}

function set_invalid_rules_C {
cat > $rules_file  << EORULES
-D
-c
-a always,exitX -F arch=b32 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k invalid
-a always,exitX -F arch=b64 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k invalid
-a always,exit -F arch=b32 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k valid
-a always,exit -F arch=b64 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k valid
EORULES
}

function set_invalid_rules_D {
cat > $rules_file  << EORULES
-D
-i
-c
-a always,exitX -F arch=b32 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k invalid
-a always,exitX -F arch=b64 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k invalid
-a always,exit -F arch=b32 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k valid
-a always,exit -F arch=b64 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k valid
EORULES
}

function set_invalid_rules_E {
cat > $rules_file  << EORULES
-D
-c
-i
-a always,exitX -F arch=b32 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k invalid
-a always,exitX -F arch=b64 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k invalid
-a always,exit -F arch=b32 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k valid
-a always,exit -F arch=b64 $VALID_OPEN_SYSCALLS -F exit=-EACCES -k valid
EORULES
}

function set_very_long_rule {
# add all available syscalls
ARCH=`arch`
[ "$ARCH" == "x86_64" ] || [ "$ARCH" == "i686" ] && ARCH=i386
[ "$ARCH" == "s390x" ] && ARCH=s390
SYSCALL_ARCH=b32
[ "$ARCH" == "aarch64" ] && SYSCALL_ARCH=b64
[ "$ARCH" == "ppc64le" ] && SYSCALL_ARCH=b64
echo -n "-a never,exit -F arch=$SYSCALL_ARCH " > $rules_file
ausyscall $ARCH --dump | egrep -v '^3|^U' | awk '{ printf "-S %s\n", $2 }' | xargs echo -n >> $rules_file
echo " -k long_rule" >> $rules_file
}

rlJournalStart


    rlPhaseStartSetup
        rlAssertRpm "audit"
        rlServiceStop "auditd" && sleep 3
        rlFileBackup --clean "/etc/audit"
        rlRun "adduser foobar" 0

        rlRun "auditctl -s > initial_state" 0 "Show initial audit status"
        rlFileSubmit "initial_state"
    rlPhaseEnd

    rlPhaseStartTest "General"
        rlRun "set_valid_rules" 0
        rlRun "service auditd start && sleep 3" 0
        rlRun "su - foobar -c 'cat /etc/shadow'" 1

        sleep 2
        rlFileSubmit "/var/log/audit/audit.log" "audit-rules-smoke-audit.log"

        for time_start in recent today yesterday this-week week-ago this-month this-year ; do
            rlRun "ausearch -k access -ts $time_start -te now -x cat" 0
        done
        rlRun "/sbin/auditctl -R /etc/audit/audit.rules" 0
        rlRun "service auditd stop && sleep 3" 0
    rlPhaseEnd

    rlPhaseStartTest "RHEL6.1 updates"
        rlRun "find /usr/share/doc/audit* -name *.rules -exec echo cp {} /etc/audit/ \;" 0
        rlRun "find /etc/audit/ -name *.rules -exec restorecon -Rvvv {} \;" 0
        rlRun "find /etc/audit/ -name *.rules -exec sed -i -n 's/\(-e 2\)/#\1/p' {} \;" 0
       
        for msgtype in VIRT_CONTROL VIRT_RESOURCE VIRT_MACHINE_ID NETFILTER_CFG NETFILTER_PKT MMAP CRYPTO_SESSION SERVICE_START ; do
    	    rlRun "auditctl -a exclude,always -F msgtype=$msgtype" 0
            rlRun "auditctl -l" 0
            rlRun "auditctl -D" 0
        done
    rlPhaseEnd

    rlPhaseStartTest "Options"
        rlLog "Invalid rules, no -i option"
        rlRun "set_invalid_rules_A" 0
        rlRun "service auditd start && sleep 3" 0
        rlRun "/sbin/auditctl -R /etc/audit/audit.rules" 1-255
        rlRun "service auditd status && service auditd stop && sleep 3" 0

        rlLog "Invalid rules, -i option"
        rlRun "set_invalid_rules_B" 0
        rlRun "service auditd start && sleep 3" 0
        rlRun "/sbin/auditctl -R /etc/audit/audit.rules" 0
        rlRun "service auditd status && service auditd stop && sleep 3" 0

    	if rlIsRHEL '>=6'; then
    	    rlLog "Invalid rules, -c option"
    	    rlRun "set_invalid_rules_C" 0
    	    rlRun "service auditd start && sleep 3" 0
    	    rlRun "su - foobar -c 'cat /etc/shadow'" 1
    	    rlRun "ausearch -k valid -ts recent -te now -x cat" 0
    	    rlRun "ausearch -k invalid -ts recent  -te now -x cat" 1-255
    	    rlRun "/sbin/auditctl -R /etc/audit/audit.rules" 1-255
    	    rlRun "service auditd status && service auditd stop && sleep 3" 0

    	    rlLog "Invalid rules, -i and -c options"
    	    rlRun "set_invalid_rules_D" 0
    	    rlRun "service auditd start && sleep 3" 0
    	    rlRun "su - foobar -c 'cat /etc/shadow'" 1
    	    rlRun "ausearch -k valid -ts recent -te now -x cat" 0
    	    rlRun "ausearch -k invalid -ts recent -te now -x cat" 1-255
    	    rlRun "/sbin/auditctl -R /etc/audit/audit.rules" 1-255
    	    rlRun "service auditd status && service auditd stop && sleep 3" 0

    	    rlLog "Invalid rules, -c and -i options"
    	    rlRun "set_invalid_rules_E" 0
    	    rlRun "service auditd start && sleep 3" 0
    	    rlRun "su - foobar -c 'cat /etc/shadow'" 1
    	    rlRun "ausearch -k valid -ts recent -te now -x cat" 0
    	    rlRun "ausearch -k invalid -ts recent -te now -x cat" 1-255
    	    rlRun "/sbin/auditctl -R /etc/audit/audit.rules" 1-255
    	    rlRun "service auditd status && service auditd stop && sleep 3" 0
    	fi
    rlPhaseEnd

  if rlIsRHEL '>=6'; then  # fixed since 6.6
      rlPhaseStartTest "test adding very long rule (BZ#1089713)"
          rlRun "set_very_long_rule" 0
          rlRun "service auditd start && sleep 3" 0
          # count the number of syscalls from $rules_fil
          CONFIGURED=`grep long_rule $rules_file | grep -o -- '-S' | wc -l`
          # count the number of watched syscalls - the output of auditctl -l looks like this
          # -a never,exit -F arch=i386 -S restart_syscall,exit,...,futimesat -F key=long_rule
          auditctl -l | grep long_rule  # just for logging purposes
          WATCHED=`auditctl -l | grep long_rule | egrep -o -- '-S [^ ]*' | cut -d ' ' -f 2 | sed 's/,/ /g' | wc -w`
          rlAssertGreater "There should be at least 200 syscalls" $CONFIGURED 199
          rlAssertEquals "The number of syscalls in $rules_file ($CONFIGURED) should match the number reported by 'auditctl -l' ($WATCHED)" $CONFIGURED $WATCHED
    	  rlRun "service auditd status && service auditd stop && sleep 3" 0
       rlPhaseEnd
  fi

  if rlIsRHEL '>=7'; then  # augenrules is disabled until RHEL-7
      rlPhaseStartTest "Loading only *.rules files (BZ#1414812)"
          rlRun "rm -rf /etc/audit/rules.d/*" 0
          rlRun "echo '-w /tmp/testA -p a -k testA' >/etc/audit/rules.d/test" 0
          rlRun "echo '-w /tmp/testB -p a -k testB' >/etc/audit/rules.d/rules" 0
          rlRun "echo '-w /tmp/testC -p a -k testC' >/etc/audit/rules.d/.rules" 0
          rlRun "echo '-w /tmp/testD -p a -k testD' >/etc/audit/rules.d/rules.test" 0
          rlRun "echo '-w /tmp/testE -p a -k testE' >/etc/audit/rules.d/test.rule" 0
          rlRun "echo '-w /tmp/testF -p a -k testF' >/etc/audit/rules.d/test.rules.test" 0
          rlRun "echo '-w /tmp/testG -p a -k testG' >/etc/audit/rules.d/test.rules." 0
          rlRun "echo '-w /tmp/testH -p a -k testH' >/etc/audit/rules.d/testrules" 0
          rlRun "echo '-w /tmp/testI -p a -k testI' >/etc/audit/rules.d/test.rules" 0
          rlRun "service auditd restart && sleep 5" 0
          rlRun "auditctl -l | grep testA" 1
          rlRun "auditctl -l | grep testB" 1
          rlRun "auditctl -l | grep testC" 1
          rlRun "auditctl -l | grep testD" 1
          rlRun "auditctl -l | grep testE" 1
          rlRun "auditctl -l | grep testF" 1
          rlRun "auditctl -l | grep testG" 1
          rlRun "auditctl -l | grep testH" 1
          rlRun "auditctl -l | grep testI" 0
      rlPhaseEnd
  fi

  rlPhaseStartCleanup
      rlRun "killall -9 -u foobar" 0-255
      rlRun "userdel -r foobar" 0
      rlRun "auditctl -D" 0
      rlFileRestore
      rlServiceRestore "auditd"
      rlRun "export PAGER=$OLD_PAGER" 0
  rlPhaseEnd

rlJournalPrintText

rlJournalEnd
