#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/auditctl-smoke
#   Description: Smoke test for auditctl
#   Author: Eduard Benes <ebenes@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2010 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

package="audit"
audit_conf="/etc/audit/auditd.conf"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $package
        rlFileBackup $audit_conf
        rlServiceStop "auditd" && sleep 3
        rlRun "sed -i 's/name_format.*/name_format = hostname/' $audit_conf"
        rlServiceStart "rsyslog"
        rlRun "service auditd start" 0
        rlRun "auditctl -D" 0
    rlPhaseEnd

    rlPhaseStartTest
        rlRun "auditctl -w /etc/passwd -p rwa -k passwd-watch"
	if ! rlIsRHEL '<6.5'; then	    
            rlRun "auditctl -l | grep '\-w /etc/passwd -p rwa -k passwd-watch'"
	else
            rlRun "auditctl -l | grep 'LIST_RULES: exit,always watch=/etc/passwd perm=rwa key=passwd-watch'"
	fi
        rlRun "cat /etc/passwd > /dev/null"
        sleep 3
        rlRun "ausearch -k passwd-watch -i -x /bin/cat"
        rlRun "ausearch -f /etc/passwd -i -x /bin/cat"
        rlRun "ausearch -k passwd-watch -ts recent"
        rlRun "auditctl -W /etc/passwd -p raw -k passwd-watch"
	if ! rlIsRHEL '<6.5'; then	    
 	    rlRun "auditctl -l | grep '\-W /etc/passwd -p rwa -k passwd-watch'" 1
	else
            rlRun "auditctl -l | grep 'LIST_RULES: exit,always watch=/etc/passwd perm=rwa key=passwd-watch'" 1
	fi
        # In RHEL 5 There is a limit 31 chars on string for the key, RHEL 6 will pass
        rlRun "auditctl -w /etc/passwd -k passwd-watch-looooooooooooong-file-naaaaameee-jupiiiiiiii -p raw" 255,0
        # Watch all files opened by given uid
        rlRun "auditctl -a exit,always -S openat -F auid=0 -k root-open-rule"
        rlRun "cat /etc/passwd > /dev/null"
        rlRun "ausearch -k root-open-rule -ts recent -i"
        rlRun "auditctl -a exit,always -S all -F pid=$$ -k pid-test-rule"
        rlRun "ausearch -ts recent -k pid-test-rule -i"
        # Following rules should not be legal. The older libs would allow them.
        rlRun "auditctl -a always,user -S openat -F filetype=file" 255
        rlRun "auditctl -a always,user -S openat -F ppid=1" 255
        # Should be legal. The older version would see it as an error.
        rlRun "auditctl -a always,exit -S openat -F dir=/etc -k allowtest" 0
        rlRun "auditctl -a always,exit -S openat -F exit=-eperm -k key1 -k key2" 0
        # New in RHEL 6.1 rebase to 2.1
        # New fields are now available 
        rlRun "auditctl -a exit,always -F subj_user=root -F subj_role=sysadm_r -F subj_type=sysadm_screen_t -F subj_sen=s0 -F subj_clr=s15:c0.c1023"
	if ! rlIsRHEL '<6.5'; then	    
            rlRun "auditctl -l | grep '\-a always,exit -S all -F subj_user=root -F subj_role=sysadm_r -F subj_type=sysadm_screen_t -F subj_sen=s0 -F subj_clr=s15:c0.c1023'"
	else
            rlRun "auditctl -l | grep 'LIST_RULES: exit,always subj_user=root subj_role=sysadm_r subj_type=sysadm_screen_t subj_sen=s0 subj_clr=s15:c0.c1023 syscall=all'"
	fi
    rlPhaseEnd

    rlPhaseStartTest "BZ#1120491"
        rlRun "auditctl -a exit,never -F auid=-1" 0
        rlRun "auditctl -l | grep '\-a never,exit -S all -F auid=-1'" 0
    rlPhaseEnd

    rlPhaseStartTest "BZ#1144252"
	rlRun "auditctl -D" 0 "Removing all audit rules"

	# Timestamp here is in local time, not UTC.
	printf -v BZ1144252_timestamp '%(%Y-%m-%d %H:%M:%S)T'

	rlRun -s "auditctl -a always,exit -S foobar" 255 "Adding invalid rule - via command line (should fail)"
	rlAssertGrep "Syscall name unknown: foobar" $rlRun_LOG
        rlRun "echo '-a always,exit -S foobar' >rules" 0
        rlRun -s "auditctl -R rules" 1 "Adding invalid rule - via rules file (should fail)"
	rlRun "test -s $rlRun_LOG" 1 "Checking that console output is empty"
	rlRun -s "journalctl --since '${BZ1144252_timestamp}' --no-pager --utc" 0 "Listing system log"
	rlAssertGrep "Syscall name unknown: foobar" $rlRun_LOG
	rlAssertGrep "There was an error in line 1 of rules" $rlRun_LOG
    rlPhaseEnd

    rlPhaseStartCleanup
        rlFileRestore
        rlRun "auditctl -D"
        rlRun "service auditd restart"
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
