#!/bin/bash
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/ausearch-checkpoint
#   Description: Sanity test for ausearch checkpoint feature
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2018 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

  rlPhaseStartSetup

      rlAssertRpm "$PACKAGE"

      rlFileBackup "/etc/audit/auditd.conf"
      rlFileBackup "/var/log/audit/audit.log"

      rlRun "service auditd stop && sleep 5" 0

      rlRun "sed -i 's/^flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
      rlRun "sed -i 's/^freq.*/freq = 1/' /etc/audit/auditd.conf" 0

      rlRun "service auditd start && sleep 5" 0
      rlRun "service auditd status" 0
      rlRun "auditctl -D"

      rlRun "auditctl -s > initial_state" 0 "Show initial audit status"
      rlFileSubmit "initial_state"

      rlRun "auditctl -l > initial_rules" 0 "Show initial audit rules"
      rlFileSubmit "initial_rules"

      rlRun "echo > /var/log/audit/audit.log" 0 "Make audit.log empty"
  rlPhaseEnd

  rlPhaseStartTest

        # Even empty log should produce checkpoint file.
        rlRun "touch test.log" 0
        rlRun "ausearch -if test.log --checkpoint chk0" 1
        rlAssertExists "chk0" 0

        # Check first checkpoint.
        rlRun "auditctl -m test1 && sleep 5" 0 "Generating message test1"
        rlRun -s "ausearch -ts recent --checkpoint chk1 -m USER" 0 \
              "Creating checkpoint 1, test1 should be reported"
        rlAssertGrep "test1" $rlRun_LOG
        # Fast forward first checkpoint.
        rlRun -s "ausearch -ts recent --checkpoint chk1 -m USER" 1 \
              "Moving checkpoint 1, test1 should not be reported"
        rlAssertNotGrep "test1" $rlRun_LOG

        # Check second checkpoint.
        rlRun -s "ausearch -ts recent --checkpoint chk2 -m USER" 0 \
              "Creating checkpoint 2, test1 should be reported"
        rlAssertGrep "test1" $rlRun_LOG

        # Fast forward first checkpoint again.
        rlRun "auditctl -m test2 && sleep 5" 0  "Generating message test2"
        rlRun -s "ausearch -ts recent --checkpoint chk1 -m USER" 0 \
              "Moving checkpoint 1, test2 should be reported"
        rlAssertGrep "test2" $rlRun_LOG

        # Fast forward first checkpoint again.
        rlRun -s "ausearch -ts recent --checkpoint chk1 -m USER" 1 \
              "Moving checkpoint 1, test2 should not be reported"
        rlAssertNotGrep "test2" $rlRun_LOG

        # Fast forward second checkpoint.
        rlRun "auditctl -m test3 && sleep 5" 0 "Generating message test1"
        rlRun -s "ausearch -ts recent --checkpoint chk2 -m USER" 0 \
              "Moving checkpoint 2, only test[23] should be reported"
        rlAssertNotGrep "test1" $rlRun_LOG
        rlAssertGrep "test2" $rlRun_LOG
        rlAssertGrep "test3" $rlRun_LOG
        
        # Fast forward second checkpoint.
        rlRun -s "ausearch -ts recent --checkpoint chk2 -m USER" 1 \
              "Moving checkpoint 2, not test should be reported"
        rlAssertNotGrep "test" $rlRun_LOG

        # Fast forward second checkpoint.        
        rlRun "auditctl -m test4 && sleep 5" 0  "Generating message test1"
        rlRun -s "ausearch -ts recent --checkpoint chk2 -m USER" 0 \
              "Moving checkpoint 2, test4 should be reported"
        rlAssertGrep "test4" $rlRun_LOG

  rlPhaseEnd

  rlPhaseStartCleanup

      rlRun "rm -f chk0 chk1 chk2 test.log" 0
      rlFileRestore
      rlRun "service auditd restart && sleep 5"

  rlPhaseEnd

rlJournalEnd

rlJournalPrintText
