#!/bin/bash
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   test.sh of /CoreOS/audit/Sanity/ausearch-eoe-timeout
#   Description: Sanity test for ausearch end_of_event_timeout
#   Author: Sergio Correia <scorreia@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2022 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"
PACKAGES="${PACKAGE} diffutils"

rlJournalStart
  rlPhaseStartSetup
      rlAssertRpm --all
      # Check if we have the expected minimum version (3.0.1).
      rlRun "packageVersion=$(rpm -q ${PACKAGE} --qf \
                              '%{name}-%{version}-%{release}\n')"
      rlTestVersion "${packageVersion}" '>=' 'audit-3.0.1-1' \
        || rlDie "Tested functionality is not in old version ${packageVersion}. Minimum expected version is audit-3.0.1"
  rlPhaseEnd

  rlPhaseStartTest "Test ausearch eoe_timeout"
        # NOTE: ausearch will format the date/time based on the system
        # timezone, so for reference, we will be using UTC here (by the
        # means of the TZ variable).

        # Default eoe-timeout is 2 seconds.
        # In the example input we have, it will change only when it's greater
        # than 4.
        for i in 2 3 4; do
          rlRun "TZ=UTC ausearch -if bz1921658.audit.log --eoe-timeout ${i} > ret-${i}.log"
          rlRun "cmp -s ret-${i}.log eoe-le4-utc.log" 0 "Comparing output when eoe_timeout is ${i}"
        done

        for i in 5 6 7 10 20; do
          rlRun "TZ=UTC ausearch -if bz1921658.audit.log --eoe-timeout ${i} > ret-${i}.log"
          rlRun "cmp -s ret-${i}.log eoe-gt4-utc.log" 0 "Comparing output when eoe_timeout is ${i}"
        done
  rlPhaseEnd

  rlPhaseStartCleanup
      rlRun "rm -f ret-*.log"
  rlPhaseEnd
rlJournalEnd

rlJournalPrintText
# vim:set ts=2 sw=2 et:
