#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/ausearch-localtime
#   Description: Test that ausearch correctly parse local time
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2015 Red Hat, Inc.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup
	rlAssertRpm $PACKAGE
	rlFileBackup "/etc/audit/auditd.conf"
	rlServiceStop "auditd" && sleep 3
	rlRun "sed -i 's/flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
	rlRun "sed -i 's/freq.*/freq = 1/' /etc/audit/auditd.conf" 0
	rlRun "service auditd start && sleep 3" 0
    rlPhaseEnd

    rlPhaseStartTest

	# Setup.
	rlRun "LANGBACKUP=$LANG" 0 "Backup LANG setting"
	rlRun "touch pass fail skip exceptions" 0

	# Test.
	if rlIsRHEL "6"; then
	    # On RHEL6 there are plenty of exception because
	    # glibc maintainer decided not to fix them.
	    EXCEPTIONS="ff_SN      \
			ff_SN.utf8 \
			ht_HT \
			ht_HT.utf8 \
			hy_AM \
			hy_AM.armscii8 \
			hy_AM.utf8 \
			my_MM \
			my_MM.utf8 \
			bo_CN \
			bo_CN.utf8 \
			bo_IN \
			bo_IN.utf8 \
			dz_BT \
			dz_BT.utf8 \
			nb_NO \
			nb_NO.iso88591 \
			nb_NO.utf8 \
			nn_NO \
			nn_NO.iso88591 \
			nn_NO.utf8 \
			no_NO \
			no_NO.iso88591 \
			no_NO.utf8 \
			norwegian \
			nynorsk"
	elif rlIsRHEL "7"; then
	    EXCEPTIONS="ff_SN      \
			ff_SN.utf8 \
			ht_HT \
			ht_HT.utf8 \
			my_MM \
			my_MM.utf8"
	elif rlIsRHEL "8"; then
	   EXCEPTIONS="ff_SN      \
			ff_SN.utf8 \
			gd_GB \
			gd_GB.iso885915 \
			gd_GB.utf8 \
			lzh_TW \
			lzh_TW.utf8 \
			my_MM \
			my_MM.utf8"
	elif rlIsFedora; then
	    EXCEPTIONS="ff_SN \
			ff_SN.utf8 \
			gd_GB \
			gd_GB.iso885915 \
			gd_GB.utf8 \
			lzh_TW \
			lzh_TW.utf8 \
			mnw_MM \
			mnw_MM.utf8 \
			my_MM \
			my_MM.utf8 \
			shn_MM \
			shn_MM.utf8"
	fi

	LOCALES=${LOCALES:-$(locale -a | grep -v 'bokm')}
	COUNT=0
	for X in $LOCALES; do

	    export LANG="$X"
	    export TZ="UTC"
	    auditctl -m '$X'

	    L[$COUNT]=$X
	    D[$COUNT]=$(date "+%x" | sed 's/[ ]*$//g')
	    T[$COUNT]=$(date "+%X" | sed 's/[ ]*$//g')

	    COUNT=$[$COUNT+1]
	done
	sleep 20

	for ((C=0; C<COUNT; C++)); do
	    fail=0

	    export LANG="${L[$C]}"
	    export TZ="UTC"
	    ausearch -r -m USER -ts "${D[$C]}" "${T[$C]}" >log 2>&1 || fail=1

	    if [ $fail -eq 0 ]; then
		echo "${L[$C]}" >>pass
		printf "%-50s %s\n" "Processing locale ${L[$C]}..." "PASS"
	    else
		if echo "$EXCEPTIONS" | grep -q "${L[$C]}"; then
		    printf "| % -15s | %15s | %s\n" "${L[$C]}" "${D[$C]} | ${T[$C]}" "$(cat log)" >>exceptions
		    printf "%-50s %s\n" "Processing locale ${L[$C]}..." "EXCEPTION"
		elif [[ "${T[$C]}" =~ $TZ ]]; then
		    printf "| % -15s | %15s | %s\n" "${L[$C]}" "${D[$C]} | ${T[$C]}" "$(cat log)" >>skip
		    printf "%-50s %s\n" "Processing locale ${L[$C]}..." "SKIP"
		else
		    printf "| %-15s | %15s | %s\n" "${L[$C]}" "${D[$C]} | ${T[$C]}" "$(cat log)" >>fail
		    printf "%-50s %s\n" "Processing locale ${L[$C]}..." "FAIL"
		fi
	    fi
	done

	# Clean-up.
	rlRun "export LANG='$LANGBACKUP'" 0 "Restoring LANG setting"

	# Reporting.
	rlLog "$( cat pass | wc -l) correct locales"
	rlLog "$( cat fail | wc -l) broken locales"
	rlLog "$( cat skip | wc -l) skipped locales"
	rlLog "$( cat exceptions | wc -l) exception locales"

	if [ $( cat fail | wc -l) -gt 0 ]; then
	    rlFail "Some locales are broken (see TESTOUT.log)"
	    cat fail
	else
	    rlPass "All locales are correct"
	fi

	# Locales broken because of glibc (BZ#1322585, BZ#1322292).
	if [ $( cat skip | wc -l) -gt 0 ]; then
	    rlLog "Some locales were skipped (see TESTOUT.log)"
	    cat skip
	fi

	# Broken locales that cannot be fixed easily.
	if [ $( cat exceptions | wc -l) -gt 0 ]; then
	    rlLog "Some locales are exceptions (see TESTOUT.log)"
	    cat exceptions
	fi

    rlPhaseEnd

    rlPhaseStartCleanup

	rlRun "service auditd stop && sleep 3" 0
	rlRun "rm -f pass fail skip exceptions" 0
	rlFileRestore
	rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
