#!/bin/bash
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/ausearch-smoke
#   Description: Sanity test for ausearch
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2013 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

LOG_DIR="http://nest.test.redhat.com/mnt/qa/scratch/omoris/audit-testing"

rlJournalStart

  rlPhaseStartSetup

      rlFileBackup "/etc/audit/auditd.conf"
      rlFileBackup "/var/log/audit/audit.log"

      rlRun "sed -i 's/^flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
      rlRun "sed -i 's/^freq.*/freq = 1/' /etc/audit/auditd.conf" 0

      rlRun "service auditd restart && sleep 5" 0

      rlRun "gcc -g -W -Wall -Wundef -D_GNU_SOURCE ausearch-test.c -o ausearch-test -lauparse -laudit" 0 "Build ausearch-test"
      rlRun "wget $LOG_DIR/audit.log.$(uname -m) -O audit.log" 0
      
  rlPhaseEnd

  if rlIsRHEL ">=7"; then
      rlPhaseStartTest "ausearch-test"
          rlRun "gcc -g -W -Wall -Wundef -D_GNU_SOURCE ausearch-test.c -o ausearch-test -lauparse -laudit"
          rlAssertExists "ausearch-test" || rlDie

          rlRun -s "./ausearch-test audit.log --continue"
          rlAssertGrep "Done - no problems detected" $rlRun_LOG

          rlRun "rm -f ausearch-test"
      rlPhaseEnd
  fi

  rlPhaseStartTest "ausearch: display of commas between categories (BZ#1120286)"
    cat > audit.log.c <<_EOF
type=AVC msg=audit(1405523880.601:4179): avc:  denied  { read } for  pid=20157 comm="bash" name="user20200" dev=vda3 ino=144202 scontext=unconfined_u:unconfined_r:sandbox_t:s0:c304,c875 tcontext=unconfined_u:object_r:mnt_t:s0 tclass=lnk_file
type=SYSCALL msg=audit(1405523880.601:4179): arch=c000003e syscall=2 success=no exit=-13 a0=87d750 a1=401 a2=180 a3=1b items=1 ppid=20156 pid=20157 auid=508 uid=508 gid=508 euid=508 suid=508 fsuid=508 egid=508 sgid=508 fsgid=508 tty=pts2 ses=395 comm="bash" exe="/bin/bash" subj=unconfined_u:unconfined_r:sandbox_t:s0:c304,c875 key=(null)
type=CWD msg=audit(1405523880.601:4179):  cwd="/nfs/staff/user20200"
type=PATH msg=audit(1405523880.601:4179): item=0 name="/u/user20200/.bash_history" nametype=UNKNOWN
_EOF
    rlRun "ausearch -if audit.log.c -m avc |grep 'type=SYSCALL.*subj=unconfined_u:unconfined_r:sandbox_t:s0:c304,c875'"
    rlRun "ausearch -if audit.log.c -m avc |grep 'type=AVC.*scontext=unconfined_u:unconfined_r:sandbox_t:s0:c304,c875'"
    rlRun "ausearch -if audit.log.c -m avc -i |grep 'type=SYSCALL.*subj=unconfined_u:unconfined_r:sandbox_t:s0:c304,c875'"
    rlRun "ausearch -if audit.log.c -m avc -i |grep 'type=AVC.*scontext=unconfined_u:unconfined_r:sandbox_t:s0:c304,c875'"
  rlPhaseEnd

if rlIsRHEL '>=6.6'; then
  rlPhaseStartTest "rebase to 2.3.7"
        rlLog "ausearch shouldn't interpret 'fp' field in CRYPTO_KEY_USER events"
        cat > audit_crypto_key.log <<_EOF
type=CRYPTO_KEY_USER msg=audit(1314776321.939:65010): user pid=633 uid=0 auid=0 ses=1445 subj=unconfined_u:system_r:sshd_t:s0-s0:c0.c1023 msg='op=destroy kind=server fp=9e:c3:25:e6:60:97:6b:2b:d0:c1:98:90:a1:b6:b4:d7 direction=? spid=633 suid=0 : exe="/usr/sbin/sshd" hostname=? addr=10.10.36.180 terminal=? res=success'
_EOF
        rlRun "ausearch -m CRYPTO_KEY_USER -i -if audit_crypto_key.log" 0
        rlRun "ausearch -m CRYPTO_KEY_USER -i -if audit_crypto_key.log | grep 'type=CRYPTO_KEY_USER.*fp=9e:c3:25:e6:60:97:6b:2b:d0:c1:98:90:a1:b6:b4:d7'" 0

        rlLog "test debug option for corrupted events entries (missing auid or pid fields)"
        cat > audit_corrupted.log <<_EOF
type=CRYPTO_KEY_USER msg=audit(1314776321.939:65011): user pid=633 uid=0 ses=1445 subj=unconfined_u:system_r:sshd_t:s0-s0:c0.c1023 msg='op=destroy kind=server fp=02:41:d6:dd:8e:57:f9:0b:48:7a:22:7c:3f:23:26:3f direction=? spid=633 suid=0 : exe="/usr/sbin/sshd" hostname=? addr=10.10.36.180 terminal=? res=success'
_EOF
        rlRun "ausearch -if audit_corrupted.log -ua 0 2>&1 | grep '<no matches>'"
        rlRun "ausearch -if audit_corrupted.log -ua 0 --debug 2>&1 | grep 'Malformed event skipped, rc=7. type=CRYPTO_KEY_USER'"

        rlLog "Interpret syscall field in SECCOMP events"
        # rhel 6 doesn't have seccomp but it still may be used as an aggregating server for rhel7/fedora events
	# syscall events for all architectures
        cat > audit_seccomp.log <<_EOF
type=SECCOMP msg=audit(1395076595.898:756): auid=1000 uid=0 gid=0 ses=1 subj=unconfined_u:unconfined_r:unconfined_t:s0-s0:c0.c1023 pid=3283 comm="test" sig=31 arch=80000015 syscall=231 compat=0 ip=0x7fe37deb9c09 code=0x0
type=SECCOMP msg=audit(1395076595.898:757): auid=1000 uid=0 gid=0 ses=1 subj=unconfined_u:unconfined_r:unconfined_t:s0-s0:c0.c1023 pid=3283 comm="test" sig=31 arch=80000016 syscall=231 compat=0 ip=0x7fe37deb9c09 code=0x0
type=SECCOMP msg=audit(1395076595.898:758): auid=1000 uid=0 gid=0 ses=1 subj=unconfined_u:unconfined_r:unconfined_t:s0-s0:c0.c1023 pid=3283 comm="test" sig=31 arch=c000003e syscall=231 compat=0 ip=0x7fe37deb9c09 code=0x0
type=SECCOMP msg=audit(1395076595.898:759): auid=1000 uid=0 gid=0 ses=1 subj=unconfined_u:unconfined_r:unconfined_t:s0-s0:c0.c1023 pid=3283 comm="test" sig=31 arch=40000003 syscall=231 compat=0 ip=0x7fe37deb9c09 code=0x0
type=SECCOMP msg=audit(1395076595.898:760): auid=1000 uid=0 gid=0 ses=1 subj=unconfined_u:unconfined_r:unconfined_t:s0-s0:c0.c1023 pid=3283 comm="test" sig=31 arch=c0000015 syscall=231 compat=0 ip=0x7fe37deb9c09 code=0x0
type=SECCOMP msg=audit(1395076595.898:761): auid=1000 uid=0 gid=0 ses=1 subj=unconfined_u:unconfined_r:unconfined_t:s0-s0:c0.c1023 pid=3283 comm="test" sig=31 arch=c00000b7 syscall=231 compat=0 ip=0x7fe37deb9c09 code=0x0
_EOF


        #Check that all syscalls are interpretted:
        rlRun "ausearch -if audit_seccomp.log -i" 0
        found_events=`ausearch -if audit_seccomp.log -i | grep '^type=SECCOMP' | wc -l`
        rlAssertEquals "Check that all syscalls are interpretted" $found_events 6
        rlRun "ausearch -m SECCOMP -if audit_seccomp.log -i" 0
        found_events2=`ausearch -m SECCOMP -if audit_seccomp.log -i | grep '^type=SECCOMP' | wc -l`
        rlAssertEquals "Check that all syscalls are interpretted" $found_events2 6

        #Check that you can search for syscall by name
        #find name of syscall 231, ausearch search by name only for current architecture
	# it internally translate syscall name to its number and searches by number
        sys_name=`ausyscall --dump | grep 231 | awk -F ' ' '{print $2}'`
        echo $sys_name
        rlRun "ausearch -if audit_seccomp.log -sc $sys_name -i" 0

        #Check that you can find syscall 231
        rlRun "ausearch -if audit_seccomp.log -sc 231 -i" 0
        found_events3=`ausearch -if audit_seccomp.log -sc 231 -i | grep '^type=SECCOMP' | wc -l`
        rlAssertEquals "Check that all syscalls are interpretted" $found_events3 6

	# check if ausearch can use directory as 'input'
	rlFileBackup --clean /var/log/audit2
	rlRun "mkdir /var/log/audit2" 0
        rlRun "auditctl -m test_non_standard_directory && sleep 10" 0
	ausearch -m USER | grep "test_non_standard_directory"
	rlRun "cp /var/log/audit/audit.log /var/log/audit2/" 0
    sleep 2
	rlRun -s "ausearch -if /var/log/audit2 -m USER"
	rlAssertGrep "test_non_standard_directory" $rlRun_LOG
  rlPhaseEnd
fi

  rlPhaseStartCleanup
    rlRun "rm -f $rlRun_LOG chk* ausearch-test" 0 "Clean-up"
    rlFileRestore
    rlRun "service auditd restart && sleep 5" 0
  rlPhaseEnd

rlJournalEnd

rlJournalPrintText
