#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/auvirt-smoke
#   Description: Sanity test for auvirt
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2013 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "audit" || rlDie
	rlAssertRpm "libvirt-client" || rlDie
	rlAssertRpm "libvirt" || rlDie
	rlServiceStart "libvirtd"
	rlFileBackup "/var/log/audit/audit.log" 
	rlRun "echo '' > /var/log/audit/audit.log" 0 "Cleaning audit log"
	rlRun "virsh define auvirt-test-A.xml" 0 "Defining test VM A"
	rlRun "virsh define auvirt-test-B.xml" 0 "Defining test VM B"

    rlPhaseEnd

    rlPhaseStartTest

        # sanity (domain name, user, start, stop)
        rlAssert0 "Checking that no auvirt records exist" `auvirt | wc -l` 
        S=`date '+%a %b %e %H:%M'`
        rlRun "virsh start auvirt-test-A" 0 "Starting test VM A"
        rlAssertEquals "Checking the number of auvirt records" `auvirt | wc -l` 1
	rlRun "auvirt | egrep \"$S [^-]\"" 0 "Checking auvirt record (lifetime)"
	rlLog "Sleeping for 80 seconds"; sleep 80 
        E=`date '+%H:%M'`
        rlRun "virsh destroy auvirt-test-A" 0 "Stopping test VM A"
        rlAssertEquals "Checking the number of auvirt records" `auvirt | wc -l` 1
	rlRun "auvirt | grep auvirt-test-A" 0 "Checking auvirt record (domain name)" 
	rlRun "auvirt | grep root" 0 "Checking auvirt record (user)" 
	rlRun "auvirt | grep \"$S - $E  (00:01)\"" 0 "Checking auvirt record (lifetime)"
		                        
        # --show-uuid
        rlRun "auvirt --show-uuid | grep 'ddc4bf8d-d60a-b8a1-775f-88a5f3323a4c'" 0 \
	    "Checking auvirt record (uuid)"

        # --start & --end
	rlAssertEquals "Checking recent auvirt" `auvirt -ts recent | wc -l` 1
	rlAssertEquals "Checking auvirt records (-ts yesterday)" `auvirt -ts yesterday | wc -l` 1
	rlAssertEquals "Checking auvirt records (-te yesterday)" `auvirt -te yesterday | wc -l` 0

        # --vm & --uuid
        rlRun "virsh start auvirt-test-B" 0 "Starting test VM A"
        rlRun "virsh destroy auvirt-test-B" 0 "Stopping test VM B"
	rlAssertEquals "Checking recent auvirt" `auvirt -ts recent | wc -l` 2
	rlAssertEquals "Checking recent auvirt (--vm)" \
	    `auvirt --vm auvirt-test-A -ts recent | wc -l` 1
	rlAssertEquals "Checking recent auvirt (--uuid)" \
	    `auvirt --uuid ddc4bf8d-d60a-b8a1-775f-88a5f3323a4c -ts recent | wc -l` 1
	rlAssertEquals "Checking recent auvirt (--uuid == --vm)" \
	    `auvirt --vm auvirt-test-A --uuid ddc4bf8d-d60a-b8a1-775f-88a5f3323a4c | wc -l` 1
	rlAssertEquals "Checking recent auvirt (--uuid !- --vm)" \
	    `auvirt --vm auvirt-test-B --uuid ddc4bf8d-d60a-b8a1-775f-88a5f3323a4c | wc -l` 0
	
	# --all-events
        rlRun "virsh start auvirt-test-A" 0 "Starting test VM A"
	rlRun "virsh setmem --size 64000 auvirt-test-A" 0
	rlRun "virsh destroy auvirt-test-A" 0 "Stopping test VM A"
	rlRun "auvirt --vm auvirt-test-A --all-events | grep start" 0 \
	    "Checking auvirt all-events-record (start)"
	rlRun "auvirt --vm auvirt-test-A --all-events | grep start" 0 \
	    "Checking auvirt all-events-record (stop)"
	rlRun "auvirt --vm auvirt-test-A --all-events | grep res | grep net" 0 \
	    "Checking auvirt all-events-record (res - net)"
	rlRun "auvirt --vm auvirt-test-A --all-events | grep res | grep vcpu" 0 \
	    "Checking auvirt all-events-record (res - vcpu)"
	rlRun "auvirt --vm auvirt-test-A --all-events | grep res | grep mem" 0 \
	    "Checking auvirt all-events-record (res - mem)"
	rlRun "auvirt --vm auvirt-test-A --all-events | grep res | grep mem | grep update" 0 \
	    "Checking auvirt all-events-record (res - mem - update)"

	# --proof 
	res=0
	counter=0
	for P in `auvirt --vm auvirt-test-A --all-events --proof | grep -A 1 '^start' | \
	          grep "Proof" | sed 's/[ ]*Proof: //'  |  tr -d ',' | tr ' ' '\n' | \
	          awk -F ':' '{print $2}'`; do
	    ausearch -a $P | grep 'type=VIRT_CONTROL' | grep 'virt=kvm op=start reason=booted'
	    res=$[$res + $?]
	    if [ $res -gt 0 ]; then
		rlLog "Checking proof $P (start) failed"
		ausearch -a $P
		auvirt --vm auvirt-test-A --all-events --proof | grep -B 1 "$P"
	    fi
	    counter=$[$counter + 1]
	done
	rlAssertEquals "Checking start proofs count" $counter 2
	rlAssertEquals "Checking start proofs content" $res 0

	# --proof 
	res=0
	counter=0
	for P in `auvirt --vm auvirt-test-A --all-events --proof | grep -A 1 '^stop' | \
	          grep "Proof" | sed 's/[ ]*Proof: //'  |  tr -d ',' | tr ' ' '\n' | \
	          awk -F ':' '{print $2}'`; do
	    ausearch -a $P | grep 'type=VIRT_CONTROL' | grep 'virt=kvm op=stop reason=destroyed'
	    res=$[$res + $?]
	    if [ $res -gt 0 ]; then
		rlLog "Checking proof $P (stop) failed"
		ausearch -a $P
		auvirt --vm auvirt-test-A --all-events --proof | grep -B 1 "$P"
	    fi
	    counter=$[$counter + 1]
	done
	rlAssertEquals "Checking stop proofs count" $counter 2
	rlAssertEquals "Checking stop proofs content" $res 0

	# --proof 
	res=0
	counter=0
	for P in `auvirt --vm auvirt-test-A --all-events --proof | grep -A 1 '^res' | \
	          grep "Proof" | sed 's/[ ]*Proof: //'  |  tr -d ',' | tr ' ' '\n' | \
	          awk -F ':' '{print $2}'`; do
	    ausearch -a $P | grep 'type=VIRT_RESOURCE' | grep 'virt=kvm resrc'
	    res=$[$res + $?]
	    if [ $res -gt 0 ]; then
		rlLog "Checking proof $P (res) failed"
		ausearch -a $P
		auvirt --vm auvirt-test-A --all-events --proof | grep -B 1 "$P"
	    fi
	    counter=$[$counter + 1]
	done
	rlAssertEquals "Checking res proofs count" $counter 4
	rlAssertEquals "Checking res proofs content" $res 0

	# --summary
	auvirt --summary
	rlRun "auvirt --summary | grep 'starts' | grep 3" 0 "Checking overall summary (starts)"
	rlRun "auvirt --summary | grep 'stops' | grep 3" 0 "Checking overall summary (stops)"
	rlRun "auvirt --summary | grep 'resource' | grep 10" 0 "Checking overall summary (resource assignments)"

	auvirt --summary --vm auvirt-test-A
	rlRun "auvirt --vm auvirt-test-A --summary | grep 'starts' | grep 2" 0 "Checking overall summary (starts)"
	rlRun "auvirt --vm auvirt-test-A --summary | grep 'stops' | grep 2" 0 "Checking overall summary (stops)"
	rlRun "auvirt --vm auvirt-test-A --summary | grep 'resource' | grep 7" 0 "Checking overall summary (resource assignments)"

    rlPhaseEnd

    rlPhaseStartCleanup

        rlRun "virsh undefine auvirt-test-A" 0 "Undefining test VM A"
        rlRun "virsh undefine auvirt-test-B" 0 "Undefining test VM B"
        rlFileRestore
	rlServiceStart "libvirtd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
