#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/fanotify
#   Description: Test for fanotify event
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2017 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1
PACKAGE="kernel"
        
if rlIsRHEL "<=7"; then
    syscall="open"
else
    syscall="openat"
fi

rlJournalStart

    rlPhaseStartSetup
        rlAssertRpm $PACKAGE || rlDie
        rlAssertRpm "audit"  || rlDie
        rlImport "audit/testing" || rlDie
        rlRun "gcc -o fanotify fanotify.c" || rlDie
        rlFileBackup "$auditDaemonConf"
        rlFileBackup "$auditLog"
        rlRun "auditConfig 'flush' 'incremental'"
        rlRun "auditConfig 'freq' '1'"
        rlRun "service auditd restart && sleep 5"
        rlRun "mkdir /tmp/fanotify-test"
	    rlRun "mount -t tmpfs -o size=8m swap /tmp/fanotify-test/"
        rlRun "auditctl -D"
        rlRun "auditctl -w /tmp/fanotify-test/testfile -k fanotify-test"
	    rlRun "./fanotify /tmp/fanotify-test &"
        rlRun "auditctl -s > initial_state"
    rlPhaseEnd

    rlPhaseStartTest 
        mark=$(date "+%T")
        rlRun "touch /tmp/fanotify-test/testfile" 0
        rlRun "sleep 3"
        rlRun "auditWaitForEvent $mark 'FANOTIFY' '' '120' 'test.msg' '-k fanotify-test -i'"
        rlAssertGrep "type=FANOTIFY .* resp=allow" test.msg
        rlAssertGrep "type=SYSCALL .* syscall=$syscall success=yes" test.msg
        rlAssertGrep "type=CWD" test.msg
        rlAssertGrep "type=PATH .* name=/tmp/fanotify-test/testfile" test.msg
        rlAssertGrep "type=PROCTITLE .* proctitle=touch /tmp/fanotify-test/testfile" test.msg
    rlPhaseEnd

    rlPhaseStartCleanup        
	    rlRun "killall fanotify"
        rlRun "auditctl -D"
	    rlRun "umount -f /tmp/fanotify-test" 
        rlRun "rm -f fanotify test.msg /tmp/fanotify-test -r"
        if rlIsRHEL '>=8'; then
            # Let's kill audit to get rid of netlink sockets piled up.
            i=0
            while pidof auditd; do
                i=$((i+1))
                rlRun "pkill auditd"
                sleep 3
            done
            rlRun "auditctl --reset_backlog_wait_time_actual >/dev/null ||:"
        fi
        rlFileSubmit "initial_state"
        rlFileRestore
        rlRun "service auditd restart && sleep 5" 0
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
