#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/flush
#   Description: Smoke test for flush config option
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

perf_log=""
perf_pid=""

count=1000
conf="/etc/audit/auditd.conf"

function perf() {

    local action=$1
    
    case $action in
        "start")
            perf_log=$(mktemp)

            while true; do
                ps -p $(pidof auditd) -o %cpu,%mem,cmd --no-headers >>${perf_log}
                sleep 1
            done &

            perf_pid=$!
            ;;

        "stop")
            kill $perf_pid
            echo "%CPU %MEM CMD"
            echo "============="
            cat $perf_log

            local max_cpu=$(awk '{print $1}' $perf_log | sort | tail -1)
            local max_mem=$(awk '{print $2}' $perf_log | sort | tail -1)

            echo "$max_cpu > 10.0" | bc -l || rlFail "Max CPU load over 10% ($max_cpu)"
            echo "$max_mem > 10.0" | bc -l || rlFail "Max MEM usage over 10% ($max_mem)"

            rlLog "Max CPU: ${max_cpu}%, Max MEM: ${max_mem}%"

            rm -f $perf_log
            ;;
    esac           
}

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "$PACKAGE"
        rlRun "rlImport audit/testing" || rlDie
        rlFileBackup "$conf"
        rlFileBackup "/var/log/audit/audit.log"
        rlServiceStop "auditd" && sleep 3

    rlPhaseEnd

    for flush in "none" "incremental" "incremental_async" "data" "sync"; do
        if [ $flush == "data" ] || [ $flush == "sync" ]; then
            [ $(uname -m) == "ppc64le" ] && continue
        fi

        rlPhaseStartTest "flush = $flush"

            # Setup.
            rlRun "sed -i \"s/^flush.*/flush = $flush/\" $conf" 0
            rlRun "sed -i \"s/^freq.*/freq = $count/\" $conf" 0
            rlRun "egrep \"^flush = $flush\$\" $conf " 0
            rlRun "service auditd start" 0
            perf "start"

            # Trigger & Verify
            ts=$(date "+%T")
            temp=$(mktemp)

            case $flush in
                "none")
                    start=$(date "+%s")
                    for i in $(seq 1 $count); do auditctl -m "test_${flush}_${i}"; done
                    stop=$(date "+%s")
                    rlRun "service auditd stop" 0
                    rlRun "service auditd start" 0
                    ;;
                
                "incremental"|"incremental_async")
                    start=$(date "+%s")
                    for i in $(seq 1 $count); do auditctl -m "test_${flush}_${i}"; done
                    stop=$(date "+%s")
                    ;;
                
                "data"|"sync")
                    start=$(date "+%s")
                    for i in $(seq 1 $count); do auditctl -m "test_${flush}_${i}"; done
                    stop=$(date "+%s")
                    ;;
            esac

            msg="test_${flush}_${count}"
            rlRun "auditWaitForEvent $ts 'USER' '$msg' 60"
            rlRun "ausearch --raw -ts $ts -m USER >$temp" 0
            rlAssertEquals "Checking number of events" $(cat $temp | grep "test_${flush}_" | wc -l) $count
            rlLog "Time spent on logging $count events is $((${stop}-${start}))s"

            # Clean-up.
            rlRun "rm -rf $temp" 0
            perf "stop"
            rlRun "service auditd stop && sleep 5" 0    
        rlPhaseEnd
    done

    rlPhaseStartCleanup

        rlFileRestore
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
