#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/internal-state-dump
#   Description: Smoke test for dump of internal state
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2018 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

# Since RHEL8 there is no seprate dispatcher.
if rlIsRHEL "<8"; then
    DISPATCHER_DIR="/etc/audisp"
else    
    DISPATCHER_DIR="/etc/audit"
fi

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "$PACKAGE"
        rlFileBackup --clean "/var/run/auditd.state"
        rlFileBackup --clean "/etc/audit/auditd.conf"
        rlFileBackup --clean "/var/log/audit/audit.log"
        rlFileBackup --clean "${DISPATCHER_DIR}/plugins.d/syslog.conf"
        
        rlServiceStop "auditd" && sleep 3
        rlRun "mount -t tmpfs -o size=$((1024 * 1024 * 8)) none /var/log/audit" 0
        rlRun "echo ''>/var/log/audit/audit.log" 0
        rlRun "restorecon -vvFR /var/log/audit" 0
        rlRun "sed -i \"s/^flush.*/flush = incremental/\" /etc/audit/auditd.conf" 0
        rlRun "sed -i \"s/^space_left .*/space_left = 7/\" /etc/audit/auditd.conf" 0
        rlRun "sed -i \"s/^admin_space_left .*/admin_space_left = 5/\" /etc/audit/auditd.conf" 0
        rlRun "sed -i \"s/^admin_space_left_action .*/admin_space_left_action = SUSPEND/\" /etc/audit/auditd.conf" 0
        rlRun "sed -i \"s/^freq.*/freq = 1/\" /etc/audit/auditd.conf" 0
        rlRun "sed -i \"s/^active.*/active = yes/\" ${DISPATCHER_DIR}/plugins.d/syslog.conf" 0
        rlRun "service auditd start && sleep 5" 0
        
    rlPhaseEnd

    rlPhaseStartTest "Sanity"

        # Sanity - dump state.
        rlRun "rm -f /var/run/auditd.state" 0-255
        rlRun "kill -s SIGCONT $(cat /var/run/auditd.pid)" 0
        rlAssertExists "/var/run/auditd.state" 

        rlRun "rm -f /var/run/auditd.state" 0-255
        rlRun "service auditd state" 0
        rlAssertExists "/var/run/auditd.state" 
        
    rlPhaseEnd

    rlPhaseStartTest "Time, current log size and dispatcher pid"

        # time = 07/13/18 11:09:15 
        # current log size = 13062 (in KB since RHEL8)
        # dispatcher pid = 0 (only on RHEL7)
        pid=$(pgrep "audispd")
        logsize=$(stat -c "%s" /var/log/audit/audit.log)
        datetime=$(date "+%D %H:%M")
        rlRun -s "service auditd state" 0

        if rlIsRHEL "<8"; then
            rlAssertGrep "^time = $datetime:[0-5][0-9]\$" $rlRun_LOG -E
            rlAssertGrep "^current log size = $logsize\$" $rlRun_LOG -E
            rlAssertGrep "^dispatcher pid = $pid\$" $rlRun_LOG -E
        else
            rlAssertGrep "^current time = $datetime:[0-5][0-9]\$" $rlRun_LOG -E
            rlAssertGrep "^current log size = $[$logsize/1024] KB\$" $rlRun_LOG -E
        fi
        
    rlPhaseEnd

    #if [ $(uname -m) != "ppc64le" ] &&  [ $(uname -m) != "s390x" ]; then
        rlPhaseStartTest "File system space (warning sent)/(action performed)"

            df -h
            # file system space (warning sent)/(action performed) = no    
            if rlIsRHEL "<8"; then
                pattern="file system space warning sent"
            else
                pattern="file system space action performed"
            fi
            rlAssertGrep "$pattern = no" $rlRun_LOG
            rlRun "dd if=/dev/zero of=/var/log/audit/blob bs=1M count=2"
            rlRun "sync && sleep 5" 0
            rlRun "auditctl -m 'test1'" 0
            rlRun "auditctl -m 'test2'" 0
            rlRun "auditctl -m 'test3'" 0
            rlRun "auditctl -m 'test4'" 0
            rlRun "auditctl -m 'test5'" 0
            rlRun "sync && sleep 5" 0
            rlRun -s "service auditd state" 0
            df -h
            rlAssertGrep "$pattern = yes" $rlRun_LOG            
            rlRun "rm -f /var/log/audit/blob" 0
            
        rlPhaseEnd
   
        rlPhaseStartTest "Admin space (warning sent)/(action performed) and logging suspended"

            df -h
            # admin space (warning sent)/(action performed) = no
            # logging suspended = no
            if rlIsRHEL "<8"; then
                pattern="admin space warning sent"
            else
                pattern="admin space action performed"
            fi
            rlAssertGrep "logging suspended = no" $rlRun_LOG
            rlAssertGrep "$pattern = no" $rlRun_LOG
            rlRun "dd if=/dev/zero of=/var/log/audit/blob bs=1M count=5"
            rlRun "sync && sleep 5" 0
            rlRun "auditctl -m 'test1'" 0
            rlRun "auditctl -m 'test2'" 0
            rlRun "auditctl -m 'test3'" 0
            rlRun "auditctl -m 'test4'" 0
            rlRun "auditctl -m 'test5'" 0
            rlRun "sync && sleep 5" 0
            rlRun -s "service auditd state" 0
            df -h
            rlAssertGrep "$pattern = yes" $rlRun_LOG
            rlAssertGrep "logging suspended = yes" $rlRun_LOG
            df -h
            rlRun "rm -f /var/log/audit/blob" 0
            rlRun "kill -s SIGUSR2 $(cat /var/run/auditd.pid)" 0
            
        rlPhaseEnd

        rlPhaseStartTest "Space left on partition"

            df -h
            # space left on partition = yes        
            rlAssertGrep "space left on partition = yes" $rlRun_LOG
            rlRun "dd if=/dev/zero of=/var/log/audit/blob bs=1M count=8" 1
            rlRun "sync && sleep 5" 0
            for i in $(seq 10); do
                rlRun "auditctl -m 'test$i'"
                rlRun "sleep 1" 
            done
            rlRun -s "service auditd state" 0
            df -h
            rlAssertGrep "space left on partition = no" $rlRun_LOG
            df -h
            rlRun "rm -f /var/log/audit/blob" 0
            
        rlPhaseEnd
        #fi

    rlPhaseStartCleanup

        rlRun "auditctl -D" 0
        rlRun "service auditd stop && sleep 5" 0
        rlRun "umount /var/log/audit" 0
        rlFileRestore
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
