#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/normalizer
#   Description: Sanity of normalizer
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "$PACKAGE" || rlDie

        rlRun "test_dir=$(mktemp -d)" 0
        rlRun "cp audit.log $test_dir" 0
        rlRun "cp audit_text_input.log $test_dir" 0
        rlRun "pushd $test_dir" 0

        types="$(ausearch -m 2>&1 | tr ' ' '\n'| egrep [A-Z] \
                 | egrep -v 'ALL|Argument|Valid|EOE|TRUSTED_APP|KERNEL_OTHER' \
                 | sort)"
        total=$(echo "$types" | wc -l)

        missing=0
        for t in $types; do
            ausearch -if audit.log -m $t --just-one --raw 2>/dev/null 1>&2
            if [ $? -ne 0 ]; then
                ((missing++))
            fi
        done
        rlLog "Using sample audit.log missing $missing of $total event types"

    rlPhaseEnd

    rlPhaseStartTest

        # Reads csv and text format and analyzes per line (event)
        rlRun -s "ausearch -if audit.log --format csv" 0

        # Write log if any error occurs at normalizing and delete lines upto csv log lines
        start_line=$(grep -Fn 'NODE,EVENT,DATE,' $rlRun_LOG |  awk -F ':' '{ print $1 }')
        for i in $(seq 1 $((start_line-1)) ); do
            rlLog "CSV error: $(sed -n "${i}p" $rlRun_LOG)"
            sed -i 1d $rlRun_LOG
        done
        sed -i 1d $rlRun_LOG
        
        events_count=$(cat $rlRun_LOG | wc -l)

        # Go through the log and process per each line
        for i in $(seq 1 $events_count ); do
            csv=$(sed -n "${i}p" $rlRun_LOG)
            text=$(ausearch -if audit.log --format text | sed -n "${i}p")
            orig_text="${text}"

            # Fill fields for analysis
            declare -A field            
            field["node"]=$(echo ${csv} | awk -F ',' '{print $1}')
            field["event"]=$(echo ${csv} | awk -F ',' '{print $2}')
            field["date"]=$(echo ${csv} | awk -F ',' '{print $3}')
            field["time"]=$(echo ${csv} | awk -F ',' '{print $4}')
            field["serial_num"]=$(echo ${csv} | awk -F ',' '{print $5}')
            field["event_kind"]=$(echo ${csv} | awk -F ',' '{print $6}')
            field["session"]=$(echo ${csv} | awk -F ',' '{print $7}')
            field["subj_prime"]=$(echo ${csv} | awk -F ',' '{print $8}')
            field["subj_sec"]=$(echo ${csv} | awk -F ',' '{print $9}')
            field["subj_kind"]=$(echo ${csv} | awk -F ',' '{print $10}')
            field["action"]=$(echo ${csv} | awk -F ',' '{print $11}')
            field["result"]=$(echo ${csv} | awk -F ',' '{print $12}')
            field["obj_prime"]=$(echo ${csv} | awk -F ',' '{print $13}')
            field["obj_sec"]=$(echo ${csv} | awk -F ',' '{print $14}')
            field["obj_kind"]=$(echo ${csv} | awk -F ',' '{print $15}')
            field["how"]=$(echo ${csv} | awk -F ',' '{print $16}')
 
            declare -A text_field
            # special case "On <node>, ..." - is often omitted
            if [ -n "${field["node"]}" ]; then
                text_field["node"]=$(echo ${text}  | awk -F ' ' '{print $2}'  | tr -d ',' )
                text=$(echo $text | awk -F ' ' '{$1=""; $2=""; print}')
            fi
            text_field["date"]=$(echo ${text} | awk -F ' ' '{print $3}')
            text_field["time"]=$(echo ${text} | awk -F ' ' '{print $2}')
            text_field["subj_prime"]=$(echo ${text} | awk -F ' ' '{print $4}' | tr -d ',' )
            # special case "<subj_prime>, acting as <subj_sec>, ..." - subj_sec is omitted when it is same as subj_prime
            if [ "${field['subj_prime']}" !=  "${field['subj_sec']}" ]; then
                text_field["subj_sec"]=$(echo ${text} | awk -F ',' '{print $2}' | awk -F ' ' '{print $3}')
                text=$(echo $text | awk -F ',' '{if (NF>1) {$2=""; print $0} else {print $0}}')
            fi
            text_field["action"]=$(echo ${text} | awk -F ' ' '{print $6}')
            text_field["result"]=$(echo ${text} | awk -F ' ' '{if ($5=="successfully"){print "success"} else if ($5=="unsuccessfully"){print "failed"}}')
            text_field["obj_prime"]=$(echo ${text} | awk -F ' ' '{print $7}')
            text_field["how"]=$(echo ${text} | grep -oP '(?<=using ).*')


            # Print all formats per the event
            echo ""
            echo "------ EVENT ${field[serial_num]} ----------------------"
            echo ""
            # Print raw format.
            echo "======== RAW  ========"
            ausearch -if  audit.log -a ${field[serial_num]} --format raw
            echo ""
            # Print csv format.
            echo "======== CSV  ========"
            echo "$csv"
            echo ""
            # Print text format.
            echo "======== TEXT ========"
            echo "$orig_text"
            echo ""


            # Mandatory fields.
            echo "===== MANDATORY FIELDS ====="
            mandatory_check_csv=0
            mandatory_check_text=0
            missing=()
            missing_text=()
            log_text=()
            for f in event date time serial_num event_kind session subj_prime action result obj_prime how; do
                echo -e "$f:    \t${field[$f]}"
                if [ -z "${field[$f]}" ]; then
                    mandatory_check_csv=1
                    missing+=($f)
                else
                    [ "$f" == "event" -o "$f" == "serial_num" -o "$f" == "event_kind" -o "$f" == "session" ] && continue

                    # When using the `text' format, ausearch will not
                    # display the "how" information, if the action starts
                    # with an "e", e.g. "ended-session" (USER_END), or
                    # "executed" (EXECVE).
                    # Reference: https://github.com/linux-audit/audit-userspace/commit/55bbbb1c8#diff-452e201fd2c95dcb32b764f89b923665dd7d5e25fbb14f99b68164de08b9b6a6R675
                    # Let's skip in these cases.
                    if [ "${f}" = "how" ]; then
                        action_s=$(printf '%c' "${field[action]}")
                        if [ "${action_s}" = "e" ]; then
                            rlLogInfo "Action (${field["action"]}) starts with an 'e', hence no 'how' is displayed in text format'"
                            continue
                        fi
                    fi

                    echo -e "\t\t${text_field[$f]}"
                    if [ "${field[$f]}" != "${text_field[$f]}" ]; then
                        mandatory_check_text=1
                        missing_text+=($f)
                    fi
                fi
            done
            echo ""


            # Optional fields.
            echo "===== OPTIONAL FIELDS ====="
            for f in node subj_sec subj_kind obj_sec obj_kind; do
                echo -e "$f:   \t${field[$f]}"
    
                if [ "$f" = "node" ] && [ -n "${field[$f]}" ]; then
                    echo -e "\t\t${text_field[$f]}"
                    if [ "${field["$f"]}" != "${text_field["$f"]}" ]; then
                        mandatory_check_text=1
                        missing_text+=("$f")
                    fi
                fi

                if [ "$f" = "subj_sec" ] && [ -n "${field[$f]}" ] && [ ${field['subj_prime']} !=  ${field['subj_sec']} ]; then
                    echo -e "\t\t${text_field[$f]}"
                    if [ "${field["$f"]}" != "${text_field["$f"]}" ]; then
                        mandatory_check_text=1
                        missing_text+=("$f")
                    fi
                fi

                
            done
            echo ""

            if [ $mandatory_check_csv -eq 1 ]; then
                rlFail "Event ${field[serial_num]} (type=${field[event]}) is missing: ${missing[*]}"
            elif [ $mandatory_check_text -eq 1 ]; then
                rlFail "Event ${field[serial_num]} (type=${field[event]}) in text format is missing or has wrong:  ${missing_text[*]}"
            fi
        done

    rlPhaseEnd

    rlPhaseStartTest

        rlRun -s "ausearch -if audit_text_input.log --format text" 0

    rlPhaseEnd

    rlPhaseStartCleanup

        rlRun "popd" 0
        rlRun "rm -rf $test_dir" 0

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
