#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/options-verify-email
#   Description: Sanity of verify_email option of auditd
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2017 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "$PACKAGE"
        rlFileBackup "/etc/audit/auditd.conf"       
        rlServiceStop "auditd" && sleep 3
    rlPhaseEnd
    
    rlPhaseStartTest "verify_email unspecified (default = yes)"

        rlRun "sed -i '/verify_email/d' /etc/audit/auditd.conf" 0

        # Setup.
        rlLog "Non-existing domain"
        rlRun "sed -i '/verify_email/d' /etc/audit/auditd.conf" 0
        rlRun "sed -i '/action_mail_acct/d' /etc/audit/auditd.conf" 0        
        rlRun "echo 'action_mail_acct = a@redhat.xxxxx' >> /etc/audit/auditd.conf" 0

        # Trigger.
        rlRun "service auditd start && sleep 10" 0

        # Validate
        rlRun -s "systemctl status auditd -f -n 100" 0
        rlAssertGrep "validate_email: failed" $rlRun_LOG

        # Clean-up.
        rlRun "service auditd stop && sleep 10" 0
        
        # Setup.
        rlLog "Existing domain"
        rlRun "sed -i '/verify_email/d' /etc/audit/auditd.conf" 0
        rlRun "sed -i '/action_mail_acct/d' /etc/audit/auditd.conf" 0        
        rlRun "echo 'action_mail_acct = a@redhat.com' >> /etc/audit/auditd.conf" 0

        # Trigger.
        rlRun "service auditd start && sleep 10" 0

        # Validate
        rlRun -s "systemctl status auditd -f -n 100" 0
        rlAssertNotGrep "validate_email: failed" $rlRun_LOG

        # Clean-up.
        rlRun "service auditd stop && sleep 10" 0
        
    rlPhaseEnd

    rlPhaseStartTest "verify_email = no"

        # Setup.
        rlLog "Non-existing domain"
        rlRun "sed -i '/verify_email/d' /etc/audit/auditd.conf" 0
        rlRun "sed -i '/action_mail_acct/d' /etc/audit/auditd.conf" 0        
        rlRun "echo 'verify_email = no' >> /etc/audit/auditd.conf" 0
        rlRun "echo 'action_mail_acct = a@redhat.xxxxx' >> /etc/audit/auditd.conf" 0
 
        # Trigger.
        rlRun "service auditd start && sleep 10" 0

        # Validate
        rlRun -s "systemctl status auditd -f -n 100" 0
        rlAssertNotGrep "validate_email: failed" $rlRun_LOG

        # Clean-up.
        rlRun "service auditd stop && sleep 10" 0
        
        # Setup.
        rlLog "Existing domain"
        rlRun "sed -i '/verify_email/d' /etc/audit/auditd.conf" 0
        rlRun "sed -i '/action_mail_acct/d' /etc/audit/auditd.conf" 0        
        rlRun "echo 'verify_email = no' >> /etc/audit/auditd.conf" 0
        rlRun "echo 'action_mail_acct = a@redhat.com' >> /etc/audit/auditd.conf" 0

        # Trigger.
        rlRun "service auditd start && sleep 10" 0

        # Validate
        rlRun -s "systemctl status auditd -f -n 100" 0
        rlAssertNotGrep "validate_email: failed" $rlRun_LOG

        # Clean-up.
        rlRun "service auditd stop && sleep 10" 0

    rlPhaseEnd

    rlPhaseStartTest "verify_email = yes"

        # Setup.
        rlLog "Non-existing domain"
        rlRun "sed -i '/verify_email/d' /etc/audit/auditd.conf" 0
        rlRun "sed -i '/action_mail_acct/d' /etc/audit/auditd.conf" 0        
        rlRun "echo 'verify_email = yes' >> /etc/audit/auditd.conf" 0
        rlRun "echo 'action_mail_acct = a@redhat.xxxxx' >> /etc/audit/auditd.conf" 0
 
        # Trigger.
        rlRun "service auditd start && sleep 10" 0

        # Validate
        rlRun -s "systemctl status auditd -f -n 100" 0
        rlAssertGrep "validate_email: failed" $rlRun_LOG

        # Clean-up.
        rlRun "service auditd stop && sleep 10" 0
        
        # Setup.
        rlLog "Existing domain"
        rlRun "sed -i '/verify_email/d' /etc/audit/auditd.conf" 0
        rlRun "sed -i '/action_mail_acct/d' /etc/audit/auditd.conf" 0        
        rlRun "echo 'verify_email = yes' >> /etc/audit/auditd.conf" 0
        rlRun "echo 'action_mail_acct = a@redhat.com' >> /etc/audit/auditd.conf" 0

        # Trigger.
        rlRun "service auditd start && sleep 10" 0

        # Validate
        rlRun -s "systemctl status auditd -f -n 100" 0
        rlAssertNotGrep "validate_email: failed" $rlRun_LOG

        # Clean-up.
        rlRun "service auditd stop && sleep 10" 0

    rlPhaseEnd

    rlPhaseStartCleanup

        rlFileRestore
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
