#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/options-write-logs
#   Description: Sanity of write_logs option of auditd
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "$PACKAGE"

        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "/var/log/audit/audit.log"
       
        rlServiceStop "auditd" && sleep 3

        rlRun "sed -i 's/^flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
        rlRun "sed -i 's/^freq.*/freq = 1/' /etc/audit/auditd.conf" 0

        file=$(mktemp)

    rlPhaseEnd

    rlPhaseStartTest "write_logs = no"

        # Setup.
        rlRun "rm -f /var/log/audit/audit.log" 0
        rlRun "sed -i 's/^write_logs.*/write_logs = no/' /etc/audit/auditd.conf" 0
        rlRun "service auditd start && sleep 5" 0
        rlRun "PAGER='' service auditd status" 0
        rlRun "auditctl -D" 0
        rlRun "auditctl -a always,exit -W $file -k WRITE_LOGS_NO" 0
        mark=$(date "+%T")
        
        # Trigger.
        rlRun "cat $file && sleep 1" 0

        # Verify.
        rlRun "ausearch -ts $mark -m SYSCALL -k WRITE_LOGS_NO --raw" 1
        rlAssertNotExists "/var/log/audit/audit.log" 0

        # Clean-up.
        rlRun "auditctl -D" 0
        rlRun "service auditd stop && sleep 5" 0

    rlPhaseEnd

    rlPhaseStartTest "write_logs = yes"

        # Setup.
        rlRun "sed -i 's/^write_logs.*/write_logs = yes/' /etc/audit/auditd.conf" 0
        rlRun "service auditd start && sleep 5" 0
        rlRun "PAGER='' service auditd status" 0
        rlRun "auditctl -D" 0
        rlRun "auditctl -a always,exit -W $file -k WRITE_LOGS_YES" 0
        mark=$(date "+%T")
        
        # Trigger.
        rlRun "cat $file && sleep 1" 0

        # Verify.
        rlRun "ausearch -ts $mark -m SYSCALL -k WRITE_LOGS_YES --raw" 0

        # Clean-up.
        rlRun "auditctl -D" 0
        rlRun "service auditd stop && sleep 5" 0

    rlPhaseEnd

    rlPhaseStartCleanup

        rlRun "rm -f $TMP_FILE" 0
        rlRun "auditctl -D" 0
        rlFileRestore
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
