#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/auparse-missing-information-when-used-with-format-text
#   Description: Check that auparse does not miss information when used with --format-text
#   Author: Martin Zeleny <mzeleny@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2022 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"
DIR="/root/bz2061726"

rlJournalStart
    rlPhaseStartSetup
        rlImport --all || rlDie "Beaker library not loaded"
        rlAssertRpm $PACKAGE || rlDie "Missing $PACKAGE"
        rlRun "rlFileBackup $auditLog"
        rlFileBackup --clean "/etc/audit/"

        rlServiceStart "auditd"
        rlRun "auditctl -D" 0 "Delete all rules before test"

        rlRun "mkdir $DIR"
        rlRun "auditctl -w $DIR -p ra -k bz2061726" 0 "Set watch rule to $DIR"
        rlRun "auditctl -l" 0 "List rules"
    rlPhaseEnd

    rlPhaseStartTest "'ls' in watched directory and query the log"
        rlRun "pushd $DIR"
        rlRun "ls"
        rlRun -s "ausearch -k bz2061726 --format text -ts recent"
        rlAssertGrep "successfully opened-file $DIR" $rlRun_LOG
        rm $rlRun_LOG
        rlRun "popd"
    rlPhaseEnd

    rlPhaseStartCleanup
        rlRun "auditctl -W $DIR -p ra -k bz2061726" 0 "Remove watch rule on $DIR"
        rlRun "rmdir $DIR"
        rlRun "rlServiceRestore auditd"
        rlRun "rlFileRestore"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
