#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1028635-audisp-remote-will-not-reconnect-to-remote-server
#   Description: Test for BZ#1028635 (audisp-remote will not reconnect to remote server)
#   Author: Patrik Kis <pkis@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2014 Red Hat, Inc.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

# TEST NOTE:
    # In this test network error EAI_AGAIN (-3) returned by getaddrinfo is simulated.
    # For that glibc need fix from BZ#1044628 (RHEL-6.5.z and higher), the resolvable domain name
    # must not be in /etc/hosts and the /etc/resolv.conf must not contain any name server.
    # If the conditions above are met, auditd won't be able to connect to remote auditd
    # and it assumes temporary error and should reconnect when the error condition disappear.

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm "$PACKAGE"

        # Imports.
        rlRun "rlImport distribution/sync" || rlDie
        rlRun "rlImport audit/testing" || rlDie

        # Backup.
        rlFileBackup --clean "/etc/hosts"
        rlFileBackup --clean "$auditDaemonConf"
        rlFileBackup --clean "$auditLog"
        rlFileBackup --clean "$auditDispatcherConf"
        rlFileBackup --clean "$auditRemotePluginConf"

        # Setup.
        rlServiceStop "auditd" && sleep 3
        rlRun "auditConfig 'flush' 'INCREMENTAL'" 0
        rlRun "auditConfig 'freq' '1'" 0
        syncRun "SERVER" "auditConfig 'tcp_listen_port' '60'" 0
        syncRun "CLIENT" "echo \"${syncSERVER} bz1028635-audit-server\" >>/etc/hosts" 0
        
        syncRun "CLIENT" "auditConfig 'active' 'yes' $auditRemotePluginConf" 0
        syncRun "CLIENT" "auditConfig 'remote_server' 'bz1028635-audit-server' $auditDispatcherConf"
        syncRun "CLIENT" "auditConfig 'port' 60 $auditDispatcherConf"

        # Stop NM, otherwise it will mess with resolv.conf
        rlServiceStop NetworkManager

        rlRun "TmpDir=$(mktemp -d)" 0
        rlRun "pushd $TmpDir" 0
    rlPhaseEnd

    rlPhaseStartTest "$syncROLE"

        #1: Server setup
        syncRun "SERVER" "service auditd start && sleep 5" 0
        syncRun "SERVER" "syncSet SERVER_SETUP_READY" 0
           
        #2: Client setup
        syncRun "CLIENT" "syncExp SERVER_SETUP_READY" 0
        syncRun "CLIENT" "ping -c2 bz1028635-audit-server" 0
        syncRun "CLIENT" "service auditd start && sleep 5" 0
        client_ts=$(date "+%T")
        for i in $(seq 1 5); do 
            syncRun "CLIENT" "auditctl -m test-msg-ready-$i" 0
        done
        syncRun "CLIENT" "syncSet CLIENT_SETUP_DONE $client_ts" 0

        #3: Server check the remote connection
        syncRun "SERVER" "syncExp CLIENT_SETUP_DONE >client_ts_file" 0
        syncRun "SERVER" "auditWaitForEvent $(cat client_ts_file) USER 'test-msg-ready.*${CLIENTS}'" 0
        syncRun "SERVER" "syncSet SERVER_TEST_READY" 0
           
        #4: Client 
        syncRun "CLIENT" "syncExp SERVER_TEST_READY" 0
        syncRun "CLIENT" "cp /etc/resolv.conf /etc/resolv.conf.bck" 0
        syncRun "CLIENT" "echo -n >/etc/resolv.conf" 0
        syncRun "CLIENT" "sed -i \"/$syncSERVER bz1028635-audit-server/d\" /etc/hosts" 0
        syncRun "CLIENT" "ping -c2 bz1028635-audit-server" 2
        syncRun "CLIENT" "auditWaitForSyslogEvent 'udisp-remote started with queue_size'" 0
        client_ts=$(date "+%T")
        for i in $(seq 1 5); do
            syncRun "CLIENT" "auditctl -m test-msg-should-arrive-later-$i" 0
        done
        syncRun "CLIENT" "echo \"$syncSERVER bz1028635-audit-server\" >>/etc/hosts" 0
        for i in $(seq 1 5); do
            syncRun "CLIENT" "auditctl -m test-msg-reconnect-now-$i"
        done
        syncRun "CLIENT" "syncSet CLIENT_TEST_DONE $client_ts" 0

        #5: Server 
        syncRun "SERVER" "syncExp CLIENT_TEST_DONE >client_ts_file" 0
        syncRun "SERVER" "auditWaitForEvent $(cat client_ts_file) USER 'test-msg-reconnect-now.*${CLIENTS}'" 0
        syncRun "SERVER" "auditWaitForEvent $(cat client_ts_file) USER 'test-msg-should-arrive-later.*${CLIENTS}'" 0
    rlPhaseEnd
    
    rlPhaseStartCleanup
        rlRun "popd" 0
        rlRun "rm -rf $TmpDir" 0
        rlFileRestore
        rlServiceRestore "NetworkManager"
        rlServiceRestore "auditd"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
