#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1053600-Systemd-is-not-sending-AUDIT-SYSTEM-SHUTDOWN
#   Description: Test for BZ#1053600 (Systemd is not sending AUDIT_SYSTEM_SHUTDOWN)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2014 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
	rlFileBackup "/var/log/audit/audit.log"
    rlPhaseEnd

    if ! [ -e /var/tmp/bz1053600-reboot-A ]; then

        rlPhaseStartTest
   	    rlRun "echo '' > /var/log/audit/audit.log" 0 "Emptying audit log"	   
	    rlRun "touch /var/tmp/bz1053600-reboot-A" 0 "Rebooting"
        rlPhaseEnd

        rhts-reboot

    elif ! [ -e /var/tmp/bz1053600-reboot-B ]; then

        rlPhaseStartTest
	    rlRun -s "aulast" 0 "Running aulast"
	    rlRun "egrep '^reboot\s+system boot\s+[^\(\)]*\$' $rlRun_LOG" 0 "Checking boot entry"
	    rlRun -s "ausearch --start today -m SYSTEM_SHUTDOWN -r" 0
	    rlAssertEquals "Checking number of SYSTEM_SHUTDOWN events" $(cat $rlRun_LOG | wc -l) 1
	    rlRun -s "ausearch --start today -m SYSTEM_BOOT -r" 0
	    rlAssertEquals "Checking number of SYSTEM_BOOT events" $(cat $rlRun_LOG | wc -l) 1
	    rlRun "touch /var/tmp/bz1053600-reboot-B" 0 "Rebooting"
        rlPhaseEnd

        rhts-reboot

    else
        rlPhaseStartTest
	    rlRun -s "aulast" 0 "Running aulast"
	    rlRun "egrep '^reboot\s+system boot\s+.*\(.+\)\$' $rlRun_LOG" 0 "Checking first boot entry"
	    rlRun "egrep '^reboot\s+system boot\s+[^\(\)]*\$' $rlRun_LOG" 0 "Checking second boot entry"
	    rlRun -s "ausearch --start today -m SYSTEM_SHUTDOWN -r" 0
	    rlAssertEquals "Checking number of SYSTEM_SHUTDOWN events" $(cat $rlRun_LOG | wc -l) 2
	    rlRun -s "ausearch --start today -m SYSTEM_BOOT -r" 0
	    rlAssertEquals "Checking number of SYSTEM_BOOT events" $(cat $rlRun_LOG | wc -l) 2
        rlPhaseEnd

    fi

    rlPhaseStartCleanup
        rlFileRestore
	rlRun "rm -f /var/tmp/bz1053600-reboot-*" 0 "Clean-up"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
