#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1180675-ppc64le-rules-with-F-arch-ppc64le-fail-to-load
#   Description: Test for BZ#1180675 (ppc64le rules with "-F arch=ppc64le" fail to load)
#   Author: Patrik Kis <pkis@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2015 Red Hat, Inc.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlAssertRpm kernel
        rlFileBackup "/etc/audit/auditd.conf"	
        rlFileBackup "/var/log/audit/audit.log"
        rlServiceStop "auditd" && sleep 3
        rlRun "sed -i 's/flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
        rlRun "sed -i 's/freq.*/freq = 1/' /etc/audit/auditd.conf" 0
        rlRun "service auditd start && sleep 3" 0
        rlRun "TmpDir=\$(mktemp -d)" 0 "Creating tmp directory"
        rlRun "pushd $TmpDir"
    rlPhaseEnd

    rlPhaseStartTest
        ARCH=`uname -p`
        rlLog "The platform architecture is $ARCH"
        D=`date +%X`
        echo D=$D
        rlRun "auditctl -D"
        rlRun "auditctl -a exit,always -F arch=$ARCH -S execve"
        sleep 1
        rlRun "auditctl -l"
        rlRun "ausearch -i -ts \"$D\" &>audit.log"
        cat audit.log
        rlAssertGrep "type=EXECVE" audit.log
        rlRun "auditctl -D"
    rlPhaseEnd

    rlPhaseStartCleanup
        rlRun "popd"
        rlRun "rm -r $TmpDir" 0 "Removing tmp directory"
        rlRun "service auditd stop && sleep 3" 0
        rlFileRestore
        rlServiceRestore "auditd"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
