#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1367703
#   Description: Test for BZ#1367703
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2017 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "audit" || rlDie
        
    rlPhaseEnd

    for TS in recent today yesterday this-week this-month this-year; do

        minute=60
        hour=$[$minute*60]
        day=$[$hour*24]
        week=$[$day*7]
        month=$[$day*31]
        year=$[$day*365]
        now=$(date +"%s")

        case $TS in
            "recent")
                pass=$[$now-5*$minute]
                fail=$[$now-15*$minute]
                ;;
            "today")
                pass=$[$now]
                fail=$[$now-2*$day]
                ;;
            "yesterday")
                pass=$[$now-$day]
                fail=$[$now-2*$day]
                ;;
            "this-week")
                pass=$[$now]
                fail=$[$now-$week]
                ;;
            "this-month")
                pass=$[$now]
                fail=$[$now-$month]
                ;;
            "this-year")
                pass=$[$now]
                fail=$[$now-$year]
                ;;
        esac

        rlPhaseStartTest "$TS"
    
            # Setup.
            rlRun "cp -f sample.log.template sample.log" 0
            rlRun "sed -i \"s/<PASS>/$pass/\" sample.log" 0
            rlRun "sed -i \"s/<FAIL>/$fail/\" sample.log" 0            

            # Trigger.
            rlRun -s "auvirt -ts $TS --proof --file sample.log" 0

            # Verify.
            rlAssertGrep    "pass" $rlRun_LOG
            rlAssertNotGrep "fail" $rlRun_LOG

            # Clean-Up.
            rlRun "rm -f sample.log" 0            
        
        rlPhaseEnd
    done          


    rlPhaseStartCleanup
    rlPhaseEnd

rlJournalPrintText
rlJournalEnd
