#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1379453
#   Description: Test for BZ#1379453
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "$PACKAGE"
        rlRun "gcc -o pid pid.c -Wall -std=c99 -pedantic -lpthread -laudit"
        rlAssertExists "pid" || rlDie

        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "/var/log/audit/audit.log"
        rlServiceStop "auditd" && sleep 5
        rlRun "sed -i \"s/^flush.*/flush = incremental/\" /etc/audit/auditd.conf" 0
        rlRun "sed -i \"s/^freq.*/freq = 1/\" /etc/audit/auditd.conf" 0
        rlRun "service auditd start && sleep 5" 0
        
    rlPhaseEnd

    rlPhaseStartTest

        # Setup.
        mark=$(date "+%T")

        # Trigger.
        rlRun -s "./pid" 0

        # Verify.
        main_pid=$(grep "Main" $rlRun_LOG | awk -F '= ' '{print $2}')

        fail=0
        for i in $(seq 0 15); do
            grep "Thread $i: PID = $main_pid" $rlRun_LOG || { fail=1; break; }
        done
        if [ $fail -eq 0 ]; then
            rlPass "Sanity check"
        else
            rlFail "Sanity check - unexpected error"
        fi
        echo "main_pid = $main_pid"

        ausearch -ts $mark -m USER -p $main_pid
        for i in $(seq 0 15); do
            rlRun "ausearch -ts $mark -m USER -p $main_pid | grep \"thread $i \"" 0 \
                  "Checking audited pid from thread $i"
        done
                
    rlPhaseEnd
        

    rlPhaseStartCleanup

        rlFileRestore
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
