#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1406525
#   Description: Test for BZ#1406525
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2017 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlRun "getenforce | grep Enforcing" 0 || rlDie

        rlRun "rlImport 'selinux-policy/common'"
        rlSESetTimestamp

        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "/var/log/audit/audit.log"

        rlServiceStop "auditd" && sleep 3

        rlRun "sed -i 's/^flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
        rlRun "sed -i 's/^freq.*/freq = 1/' /etc/audit/auditd.conf" 0
        rlRun "sed -i 's/^log_format.*/log_format = RAW/' /etc/audit/auditd.conf" 0
        
        rlRun "service auditd start && sleep 5" 0

        tmp_file=$(mktemp)
        mark=$(date "+%T")

    rlPhaseEnd

    rlPhaseStartTest

        rlRun     "chcon -t games_t $tmp_file" 1
        rlRun -s  "ausearch -ts $mark -m AVC -sv no --raw" 0
        rlRun -ts "cat $rlRun_LOG | audit2allow" 0
        rlAssertNotGrep "STDERR" "$rlRun_LOG"
        
    rlPhaseEnd

    rlPhaseStartCleanup

        rlRun "rm -f $tmp_file $rlRun_LOG" 0
        rlSECheckAVC --ignore 'type=AVC.*denied.* { relabelto } .*games_t.*'
        rlFileRestore
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText
rlJournalEnd
