#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1437626
#   Description: Test for BZ#1437626
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2017 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "audit" || rlDie
        
        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "/var/log/audit/audit.log"

        rlServiceStop "auditd" && sleep 3

        rlRun "sed -i 's/^flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
        rlRun "sed -i 's/^freq.*/freq = 1/' /etc/audit/auditd.conf" 0
        rlRun "sed -i 's/^log_format.*/log_format = RAW/' /etc/audit/auditd.conf" 0
        
        rlRun "service auditd start && sleep 5" 0
        rlRun "auditctl -D" 0

    rlPhaseEnd

    rlPhaseStartTest

        # Setup.
        eth="$(ip route list | grep 'default' | awk '{ print $5; }' | head -1)"
        mark=$(date "+%T")

        syscall="bind"
        rlIsRHEL "<8" && [[ $(uname -m) =~ ppc ]] && syscall="socketcall"        
        rlRun "auditctl -a exit,always -F arch=b64 -S $syscall -k BZ1437626" 0
        
        # Trigger.
        rlIsRHEL ">=8" && python="/usr/libexec/platform-python" || python="python"
        rlRun "$python -c \"import socket; socket.socket(socket.PF_PACKET, socket.SOCK_RAW).bind(('$eth', 0x800))\"" 0
        rlRun "sleep 5" 0
        
        # Verify.
        rlRun "auditctl -l" 0
        rlRun "ausearch -ts $mark -m SOCKADDR -k BZ1437626" 0
        rlRun "ausearch -ts $mark -m SOCKADDR -k BZ1437626 -i" 0
        
        # Clean-up.
        rlRun "auditctl -D" 0
        
    rlPhaseEnd

    rlPhaseStartCleanup

        rlRun "rm -f $tmp_file $rlRun_LOG" 0
        rlFileRestore
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText
rlJournalEnd
