#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1465558
#   Description: Test for BZ#1465558
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2010 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="kernel"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "kernel" || rlDie
        rlAssertRpm "audit"  || rlDie
        rlRun "uname -a" 0 
        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "/var/log/audit/audit.log"
        rlServiceStop "auditd" && sleep 5
        rlRun "sed -i \"s/^flush.*/flush = incremental/\" /etc/audit/auditd.conf" 0
        rlRun "sed -i \"s/^freq.*/freq = 1/\" /etc/audit/auditd.conf" 0
        rlRun "service auditd start && sleep 5" 0
        rlRun "useradd testuser" 0
        
    rlPhaseEnd

    rlPhaseStartTest 

        # Setup.
        mark=$(date "+%T")
        caps="cap_fp=none cap_fi=none cap_fe=0 cap_fver=0"
	rlRun "auditctl -D" 0
        rlRun "auditctl -w /etc/shadow -p rwx -F key=bz1465558" 0
    
        # Trigger.        
        rlRun "cat /etc/shadow" 0        
        
        # Verify.
        sleep 30
        rlRun -s "ausearch -ts $mark -k bz1465558 -i"
        rlRun "grep -e \"type=PATH .* name=/etc/shadow .* $caps\" $rlRun_LOG" 0

    rlPhaseEnd

    rlPhaseStartCleanup

        rlRun "auditctl -D" 0
        rlFileRestore    
        rlServiceRestore "auditd"
        rlRun "userdel -r testuser" 0
    
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
