#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1488822
#   Description: Test for BZ#1488822
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2018 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="kernel"

rlJournalStart
       
rlPhaseStartSetup

    rlAssertRpm "kernel" || rlDie
    rlAssertRpm "audit"  || rlDie

    # Setup (1/2).
    rlFileBackup "/var/log/audit/audit.log" 
    rlRun "echo 'echo \$*' >trigger" 0
    rlRun "chmod a+x trigger" 0
    
    # Sanity check.
    rlRun "./trigger {1..10}" 0

    # Setup (2/2).
    if [[ $(uname -m) =~ i[36]86 ]]; then
        arch=b32
    else
        arch=b64
    fi
    rlRun "auditctl -a always,exit -F arch=${arch} -S execve" 0
    
rlPhaseEnd

rlPhaseStartTest 

    # Verify.
    for i in 10 24 25 50 100 1000 10000 100000; do
        rlRun "./trigger {1..$i} >/dev/null" 0
    done
    
rlPhaseEnd
        
rlPhaseStartCleanup    

    # Clean-up.
    rlRun "auditctl -D" 0
    rlRun "rm -f trigger" 0
    rlFileRestore
    
rlPhaseEnd

rlJournalPrintText

rlJournalEnd
