#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1573889-auditd-busy-loop-in-rotate-logs-with-num-logs
#   Description: Test for BZ#1573889 (auditd busy loop in rotate_logs() with num_logs <)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2018 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm $PACKAGE
        rlFileBackup --clean "/etc/audit/auditd.conf"
        rlServiceStop "auditd" && sleep 3
        rlRun "sed -i \"s/^num_logs.*/num_logs = 0/\" /etc/audit/auditd.conf" 0
        rlRun "sed -i \"s/^max_log_file_action.*/max_log_file_action = IGNORE/\" /etc/audit/auditd.conf" 0        
        rlRun "service auditd start && sleep 5" 0

    rlPhaseEnd

    rlPhaseStartTest

        # Setup.
        auditd_pid=$(cat /var/run/auditd.pid)
        strace_log=$(mktemp)
        ts=$(date "+%T")
        av_before=$(stat --format "%a" /var/log/audit/audit.log)
        rlRun "strace -f -p $auditd_pid -o $strace_log &" 0 "Start strace"
        strace_pid=$!
        
        # Trigger.
        rlRun "kill -USR1 $auditd_pid" 0 "Force log rotate"
        rlRun "sleep 5" 0 "Wait a few seconds"

        # Clean-up.
        rlRun "kill $strace_pid" 0 "Stop strace"
        rlRun "service auditd stop && sleep 5" 0 "Stop auditd"
        kill -0 $auditd_pid && rlRun "kill -9 $auditd_pid" 0 "Force stop auditd"

        # Verify.        
        av_after=$(stat --format "%a" /var/log/audit/audit.log)
        rlRun "ausearch -ts $ts -m DAEMON_ROTATE" 0 "Rotation should be logged"
        rlAssertNotGrep "rename(\"/var/log/audit/audit\.log\.[0-9]\+\".*) = -1 ENOENT" $strace_log
        rlAssertEquals "Check log permissions" $av_before $av_after
        
    rlPhaseEnd

    rlPhaseStartCleanup

        rlFileRestore
        rlServiceRestore "auditd"
        rlRun "rm -r $strace_log" 0
        
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
