#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1625156-audisp-remote-does-not-connect-when-remote-ending
#   Description: Test for BZ#1625156 (audisp-remote does not connect when remote ending)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2019 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup
        rlAssertRpm "$PACKAGE"
        rlRun "rlImport audit/testing" || rlDie
        rlFileBackup --clean "/var/log/audit/"
        rlFileBackup --clean "$auditDaemonConf"
        rlFileBackup --clean "$auditDispatcherConf"

        # Stop auditd.
        rlServiceStop "auditd" && sleep 3

        # Initial configuration of auditd.
        rlRun "auditConfig 'flush' 'INCREMENTAL'" 0
        rlRun "auditConfig 'freq' '1'" 0

        # Start auditd.
        rlServiceStart "auditd" && sleep 3
        
        # Generate test message.
        ts=$(date "+%T")
        rlRun "auditctl -m 'test-first'" 0
        rlRun "auditWaitForEvent $ts 'USER' 'test-first' 30 test.msg" 0
        rlRun "sed -i 's/test-first/test-second/' test.msg" 0

        # Configure machine to be both client and server for remote logging.
        rlRun "auditConfig 'tcp_listen_port' '60'" 0
        rlRun "auditConfig 'remote_server' '127.0.0.1' $auditDispatcherConf" 0
        rlRun "auditConfig 'remote_ending_action' 'ignore' $auditDispatcherConf" 0

        # Restart auditd.
        rlRun "service auditd restart && sleep 3" 0
    rlPhaseEnd

    rlPhaseStartTest
    
        # Act as a client and sent message to server via audisp-remote.
        rlRun "( cat test.msg; sleep 5 ) | audisp-remote" 0

        # Check that message arrived and check /var/log/messages too.
        rlRun "auditWaitForSyslogEvent 'audisp-remote.*: Connected to 127.0.0.1' 30" 0
        rlRun "auditWaitForEvent $ts 'USER' 'test-second' 30" 0

    rlPhaseEnd

    rlPhaseStartCleanup
        rlRun "rm -f test.msg" 0
        rlFileRestore
        rlServiceRestore "auditd"        
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
