#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz1693470-libauparse-memory-leak
#   Description: Test for BZ#1693470 (libauparse memory leak)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2019 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE

        rlServiceStop "audit" && sleep 3
        rlFileBackup --clean "/var/log/audit/audit.log"
        rlFileBackup --clean "/etc/audit/"
        rlRun "service auditd restart && sleep 3" 0 "Starting auditd service"

        rlRun "rlImport 'selinux-policy/common'"
        rlSESetTimestamp
        rlRun "gcc parser.c -o parser -Wall -lauparse"
        rlRun "touch testfile" 0
        rlRun "setenforce 0" 0
    rlPhaseEnd

    rlPhaseStartTest
        # Generate some AVCs.
        for X in $(seq 1 1000); do chcon -t games_t testfile; done
        sleep 10
        rlRun -s "valgrind --leak-check=full ./parser /var/log/audit/audit.log" 0-255
        rlAssertGrep "definitely lost: 0 bytes in 0 blocks" $rlRun_LOG
        rlAssertGrep "indirectly lost: 0 bytes in 0 blocks" $rlRun_LOG
        rlAssertGrep "possibly lost: 0 bytes in 0 blocks" $rlRun_LOG
    rlPhaseEnd

    rlPhaseStartCleanup
        rlRun "rm -f parser testfile" 0
        rlRun "setenforce 1" 0
        rlSECheckAVC --ignore 'type=AVC.*denied.*games_t.*'
        rlFileRestore
        rlRun "service auditd restart && sleep 3" 0 "Restarting auditd service"
        rlServiceRestore "auditd"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
