#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz607823-inodes-display-incorrectly-when-listing-rules
#   Description: When listing audit rules, large inode values may not display properly.
#   Author: Eduard Benes <ebenes@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2010 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
    rlPhaseEnd

    rlPhaseStartTest
        INODE=`ls -i /proc/cpuinfo | awk '{print $1}'`
        rlLog "/proc/cpuinfo has inode number $INODE"
        rlRun "/sbin/auditctl -a exit,always -S all -F inode=$INODE"
        rlLog "Example of Bad results: LIST_RULES: exit,always inode=-268435445 (0xf000000b) syscall=all"
        rlLog "Example of Good results: LIST_RULES: exit,always inode=4026531851 (0xf000000b) syscall=all "
        rlLog "/sbin/auditctl -l output: `/sbin/auditctl -l` " 
        rlRun "/sbin/auditctl -l | grep -e 'inode=$INODE'" 0
    rlPhaseEnd

    rlPhaseStartCleanup
        rlRun "auditctl -d exit,always -S all -F inode=`ls -i /proc/cpuinfo | awk '{print $1}'`"
        rlRun "/sbin/auditctl -l"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
