#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz654883-service-auditd-restart-output-in-immutable-mode-is
#   Description: Test for bz654883 (service auditd restart output in immutable mode is)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm $PACKAGE

        if ! [[ -e /var/tmp/bz654883-rebooted ]]; then
            rlServiceStop "auditd" && sleep 3
            rlRun "auditctl -D" 0
            rlRun "auditctl -e 2" 0
            rlRun "service auditd start && sleep 5" 0
        fi

    rlPhaseEnd

    if ! [[ -e /var/tmp/bz654883-rebooted ]]; then
        rlPhaseStartTest

            msg="The audit system is in immutable mode, no rule changes allowed"

            # Start logging.
            tail -f -n 0 /var/log/messages >log &
            pid=$!

            # Trigger 1/2.
            rlRun "service auditd restart && sleep 5" 0

            # Stop logging.
            kill $pid

            # Verify 1/2.
            rlAssertGrep "$msg" log

            # Trigger 2/2.
            rlRun "auditctl -D" 0
            rlRun "auditctl -w /etc/shadow -p wa >out 2>&1" 0-1

            # Verify 2/2.
            rlAssertGrep "$msg" out

            # Print out verbose logs if anything failed
            if ! rlGetTestState; then
                rlRun "cat log"
                rlRun "cat out"
            fi
        rlPhaseEnd
    fi

    if ! [[ -e /var/tmp/bz654883-rebooted ]]; then
        touch /var/tmp/bz654883-rebooted
        rhts-reboot
    fi

    rlPhaseStartCleanup

        rlRun "rm -f out log" 0 "Removing tmp file"
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
