#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz810749-audit-allows-tcp-max-per-addr-1-connections
#   Description: Test for BZ#810749 (audit allows tcp_max_per_addr + 1 connections)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2014 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "/var/log/audit/audit.log"
	! rlIsRHEL 5 6 && rlRun "systemctl reset-failed auditd"
        rlServiceStop "auditd" && sleep 3
    rlPhaseEnd

    rlPhaseStartTest
        rlRun "sed -i '/tcp_listen_port/d' /etc/audit/auditd.conf" 0 \
            "Removing tcp_listen_port from configuration"
        rlRun "sed -i '/tcp_max_per_addr/d' /etc/audit/auditd.conf" 0 \
            "Removing tcp_max_per_addr from configuration"
        rlRun "echo 'tcp_listen_port = 60' >> /etc/audit/auditd.conf" 0 \
            "Adding tcp_listen_port = 60 to configuration"
        rlRun "echo 'tcp_max_per_addr = 1' >> /etc/audit/auditd.conf" 0 \
            "Adding tcp_max_per_addr = 1 to configuration"
        rlServiceStart auditd
	sleep 5
	rlRun "echo '' > /var/log/audit/audit.log" 0 "Emptying audit log"
        rlRun "(echo 'a' ; sleep 20 ) | nc 127.0.0.1 60 -p 10000 &" 0 \
	    "Making connection 1"
        sleep 5
        rlRun "(echo 'a' ; sleep 20 ) | nc 127.0.0.1 60 -p 10001 &" 0 \
	    "Making connection 2"
	rlRun "sleep 30" 0 "Waiting for connections to terminate"
        rlRun "ausearch -m DAEMON_ACCEPT -ts recent | grep port=10000" 0 \
	    "Checking that connection 1 was seen"
        rlRun "ausearch -m DAEMON_ACCEPT -ts recent | grep port=10001" 0 \
	    "Checking that connection 2 was seen"
        rlRun "ausearch -m DAEMON_ACCEPT -ts recent | grep port=10000 | \
               grep res=success" 0 \
	    "Checking that connection 1 was permitted"
        rlRun "ausearch -m DAEMON_ACCEPT -ts recent | grep port=10001 | \
               grep res=no" 0 \
	    "Checking that connection 2 was rejected"
    rlPhaseEnd

    rlPhaseStartCleanup
        rlFileRestore
        rlServiceRestore "auditd"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
