#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz952295-Add-msgtype-to-user-filter
#   Description: Test for BZ#952295 (Add msgtype to user filter)
#   Author: Karel Srot <ksrot@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2014 Red Hat, Inc.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

USER=bz952295user
LOG=/var/log/audit/audit.log

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "/var/log/audit/audit.log"       
        rlServiceStop "auditd" && sleep 3
        rlRun "sed -i 's/^flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
        rlRun "sed -i 's/^freq.*/freq = 1/' /etc/audit/auditd.conf" 0
        rlRun "TmpDir=\$(mktemp -d)" 0 "Creating tmp directory"
        rlRun "pushd $TmpDir"
	rlRun "useradd -m $USER"
	rlFileBackup $LOG
	rlRun "service auditd start && sleep 5" 0        
	rlRun "> $LOG"
    rlPhaseEnd

    rlPhaseStartTest
	ts=$(date "+%T")
        rlRun "su - $USER -c 'who'" 0 "Generate USER_START event"
        # On slower system it might take a few seconds to populate events.
        sleep 10
	rlRun "ausearch -m USER_START -ts $ts > log1"
	rlAssertGrep "session_open" log1 -E
	rlAssertGrep "acct.*$USER" log1 -E
	sleep 1
	rlRun "> $LOG"
	rlRun "auditctl -a never,user -F msgtype=USER_START"
	ts=$(date "+%T")
        rlRun "su - $USER -c 'who'" 0 "Generate USER_START event"
        # On slower system it might take a few seconds to populate events.
        sleep 10
	rlRun "ausearch -m USER_START -ts $ts  > log2" 1
	rlAssertNotGrep "session_open" log2 -E
	rlAssertNotGrep "acct.*$USER" log2 -E
    rlPhaseEnd

    rlPhaseStartCleanup
        rlRun "auditctl -D" 0
	rlRun "userdel -r $USER"
        rlRun "popd"
        rlRun "rm -r $TmpDir" 0 "Removing tmp directory"
	rlFileRestore
	rlServiceRestore auditd
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
