#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/ambient-capabilities
#   Description: Test for ambient capabilities
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2010 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm $PACKAGE || rlDie
	      rlRun "gcc -o ambient_test ambient_test.c -lcap-ng" || rlDie
        rlRun "setcap cap_net_raw,cap_net_admin,cap_sys_nice+p ambient_test" 0

        if rlIsRHEL "<=7"; then
            CAP="0000001fffffffff"
            [ "$(uname -m)" == "aarch64" ] && CAP="0000003fffffffff"
        elif rlIsRHEL "<=8.3"; then
            CAP="0000003fffffffff"
        elif rlIsRHEL "8.4"; then
            CAP="000000ffffffffff"
        else
            CAP="000001ffffffffff"
        fi
        
    rlPhaseEnd

    rlPhaseStartTest 

        rlRun -s "cat /proc/$$/status | grep '^Cap' | tr -d '\t'"
        rlAssertGrep "CapInh:0000000000000000" $rlRun_LOG
        rlAssertGrep "CapPrm:${CAP}" $rlRun_LOG
        rlAssertGrep "CapEff:${CAP}" $rlRun_LOG
        rlAssertGrep "CapBnd:${CAP}" $rlRun_LOG
        rlAssertGrep "CapAmb:0000000000000000" $rlRun_LOG

        rlRun -s "./ambient_test /bin/bash -c 'cat /proc/\$\$/status | grep ^Cap | tr -d \"\\t\"'"
        rlAssertGrep "CapInh:0000000000803000" $rlRun_LOG
        rlAssertGrep "CapPrm:${CAP}" $rlRun_LOG
        rlAssertGrep "CapEff:${CAP}" $rlRun_LOG
        rlAssertGrep "CapBnd:${CAP}" $rlRun_LOG
        rlAssertGrep "CapAmb:0000000000803000" $rlRun_LOG

    rlPhaseEnd

    rlPhaseStartCleanup        
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
