#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/audit-log-update
#   Description: Automatic update of testing audit logs
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2017 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

SAMPLES_URL="http://nest.test.redhat.com/mnt/qa/scratch/omoris/audit-testing/"
SAMPLES_DIR="/mnt/qa/scratch/omoris/audit-testing/"

RECIPIENT="omoris@redhat.com"

MAILX=s-nail
if  rlIsRHEL '<9'; then
    MAILX=mailx
fi

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "audit" || rlDie
        rlAssertRpm "${MAILX}" || rlDie
        
        rlRun "test_dir=$(mktemp -d)" 0
        rlRun "pushd $test_dir" 0
        
        [ -z "$SAMPLE_LOG" ] && rlRun "wget $SAMPLES_URL/audit.log.$(uname -m) -O audit.log" 0
        
    rlPhaseEnd

    rlPhaseStartTest
    
        new_types_count=0
        actual_log=${ACTUAL_LOG:-"/var/log/audit/audit.log"}
        sample_log=${SAMPLE_LOG:-"audit.log"}
        types=$(ausearch -m 2>&1  | \
                    tr ' ' '\n'   | \
                    grep -E '[A-Z]' | \
                    grep -Ev 'ALL|Argument|Valid|EOE|TRUSTED_APP|KERNEL_OTHER' | \
                    grep -Ev 'USER_MAC_CONFIG_CHANGE' | \
                    sort)
            # USER_MAC_CONFIG_CHANGE must be an exception as it does not meet the field specification
        count_all=$(echo $types | wc -w)
        count_log=0

        rlLog "Actual log is $actual_log"
        rlLog "Sample log is $sample_log"
        
        for t in $types; do
	    ausearch -if $actual_log -m $t --just-one --raw 2>/dev/null 1>&2
            actual_hit=$?

	    ausearch -if $sample_log -m $t --just-one --raw 2>/dev/null 1>&2
            sample_hit=$?

	    if [ $actual_hit -eq 0 ] && [ $sample_hit -ne 0 ] ; then
		ausearch -if $actual_log -m $t --just-one --raw 2>/dev/null >> $sample_log
                rlLog "$t event found, appended to sample log"
		new_types_count=$[$new_types_count + 1]
	    fi

            [ $sample_hit -eq 0 ] && count_log=$[$count_log + 1]
        done

        rlLog "$new_types_count new types found"
        rlLog "Sample log contains $count_log of $count_all types"

        for t in $types; do
	    ausearch -if $sample_log -m $t >/dev/null 2>&1 || rlLog "Missing $t"
        done

        if [ $new_types_count -gt 0 ]; then
            if [ "$DEBUG_MODE" == "1" ]; then
                rlLog "Test is in debug mode, samples not updated"
            else
                rlCheckMountQa && { qa_mounted=1; rlRun "umount /mnt/qa" 0; }
                rlMountQa "rw"
                rlRun "cp audit.log $SAMPLES_DIR/audit.log.$(uname -m)" 0 "Samples updated"
                
                subject="Automatic update of audit testing logs"
                recipient=$RECIPIENT
                body+="Audit testing log on $(uname -m) was updated:\n\n"
                body+="  * $new_types_count new types found\n"
                body+="  * sample log contains $count_log of $count_all types\n\n"
                body+="See sample at $SAMPLES_URL/$sample_log}.\n"
                
                echo -e "$body" | "${MAILX}" -s "$subject" "$recipient"
                
                rlRun "umount /mnt/qa" 0
                [ "$qa_mounted" == "1" ] && rlMountQa
            fi
        fi
        
    rlPhaseEnd

    rlPhaseStartCleanup

        rlRun "popd" 0
        rlRun "rm -rf $test_dir" 0

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
