#!/bin/bash
# runtest.sh - audit-updates-rhel53 - Bugzilla(s) 446080
# Author: Eduard Benes <ebenes@redhat.com>
# Location: /CoreOS/audit/audit-updates-rhel53/runtest.sh

# Description: Functionality tests for audit packages that have been updated to the newer upstream version 1.7.7

# Copyright (c) 2008 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE=audit

rlJournalStart

    rlPhaseStartSetup
        rlAssertRpm ${PACKAGE}
        rlRun "rlImport audit/testing" || rlDie
        rlFileBackup --clean "/var/log/audit/" 
        rlFileBackup --clean "$auditDaemonConf" 
        rlRun "auditConfig 'flush' 'incremental'" 0
        rlRun "auditConfig 'freq' '1'" 0
        rlServiceStop "auditd"
        rlRun "service auditd start && sleep 3" 0
        rlRun 'auditctl -D' 0
    rlPhaseEnd

    rlPhaseStartTest "multiple-keys"
        ts=`date +%T`
        rlLog "adding rule with 2 defined keys (key1, key2) "
        rlRun 'auditctl -a always,exit -S all -F path=/etc/motd -k key1 -k key2' 0 
        rlRun "cat /etc/motd" 0        
        rlRun "auditctl -l | grep '\-a always,exit -S all -F path=/etc/motd -F key=key1 -F key=key2'"
        rlRun "auditctl -D" 0
        rlRun "auditWaitForEvent $ts 'SYSCALL' '' '30' '' --key key1 -i"
        rlRun "auditWaitForEvent $ts 'SYSCALL' '' '30' '' --key key2 -i"
        rlRun "aureport --start today --key -i -ts $ts"
        rlRun "aureport --start today --key --summary -i -ts $ts"
    rlPhaseEnd

    rlPhaseStartTest "illegal-auditctl"
        rlLog "checkubg that illegal auditctl rules are not accepted"
        rlRun 'auditctl -a always,user -S openat -F filetype=file' 255 
        rlRun 'auditctl -a always,user -S openat -F ppid=1' 255
        rlRun 'auditctl -a always,exit -S openat -F dir=/etc -k test'
    rlPhaseEnd


    rlPhaseStartTest "illegal-value-aureport"
        rlLog "illegal value for aureport, should produce an error"
        rlRun 'aureport --start today PM' 1
    rlPhaseEnd

    rlPhaseStartTest "interpretate-i386-calls-on-x86_64"
        rlLog "ausearch interpretation of i386 syscalls on an x86_64"
        tmpf1=`mktemp` || log "failed to create tempfile"
        tmpf2=`mktemp` || log "failed to create tempfile"
        cat > $tmpf1 << _EOT
type=SYSCALL msg=audit(1224864719.162:10038): arch=40000003 syscall=102 success=yes exit=5 a0=a a1=bfcc1f80 a2=25b0c4 a3=0 items=0 ppid=1 pid=11761 auid=0 uid=0 gid=0 euid=0 suid=0 fsuid=0 egid=0 sgid=0 fsgid=0 tty=(none) ses=1 comm="prelude-manager" exe="/usr/bin/prelude-manager" subj=system_u:system_r:prelude_t:s0-s15:c0.c1023 key=(null)
_EOT
        rlLog "syscall value should be should be a socketcall(recv)"
        cat $tmpf1 | ausearch -i | tee $tmpf2
        rlRun "grep 'syscall=socketcall(recv)' $tmpf2 "
        rm -rf $tmpf1 $tmpf2
    rlPhaseEnd

    rlPhaseStartTest "ausyscall-basic-functionality"
        rlLog 'Functionality test for ausyscall program'
        if rlIsRHEL "7"; then
            ARCH_LIST="i386 x86_64 ppc ppc64 ppc64le ia64 s390 s390x"
        else
            ARCH_LIST="i386 x86_64 ppc64le s390x"
        fi
        for TEST_ARCH in $ARCH_LIST ; do 
            rlLog "Testing architecture $TEST_ARCH"
            CALL_NUM=`ausyscall $TEST_ARCH --dump | grep -v "Using" | wc -l`
            rlLog "Number of syscalls for $TEST_ARCH: $CALL_NUM"
            rlRun "ausyscall $TEST_ARCH dup "
            rlRun "ausyscall $TEST_ARCH dup --exact "
            CALL_SUM=0
            for i in `ausyscall $TEST_ARCH --dump | cut -f1 | grep -v "Using"` ; do
                R=`ausyscall $TEST_ARCH $i --exact` 
                L=`ausyscall $TEST_ARCH $R --exact`
                [ "$L" == "$i" ] && let "CALL_SUM += 1" 
            done
            rlAssertEquals 'Mapping syscall names to numbers and vice versa' $CALL_SUM $CALL_NUM 
            rlLog "Finished subtest for arch $TEST_ARCH"
        done
    rlPhaseEnd 

    rlPhaseStartCleanup
        rlRun "auditctl -D" 0

        if rlIsRHEL '>=8'; then
            # Let's kill audit to get rid of netlink sockets piled up.
            i=0
            while pidof auditd; do
                i=$((i+1))
                rlRun "pkill auditd" 0 "Kil auditd - attempt #${i}"
                sleep 3
            done
            rlRun "auditctl --reset_backlog_wait_time_actual >/dev/null ||:"
        fi

        rlFileRestore
        rlServiceRestore "auditd"
    rlPhaseEnd

rlJournalPrintText
rlJournalEnd


