#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/aureport-smoke
#   Description: Test sanity of aureport
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2017 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

package="audit"
audit_log="/var/log/audit/audit.log"
test_log="test.log"

# Since RHEL8 there is no separate dispatcher.
if rlIsRHEL "<8"; then
    DISPATCHER_DIR="/etc/audisp"
else    
    DISPATCHER_DIR="/etc/audit"
fi

function check_events() {
    local aureport_output="$1"
    local log="$2"
    local expected="$3"
    local missing=""
    local event_col=$(sed -n '4p' $aureport_output | tr ' ' '\n' | nl | grep event | awk '{print $1}')
    echo "XXXX $event_col XXXX"
    local events=$(tail -n +6 $aureport_output | awk "{print \$$event_col}")
    local found=0
    local rc=0

    echo "events: $events"
    for e in $events; do
        ausearch -a $e -if $log || missing+="$e "
        found=$[$found+1]
    done

    if [ "$found" != "$expected" ]; then
        rlLog "Incorrect number of events ($expected expected, $found found)"
        rc=1
    fi
    
    if [ "$missing" != "" ]; then
        rlLog "Missing events: $missing"
        rc=2
    else
        rlLog "All reported events found"
    fi

    return $rc
}

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "audit" || rlDie
        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "${DISPATCHER_DIR}/plugins.d/syslog.conf"
        rlFileBackup "/var/log/audit/audit.log"

        rlServiceStop "auditd" && sleep 3
        
        rlRun "sed -i -e 's/name_format = .*$/name_format = HOSTNAME/' /etc/audit/auditd.conf"
        rlRun "sed -i 's/^active.*/active = yes/' ${DISPATCHER_DIR}/plugins.d/syslog.conf" 0
        rlRun "sed -i 's/^flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
        rlRun "sed -i 's/^freq.*/freq = 1/' /etc/audit/auditd.conf" 0
        
        rlRun "service auditd start && sleep 5" 0
        rlRun "auditctl -D"
        
    rlPhaseEnd

    rlPhaseStartTest "Sanity" 

        rlRun "aureport --start today PM" 1 
        rlRun "auditctl -a always,exit -F arch=b64 -S openat -k key1"
        rlRun "auditctl -a always,exit -F arch=b64 -S execve -k key2"
        cat /etc/motd
        sleep 60
        rlRun -s "aureport --start today --key " 0
        rlRun "auditctl -D"
        rlAssertGrep "key1" $rlRun_LOG
        rlAssertGrep "key2" $rlRun_LOG
        check_events $rlRun_LOG $audit_log 1
        rlRun -s "aureport --start today --key --summary" 0
        rlAssertGrep "key1" $rlRun_LOG
        rlAssertGrep "key2" $rlRun_LOG

    rlPhaseEnd

    rlPhaseStartTest "Options"

        # BZ#1511606 will never be resolved on RHEL-6.
        if rlIsRHEL ">=7"; then
            rlRun -s "aureport -a -if $test_log" 0 "AVC report"
            rlRun "check_events $rlRun_LOG $test_log 1" 0 "Checking report"
            rlAssertGrep "bash user_u:user_r:user_t:s0 59 file execute unconfined_u:object_r:user_tmp_t:s0 denied" $rlRun_LOG
        fi

        rlRun -s "aureport -au -if $test_log" 0 "Authentication report"
        rlRun "check_events $rlRun_LOG $test_log 2" 0 "Checking report"
        rlAssertGrep "root 10.43.21.93 ssh /usr/sbin/sshd yes" $rlRun_LOG
        rlAssertGrep "testuser 10.43.21.93 ssh /usr/sbin/sshd yes" $rlRun_LOG

        rlRun -s "aureport --comm -if $test_log" 0 "Commands report"
        rlRun "check_events $rlRun_LOG $test_log 4" 0 "Checking report"
        rlAssertGrep "systemd ? ? -1" $rlRun_LOG
        rlAssertGrep "setsebool pts0 ? 0" $rlRun_LOG

        rlRun -s "aureport -c -if $test_log" 0 "Config change report"
        rlRun "check_events $rlRun_LOG $test_log 2" 0 "Checking report"
        rlAssertGrep "CONFIG_CHANGE 0 yes" $rlRun_LOG
        rlAssertGrep "CONFIG_CHANGE 0 yes" $rlRun_LOG

        rlRun -s "aureport -cr -if $test_log" 0 "Crypto report"
        rlRun "check_events $rlRun_LOG $test_log 2" 0 "Checking report"
        rlAssertGrep "CRYPTO_KEY_USER yes" $rlRun_LOG
        rlAssertGrep "CRYPTO_SESSION yes" $rlRun_LOG
        
        # rlRun -s "aureport -e -if $test_log" 0 "Event report"
        # rlRun -s "aureport -f -if $test_log" 0 "File name report"
        # rlRun -s "aureport --failed -if $test_log" 0 "Failed report"
        # rlRun -s "aureport --success -if $test_log" 0 "Successful report"
        
        rlRun -s "aureport -h -if $test_log" 0 "Remote host report"
        rlRun "check_events $rlRun_LOG $test_log 8" 0 "Checking report"
        rlAssertGrep "qeos-72.lab.eng.rdu2.redhat.com 0 0 2468" $rlRun_LOG
        rlAssertGrep "10.43.21.39 0 -1 88" $rlRun_LOG
        
        # rlRun -s "aureport --integrity -if $test_log" 0 "Integrity event report"

        # BZ#1448526 reproducer (second assert fails with auid -1)
        # Bug is fixed since RHEL7
        rlRun -s "aureport -l -if $test_log" 0 "Login report"
        rlRun "check_events $rlRun_LOG $test_log 2" 0 "Checking report"
        rlAssertGrep "(unknown) 10.43.21.39 ssh /usr/sbin/sshd no" $rlRun_LOG
        if rlIsRHEL ">=7"; then
            rlAssertGrep "0 10.43.21.39 /dev/pts/0 /usr/sbin/sshd yes" $rlRun_LOG
        else
            rlAssertGrep "10.43.21.39 /dev/pts/0 /usr/sbin/sshd yes" $rlRun_LOG
        fi

        # rlRun -s "aureport -k -if $test_log" 0 "Key report"
        # rlRun -s "aureport -m -if $test_log" 0 "Modification to accounts report"
        # rlRun -s "aureport -ma -if $test_log" 0 "MAC report"
        # rlRun -s "aureport -p -if $test_log" 0 "PID report"
        # rlRun -s "aureport -n -if $test_log" 0 "Anomaly report"
        # rlRun -s "aureport -s -if $test_log" 0 "Syscall report"
        # rlRun -s "aureport -u -if $test_log" 0 "Username report"
        # rlRun -s "aureport --virt -if $test_log" 0 "Virtualization report"

        rlRun -s "aureport -x -if $test_log" 0 "Executable report"
        rlRun "check_events $rlRun_LOG $test_log 10" 0 "Checking report"
        rlAssertGrep "/usr/sbin/setfiles ? ? 0" $rlRun_LOG

        # rlRun -s "aureport -tm -if $test_log" 0 "Terminal name report"

        rlRun -s "aureport --tty -if $test_log" 0 "TTY keystrokes report"
        rlRun "check_events $rlRun_LOG $test_log 2" 0 "Checking report"
        rlAssertGrep "0 ? 14 ? \"ls\"" $rlRun_LOG
        rlAssertGrep "0 ? 14 bash" $rlRun_LOG
        
	# -nc,--no-config			Don't include config events
	# --node <node name>		Only events from a specific node
	# -r,--response			Response to anomaly report
	# --summary			sorted totals for main object in report
	# -t,--log			Log time range report

    rlPhaseEnd

    rlPhaseStartCleanup

        rlFileRestore
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
