#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/bz1985630-audit-shipped-rules
#   Description: Test audit shipped sample rules
#   Author: Sergio Correia <scorreia@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2021 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

COMPLETED_FLAG=/audit-shipped-rules-done
unset REBOOT_DONE

reboot_test() {
    # Return the configuration to the default state, so that it will
    # not interefere in further tests -- I am looking at you,
    # immutable flag.
    if [ ! -e "${COMPLETED_FLAG}" ]; then
        touch "${COMPLETED_FLAG}"
        rhts-reboot
    fi
}

cleanup_pre_reboot() {
        rlPhaseStartTest "Cleanup pre-reboot"

        rlRun "service auditd stop && sleep 5"
        rlRun "rlFileRestore"

        rlRun "auditctl -s > final_state" 0 "Show final audit status"
        rlFileSubmit "final_state"

        rlRun "auditctl -l > final_rules" 0 "Show final audit rules"
        rlFileSubmit "final_rules"
    rlPhaseEnd
}

cleanup_post_reboot() {
    rlPhaseStartCleanup
        rlRun "auditctl -s > completed_state" 0 "Show completed audit status"
        rlFileSubmit "completed_state"

        rlRun "auditctl -l > completed_rules" 0 "Show completed audit rules"
        rlFileSubmit "completed_rules"
    rlPhaseEnd
}

tests() {
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE

        # Backup existing rules and remove them.
        rlRun "rlFileBackup --clean /etc/audit/rules.d/"
        rlRun "rm -f /etc/audit/rules.d/*"

        # Let's set up the default rules, while also saving the initial
        # ones, so that we can compare if something bad happens.
        rlRun "auditctl -s > initial_state" 0 "Show initial audit status"
        rlFileSubmit "initial_state"

        rlRun "auditctl -l > initial_rules" 0 "Show initial audit rules"
        rlFileSubmit "initial_rules"

        # Copy shipped sample rules.
        rlRun "cd /usr/share/audit/sample-rules/ && cp 10-base-config.rules 11-loginuid.rules 30-ospp*.rules 43-module-load.rules /etc/audit/rules.d/"

        rlRun "auditctl -s > post_setup_state" 0 "Show post-setup audit status"
        rlFileSubmit "post_setup_state"

        rlRun "auditctl -l > post_setup_rules" 0 "Show post-setup audit rules"
        rlFileSubmit "post_setup_rules"

        # Restart auditd.
        rlRun "service auditd stop && sleep 5"
        rlRun "systemctl start auditd"

        rlRun -s "systemctl --no-pager status auditd"
        rlAssertNotGrep "FAILURE" $rlRun_LOG
        rm -f $rlRun_LOG
        rlRun "augenrules --load"
    rlPhaseEnd
}

rlJournalStart

if [ -e "${COMPLETED_FLAG}" ]; then
    rm -f "${COMPLETED_FLAG}"
    export REBOOT_DONE=1
fi

if [ -z "${REBOOT_DONE}" ]; then
    tests
    cleanup_pre_reboot
    reboot_test
else
    cleanup_post_reboot
fi

rlJournalPrintText
rlJournalEnd
