#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/filter
#   Description: Test for audit filter (auditctl -F)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2010 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm $PACKAGE || rlDie
        rlRun "rlImport audit/testing" || rlDie        
        rlRun "gcc do_open.c -std=c99  -o do_open"
        rlAssertExists "do_open" || rlDie

        rlFileBackup --clean "$auditDaemonConf"
        rlFileBackup --clean "/var/log/audit/"
        rlServiceStop "auditd" && sleep 3
        rlRun "auditConfig 'flush' 'INCREMENTAL'" 0
        rlRun "auditConfig 'freq' '1'" 0
        rlServiceStart "auditd" && sleep 3

    rlPhaseEnd

    if [ "$(uname -m)" != "aarch64" ]; then
    rlPhaseStartTest "-F pid/exe"

        # Setup.
        rlRun "cp do_open do_open_exe" 0
        rlRun "cp do_open do_open_pid" 0

        BIN_PID=$(pwd)/do_open_pid
        BIN_EXE=$(pwd)/do_open_exe
        TMP_PID=$(mktemp)
        TMP_EXE=$(mktemp)
        KEY_PID=$(echo $TMP_PID | awk -F '.' '{print $2}')
        KEY_EXE=$(echo $TMP_EXE | awk -F '.' '{print $2}')
        
        # Test.
        SLEEP=10
        COUNT=5
        rlRun "$BIN_EXE $SLEEP $COUNT $TMP_EXE &" 0 
        rlRun "$BIN_PID $SLEEP $COUNT $TMP_PID &" 0 
        
        PID=$!
        rlRun "auditctl -a always,exit -F pid=$PID     -S open -k $KEY_PID" 0
        rlRun "auditctl -a always,exit -F exe=$BIN_EXE -S open -k $KEY_EXE" 0
        rlRun "auditctl -l"

        rlRun "sleep $[$SLEEP*2]" 0

        OPTIONS="-m SYSCALL -sc open --raw -ts recent"
        rlRun "ausearch $OPTIONS -k $KEY_PID | tee $KEY_PID" 0
        rlRun "ausearch $OPTIONS -k $KEY_EXE | tee $KEY_EXE" 0

        COUNT_PID=$(cat $KEY_PID | wc -l)
        COUNT_EXE=$(cat $KEY_EXE | wc -l)
        rlAssertEquals "Checking number of events" "$COUNT_PID" "$COUNT_EXE"

        # Cleanup.
        rlRun "rm -f $TMP $KEY_PID $KEY_EXE $BIN_EXE $BIN_PID" 0
        rlRun "auditctl -D" 0
       
    rlPhaseEnd
    fi

    if rlIsRHEL ">=8.1"; then
    rlPhaseStartTest "-F saddr_fam"
            
        # SETUP
        # =====

        # Valid rules.
        rlRun "auditctl -a always,exit -S all -F saddr_fam=2 -F key=inet-sockaddr" 0
        rlRun "auditctl -a always,exit -S all -F saddr_fam=16 -F key=netlink-sockaddr" 0

        # Invalid rule.
        rlRun "auditctl -a always,exit -S all -F saddr_fam=100 -F key=invalid-sockaddr" 1-255
        
        # List rules for debugging purposes.
        rlRun "auditctl -l"

        # Start listening on TCP port.
        rlRun "nc -l 127.0.0.1 24242 >nc.out &" 0
        nc_pid=$!

        # Test mark for ausearch.
        ts=$(date "+%T")
        
        # TEST
        # ====

        # Trigger AF_INET SOCKADDR event.
        rlRun "echo 'hi' | nc 127.0.0.1 24242" 0

        # Trigger AF_NETLINK SOCKADDR event.
        rlRun "auditctl -s" 0

        # VERIFY
        # ======

        # Verify that connection TCP port was successfull.
        rlAssertGrep "hi" nc.out

        # Check AF_INET SOCKADDR event.
        msg="saddr={ saddr_fam=inet laddr=127.0.0.1 lport=24242 }"
        rlRun "auditWaitForEvent $ts 'SOCKADDR' '$msg' 30 inet.msg -i -k inet-sockaddr" 0
        rlRun "test $(cat inet.msg | wc -l) -ge 1" 0

        # Check AF_NETLINK SOCKADDR event.
        msg="saddr={ saddr_fam=netlink nlnk-fam=16 nlnk-pid=0 }"
        rlRun "auditWaitForEvent $ts 'SOCKADDR' '$msg' 30 netlink.msg -i -k netlink-sockaddr" 0
        rlRun "test $(cat netlink.msg | wc -l) -ge 1" 0

        # CLEAN-UP
        # ========

        rlWait $nc_pid
        rlRun "rm -f *.msg" 0
        rlRun "auditctl -D" 0
       
    rlPhaseEnd
    fi

    rlPhaseStartCleanup
    
        rlFileRestore
        if rlIsRHEL '>=8'; then
            rlRun "auditctl --reset_backlog_wait_time_actual >/dev/null ||:"
        fi
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
