#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/initscript
#   Description: Make sure audit complies to guidelines for SysV-style Initscripts.
#   Author: Eduard Benes <ebenes@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2010 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"
SERVICE="auditd"

rlJournalStart

    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlFileBackup --clean /etc/audit /etc/audisp
        rlRun 'adduser testuserqa'
        rlServiceStop $SERVICE
    rlPhaseEnd

    rlPhaseStartTest
        rlLog ">>>>>>>>> service start"
          rlRun "service $SERVICE start" 0 " Service must start without problem"
          rlRun "service $SERVICE status" 0 " Then Status command "
          rlRun "service $SERVICE start" 0 " Already started service "
          rlRun "service $SERVICE status" 0 " Again status command "

        rlLog ">>>>>>>>> service restart"
          rlRun "service $SERVICE restart" 0 " Restarting of service"
          rlRun "service $SERVICE status" 0 " Status command "

        rlLog ">>>>>>>>> service stop"
          rlRun "service $SERVICE stop" 0 " Stopping service "
          rlRun "service $SERVICE status" 3 " Status of stopped service "
          rlRun "service $SERVICE stop" 0 " Stopping service again "
          rlRun "service $SERVICE status" 3 " Status of stopped service "

        if rlIsRHEL 6; then
            rlLog ">>>>>>>>> pid file"
              service $SERVICE start
              rlAssertExists "/var/run/$SERVICE.pid" "Pid file /var/run/$SERVICE.pid must exist"
              service $SERVICE stop
              rlRun "echo 666666 > /var/run/$SERVICE.pid"
              rlRun "service $SERVICE status" 1 " Existing pid file, but service not started "
              service $SERVICE stop
              rm -fv /var/run/$SERVICE.pid

            rlLog ">>>>>>>>> lock file"
              service $SERVICE start
              rlAssertExists "/var/lock/subsys/$SERVICE" "Lock file /var/lock/subsys/$SERVICE must exist"
              service $SERVICE stop    
              rlRun "touch /var/lock/subsys/$SERVICE"  
              rlRun "service $SERVICE status" 2 " Existing lock file, but service not started " 
              service $SERVICE stop    
              rm -fv /var/lock/subsys/$SERVICE 
        fi

        rlLog ">>>>>>>>> insufficient rights"
          rlRun "service $SERVICE start " 0 " Starting service for restarting under non-priv. user."
          rlRun "su testuserqa -c 'service $SERVICE restart'" 0,4 "Insufficient rights, restarting service under non-privileged user must fail."              

        rlLog ">>>>>>>>> operations"
          rlRun "service $SERVICE start" 0 " Service have to implement start function."
          rlRun "service $SERVICE restart" 0 " Service have to implement restart function."
          rlRun "service $SERVICE status" 0 " Service have to implement status function."
          rlRun "service $SERVICE condrestart" 0 " Service have to implement condrestart function."
          # according to lnykryn the try-restart should fail for auditd, the user shoul use condrestart instead
          if rlIsRHEL 6; then
            rlRun "service $SERVICE try-restart" 0 " Service have to implement try-restart function."
          else
            rlRun "service $SERVICE try-restart" 4 " Service have to implement try-restart function."
          fi
          rlRun "service $SERVICE reload" 0 " Service have to implement reload function."
          rlRun "service $SERVICE force-reload" 0 " Service have to implement force-reload function."

        rlLog ">>>>>>>>> nonexist operations"
          rlRun "service $SERVICE noexistop" 2,3 " Testing proper return code when nonexisting function."

        rlLog ">>>>>>>>> invalid arguments" 
          rlRun "service $SERVICE" 2,3 " When no arguments added to service, it must fail with proper return code."
    rlPhaseEnd

    rlPhaseStartTest "Test service reload effect on plugins"
        sleep 10 # wait for a while to recover from previous tests
        rlRun "sed -i 's/^remote_server.*/remote_server = $HOSTNAME/' /etc/audisp/audisp-remote.conf"
        rlRun "sed -i 's/^active.*/active = no/' /etc/audisp/plugins.d/au-remote.conf"
        rlRun "service $SERVICE restart"
        sleep 10
        rlRun "ps -fC audisp-remote" 1
        rlRun "sed -i 's/^active.*/active = yes/' /etc/audisp/plugins.d/au-remote.conf"
        rlRun "service $SERVICE reload"
        sleep 10
        rlRun "ps -fC audisp-remote" 0
        ps -ef |grep -v grep |grep audi
        rlRun "sed -i 's/^active.*/active = no/' /etc/audisp/plugins.d/au-remote.conf"
        rlRun "service $SERVICE reload"
        sleep 30 # it takes more time to stop the remote logger
        rlRun "ps -fC audisp-remote" 1
        ps -ef |grep -v grep |grep audi
    rlPhaseEnd

    rlPhaseStartCleanup
        rlFileRestore
        rlRun 'userdel -rf testuserqa'
        rlServiceRestore $SERVICE
   rlPhaseEnd
rlJournalPrintText
rlJournalEnd

