#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/multikey
#   Description: Smoke test for multikey support (added in 2.6.6)
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "$PACKAGE"
        rlFileBackup --clean "/etc/audit/auditd.conf"
        rlFileBackup --clean "/var/log/audit/audit.log"  
        rlServiceStop "auditd" && sleep 3
        rlRun "sed -i \"s/^flush.*/flush = incremental/\" /etc/audit/auditd.conf" 0
        rlRun "sed -i \"s/^freq.*/freq = 1/\" /etc/audit/auditd.conf" 0
        rlRun "service auditd start && sleep 5" 0
        rlRun "auditctl -D" 0
        
    rlPhaseEnd

    rlPhaseStartTest

        # Setup.
        temp_s1=$(mktemp)
        temp_s2=$(mktemp)
        temp_m12=$(mktemp)
        temp_m23=$(mktemp)
        temp_m34=$(mktemp)

        rlRun "auditctl -w $temp_s1 -k singlekey1" 0
        rlRun "auditctl -w $temp_s2 -k singlekey2" 0
        
        rlRun "auditctl -w $temp_m12 -k multikey1 -k multikey2" 0
        rlRun "auditctl -w $temp_m23 -k multikey2 -k multikey3" 0
        rlRun "auditctl -w $temp_m34 -k multikey3 -k multikey4" 0

        # Trigger
        rlRun "rm -f $temp_s1" 0 "Trigger singlekey1 event"
        rlRun "rm -f $temp_s2" 0 "Trigger singlekey2 event"
        rlRun "rm -f $temp_m12" 0 "Trigger multikey1 and multikey2 event"
        rlRun "rm -f $temp_m23" 0 "Trigger multikey2 and multikey3 event"
        rlRun "rm -f $temp_m34" 0 "Trigger multikey3 and multikey4 event"
        
        # Verify
        rlAssertEquals "Checking the number of rules" "$(auditctl -l | wc -l)" "5"
        rlRun "sleep 10" 0
        
        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k singlekey1" 0
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "1"
        
        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k singlekey2" 0
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "1"
        
        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k singlekey" 0
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "2"

        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k singlekey --word" 1
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "0"

        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k multikey1" 0
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "1"

        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k multikey2" 0
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "2"

        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k multikey3" 0
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "2"

        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k multikey4" 0
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "1"

        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k multikey1" 0
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "1"

        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k multikey2" 0
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "2"
        
        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k multikey3" 0
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "2"

        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k multikey4" 0
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "1"

        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k multikey" 0
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "3"

        rlRun -s "ausearch -m syscall -sc unlinkat --raw -k multikey --word" 1 
        rlAssertEquals "Checking the number of events" "$(cat $rlRun_LOG | grep SYSCALL | wc -l)" "0"

    rlPhaseEnd
        

    rlPhaseStartCleanup

        rlRun "auditctl -D" 0
        rlRun "service auditd stop && sleep 5" 0
        rlFileRestore
        if rlIsRHEL '>=8'; then
            rlRun "auditctl --reset_backlog_wait_time_actual >/dev/null ||:"
        fi
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
