#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/options-log-format
#   Description: Sanity of log_format options of auditd
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

# Since RHEL8 there is no seprate dispatcher.
if rlIsRHEL "<8"; then
    DISPATCHER_DIR="/etc/audisp"
else    
    DISPATCHER_DIR="/etc/audit"
fi

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm "$PACKAGE"

        rlFileBackup "/etc/audit/auditd.conf"
        rlFileBackup "/var/log/audit/audit.log"
        rlFileBackup "${DISPATCHER_DIR}/plugins.d/syslog.conf"
       
        rlServiceStop "auditd" && sleep 3

        rlRun "sed -i 's/^active.*/active = yes/' ${DISPATCHER_DIR}/plugins.d/syslog.conf" 0
        rlRun "sed -i 's/^flush.*/flush = INCREMENTAL/' /etc/audit/auditd.conf" 0
        rlRun "sed -i 's/^freq.*/freq = 1/' /etc/audit/auditd.conf" 0

        file=$(mktemp)
        
    rlPhaseEnd

    rlPhaseStartTest "log_format = NOLOG"

        # Setup.
        rlRun "sed -i 's/^log_format.*/log_format = NOLOG/' /etc/audit/auditd.conf" 0
        rlRun "service auditd start && sleep 5" 0
        rlRun "PAGER='' service auditd status" 0
        rlRun "auditctl -D" 0
        rlRun "auditctl -a always,exit -W $file -k NOLOG" 0
        mark=$(date "+%T")
        
        # Trigger.
        rlRun "cat $file && sleep 1" 0

        # Verify.
        rlRun "ausearch -ts $mark -m SYSCALL -k NOLOG --raw" 1
        
        # Clean-up.
        rlRun "auditctl -D" 0
        rlRun "service auditd stop && sleep 5" 0
        
    rlPhaseEnd

    rlPhaseStartTest "log_format = RAW"

        # Setup.
        rlRun "sed -i 's/^log_format.*/log_format = RAW/' /etc/audit/auditd.conf" 0
        rlRun "service auditd start && sleep 5" 0
        rlRun "PAGER='' service auditd status" 0
        rlRun "auditctl -D" 0
        rlRun "auditctl -a always,exit -W $file -k RAW" 0
        mark=$(date "+%T")
        
        # Trigger.
        rlRun "cat $file && sleep 1" 0

        # Verify.
        rlRun -s "ausearch -ts $mark -m SYSCALL -k RAW --raw" 0
        last_char=$(cat $rlRun_LOG | grep -o '.$' | xxd -ps -u -l 1)

        # BZ#1406525
        rlAssertNotEquals "Checking that separator is not displayed" "$last_char" "1D"
        
        # Clean-up.
        rlRun "auditctl -D" 0
        rlRun "service auditd stop && sleep 5" 0

    rlPhaseEnd

    rlPhaseStartTest "log_format = ENRICHED"

        # Setup.
        rlRun "sed -i 's/^log_format.*/log_format = ENRICHED/' /etc/audit/auditd.conf" 0
        rlRun "service auditd start && sleep 5" 0
        rlRun "PAGER='' service auditd status" 0
        rlRun "auditctl -D" 0
        rlRun "auditctl -a always,exit -W $file -k ENRICHED" 0
        mark=$(date "+%T")

        # Trigger.
        rlRun "cat $file && sleep 1" 0

        # Verify.
        rlRun -s "ausearch -ts $mark -m SYSCALL -k ENRICHED --raw" 0
        RAW=$( cat $rlRun_LOG | grep 'SYSCALL' | grep 'ENRICHED' | awk -F "\x1D" '{ print $1 }' )
        ENRICHED=$( cat $rlRun_LOG | grep 'SYSCALL' | grep 'ENRICHED' | awk -F "\x1D" '{ print $2 }' )

        rlRun -s "ausearch -ts $mark -m SYSCALL -k ENRICHED -i" 0
        RAW_I=$( cat $rlRun_LOG | grep 'SYSCALL' | grep 'ENRICHED' | awk -F "\x1D" '{ print $1 }' )
        
        rlLog "RAW: $RAW"
        rlLog "RAW (interpreted): $RAW_I"
        rlLog "ENRICHED: $ENRICHED"

        for F in arch syscall auid uid gid euid suid fsuid egid sgid fsgid; do

            R=$( echo $RAW      | tr ' ' '\n' | egrep -i "^${F}=" | awk -F '=' '{print $2}' | tr -d '"' )
            I=$( echo $RAW_I    | tr ' ' '\n' | egrep -i "^${F}=" | awk -F '=' '{print $2}' | tr -d '"' )
            E=$( echo $ENRICHED | tr ' ' '\n' | egrep -i "^${F}=" | awk -F '=' '{print $2}' | tr -d '"' )

            case $F in
                "arch")
                    V=$( uname -m )
                    ;;
                "syscall")
                    if ! rlIsRHEL "<8" || [ $( uname -m ) == "aarch64" ]; then
                        V="openat"
                    else
                        V="open"
                    fi
                    ;;
                "auid")
                    if [ "$R" == "4294967295" ]; then
                        V="unset"
                    else
                        V="root"
                    fi
                    ;;
                *)
                    V="root"
                    ;;
            esac

            rlLog "$F RAW: $R"
            rlLog "$F RAW (interpreted): $I"
            rlLog "$F ENRICHED: $E"
            
            rlAssertEquals "Checking $F (equivalency)" "$I" "$E"
            rlAssertEquals "Checking $F (correctnes)"  "$E" "$V"
        done
        
        # Clean-up.
        rlRun "auditctl -D" 0
        rlRun "rm -f $rlRun_LOG" 0
        rlRun "service auditd stop && sleep 5" 0

    rlPhaseEnd

    rlPhaseStartCleanup

        rlRun "rm -f $file" 0
        rlRun "auditctl -D" 0
        rlFileRestore
        rlServiceRestore "auditd"
        
    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
