#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/options-space-left
#   Description: Sanity of admin_space_left and space_left
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2019 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

# Helper functions
function send_audit_msgs() {
    for i in `seq 1 ${2:-5}` ; do
        if [ $[$i % 2] == 1 ]; then
            auditctl -m ${1}
        else
            auditctl -m ${1}.
        fi
    done
}

rlJournalStart
    rlPhaseStartSetup

        rlAssertRpm "$PACKAGE"
        rlRun "rlImport audit/testing" || rlDie
        rlFileBackup --clean "$auditDaemonConf"
        rlFileBackup --clean "/var/log/audit/"
        rlFileBackup --clean "/var/log/messages"

        # Stop auditd.
        rlServiceStop "auditd" && sleep 3

        # Initial configuration of auditd.
        rlRun "auditConfig 'flush' 'INCREMENTAL'" 0
        rlRun "auditConfig 'freq' '1'" 0
        rlRun "auditConfig 'space_left' '80'" 0
        rlRun "auditConfig 'space_left_action' 'syslog'" 0
        rlRun "auditConfig 'admin_space_left' '40'" 0
        rlRun "auditConfig 'admin_space_left_action' 'suspend'" 0
        
        # Create a fake partition for audit logging (100MB).
        rlRun "mount -t tmpfs -o size=$((1024 * 1024 * 160)) none /var/log/audit" 0
        rlRun "touch /var/log/audit/audit.log" 0
        rlRun "chmod 600 /var/log/audit/audit.log" 0
        rlRun "restorecon -vFR /var/log/audit" 0

        # Start auditd.
        rlServiceStart "auditd" && sleep 3

        # Start rsyslog.
        rlServiceStart "rsyslog" && sleep 3

        # Log message, it should work.
        msg="SETUP_MSG"
        ts=$(date "+%T")
        send_audit_msgs $msg
        rlRun "auditWaitForEvent $ts 'USER' '$msg' 30" 0

        # Check the disk space
        rlLog "$(df -hl /var/log/audit) | tail -1" 0

    rlPhaseEnd

    rlPhaseStartTest

        # Reset audit.log and syslog.
        rlRun "echo ''> /var/log/audit/audit.log" 0 
        rlRun "echo ''> /var/log/messages" 0 

        # Space left <80M trigger space_left default quota in auditd.conf 
        rlRun "dd if=/dev/urandom of=/var/log/audit/blob bs=1M count=90"

        # Check the disk space
        rlLog "$(df -hl /var/log/audit) | tail -1" 0
        
        # Emit message to trigger disk space check.
        msg="SYSLOG_MSG"
        ts=$(date "+%T")
        send_audit_msgs $msg
        rlRun "auditWaitForEvent $ts 'USER' '$msg' 30" 0

        # There should be a warning sent via syslog about low disk space.
        rlRun "auditWaitForSyslogEvent 'Audit daemon is low on disk space for logging' 30" 0

        # Reset audit.log and syslog.
        rlRun "echo ''> /var/log/audit/audit.log" 0 
        rlRun "echo ''> /var/log/messages" 0 
        rlRun "rm -f /var/log/audit/blob" 0

        # Space left <40M trigger admin_space_left default quota in auditd.conf
        rlRun "dd if=/dev/urandom of=/var/log/audit/blob bs=1M count=130" 0

        # Check the disk space
        rlLog "$(df -hl /var/log/audit) | tail -1" 0

        # Emit message to trigger disk space check.
        msg="SYSLOG_MSG"
        ts=$(date "+%T")
        send_audit_msgs $msg
        rlRun "auditWaitForEvent $ts 'USER' '$msg' 30" 0

        # Check that the audit daemon really suspended and sent warning to syslog.
        rlRun "auditWaitForSyslogEvent 'Audit daemon is suspending logging due to low disk space' 30" 0

    rlPhaseEnd

    if ! rlIsRHEL "<8.1" || rlIsRHEL ">=7.7"; then
    rlPhaseStartTest "Allow space left to be expressed as a percentage (BZ#1650670)"

        rlRun "auditConfig 'space_left' '50%'" 0
        rlRun "auditConfig 'space_left_action' 'syslog'" 0
        rlRun "auditConfig 'admin_space_left' '25%'" 0
        rlRun "auditConfig 'admin_space_left_action' 'suspend'" 0

        # Reset audit.log and syslog.
        rlRun "echo ''> /var/log/audit/audit.log" 0 
        rlRun "rm -f /var/log/audit/blob" 0
        rlRun "echo ''> /var/log/messages" 0 

        # Restart auditd.
        rlRun "service auditd restart && sleep 3" 0

        # Space left <50% trigger space_left default quota in auditd.conf 
        rlRun "dd if=/dev/urandom of=/var/log/audit/blob bs=1M count=90"

        # Check the disk space
        rlLog "$(df -hl /var/log/audit) | tail -1" 0
        
        # Emit message to trigger disk space check.
        msg="SYSLOG_MSG"
        ts=$(date "+%T")
        send_audit_msgs $msg
        rlRun "auditWaitForEvent $ts 'USER' '$msg' 30" 0

        # There should be a warning sent via syslog about low disk space.
        rlRun "auditWaitForSyslogEvent 'Audit daemon is low on disk space for logging' 30" 0

        # Reset audit.log and syslog.
        rlRun "echo ''> /var/log/audit/audit.log" 0 
        rlRun "echo ''> /var/log/messages" 0 
        rlRun "rm -f /var/log/audit/blob" 0

        # Space left <50% trigger admin_space_left default quota in auditd.conf
        rlRun "dd if=/dev/urandom of=/var/log/audit/blob bs=1M count=130" 0

        # Check the disk space
        rlLog "$(df -hl /var/log/audit) | tail -1" 0

        # Emit message to trigger disk space check.
        msg="SYSLOG_MSG"
        ts=$(date "+%T")
        send_audit_msgs $msg
        rlRun "auditWaitForEvent $ts 'USER' '$msg' 30" 0

        # Check that the audit daemon really suspended and sent warning to syslog.
        rlRun "auditWaitForSyslogEvent 'Audit daemon is suspending logging due to low disk space' 30" 0
    rlPhaseEnd
    fi

    if ! rlIsRHEL "<8.1"; then
    rlPhaseStartTest "Space left action rotate should happen always (BZ#1718444)"

        rlRun "auditConfig 'max_log_file' '10'" 0
        rlRun "auditConfig 'max_log_file_action' 'keep_logs'" 0
        rlRun "auditConfig 'space_left' '140'" 0
        rlRun "auditConfig 'space_left_action' 'rotate'" 0

        # Reset audit.log and syslog.
        rlRun "echo ''> /var/log/audit/audit.log" 0 
        rlRun "rm -f /var/log/audit/blob /var/log/audit/*" 0
        rlRun "echo ''> /var/log/messages" 0 

        # Restart auditd.
        rlRun "service auditd restart && sleep 3" 0
        rlRun "auditctl -D" 0
        rlRun "auditctl -a never,task" 0

        rlRun "auditctl -s > bz1718444_state" 0 "Show audit status"
        rlFileSubmit "bz1718444_state"
        rlFileSubmit "/etc/audit/auditd.conf" "bz1718444_config"

        # Fill the log to cause rotation.
        rlRun "auditIncreaseLogSize 15" 0
        
        # The following message should end up in the main log.
        ts=$(date "+%T")
        rlRun "auditctl -m 'last-A'" 0
        rlRun "auditWaitForEvent $ts 'USER' 'last-A' 30" 0
        rlAssertGrep "last-A" /var/log/audit/audit.log
        
        # There should be main log and one rotation.
        rlAssertExists "/var/log/audit/audit.log" 0
        rlAssertExists "/var/log/audit/audit.log.1" 0
        rlAssertNotExists "/var/log/audit/audit.log.2" 0

        # Fill another space so that space left quota is used.
        rlRun "dd if=/dev/urandom of=/var/log/audit/blob bs=1M count=10" 0

        # Check the disk space
        rlLog "$(df -hl /var/log/audit) | tail -1" 0
        
        # Trigger space left actions (rotation).
        ts=$(date "+%T")
        for m in "B" "C" "D" "E" "F" "G" "I" "J"; do
            rlRun "auditctl -m 'last-${m}'" 0
            sleep 1
        done
        rlRun "auditWaitForEvent $ts 'USER' 'last-B' 30" 0
        rlRun "auditWaitForEvent $ts 'USER' 'last-C' 30" 0
        rlRun "auditWaitForEvent $ts 'USER' 'last-D' 30" 0
        rlRun "auditWaitForEvent $ts 'USER' 'last-E' 30" 0
        rlRun "auditWaitForEvent $ts 'USER' 'last-F' 30" 0
        rlRun "auditWaitForEvent $ts 'USER' 'last-G' 30" 0
        
        # There should be main log and four rotations exactly because
        # rotations remove initial 10MB rotation and space used drops
        # below space left quota after fourth rotation.
        rlAssertExists "/var/log/audit/audit.log" 0
        rlAssertExists "/var/log/audit/audit.log.1" 0
        rlAssertExists "/var/log/audit/audit.log.2" 0
        rlAssertExists "/var/log/audit/audit.log.3" 0
        rlAssertExists "/var/log/audit/audit.log.4" 0
        rlAssertNotExists "/var/log/audit/audit.log.5" 0
        
        # Check the disk space and some debugging data.
        rlLog "$(df -hl /var/log/audit) | tail -1" 0
        ls -la /var/log/audit/
        grep -R last /var/log/audit
                
        # Reset audit.log and syslog.
        rlRun "echo ''> /var/log/audit/audit.log" 0 
        rlRun "echo ''> /var/log/messages" 0 
        rlRun "rm -f /var/log/audit/blob" 0
        rlRun "rm -f /var/log/audit/*.log.*" 0

    rlPhaseEnd
    fi

    rlPhaseStartCleanup

        rlRun "auditctl -D" 0
        rlRun "service auditd stop && sleep 5" 0
        rlRun "umount -f /var/log/audit" 0
        rlServiceRestore "auditd"

    rlPhaseEnd

    rlJournalPrintText

rlJournalEnd
