#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/packaged-rules-files
#   Description: Test sanity of packaged rules files
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2014 Red Hat, Inc.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

# RHEL6, nispom.rules, -F dir is internally translated to -w, hence no match
EXCEPTIONS[0]="-a always,exit -F dir=/etc/NetworkManager/ -F perm=wa -F key=system-locale"

# RHEL6, stig.rules, -F dir is internally translated to -w, hence no match
EXCEPTIONS[1]="-a always,exit -F dir=/etc/selinux/ -F perm=wa -F key=MAC-policy"

# RHEL7, nispom.rules, missing -p is internally translated to -rwxa, hence no match
EXCEPTIONS[2]="-w /var/log/audit/ -k audit-logs"

# RHEL7, stig.rules, missing -p is internally translated to -rwxa, hence no match
EXCEPTIONS[3]="-w /etc/sudoers.d/ -p wa -k actions"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[4]="-a always,exit -F path=/etc/group -F perm=wa -F key=10.2.5.c-accounts"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[5]="-a always,exit -F path=/etc/passwd -F perm=wa -F key=10.2.5.c-accounts"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[6]="-a always,exit -F path=/etc/gshadow -F perm=wa -F key=10.2.5.c-accounts"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[7]="-a always,exit -F path=/etc/shadow -F perm=wa -F key=10.2.5.c-accounts"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[8]="-a always,exit -F path=/etc/security/opasswd -F perm=wa -F key=10.2.5.c-accounts"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[9]="-a always,exit -F dir=/var/log/audit/ -F perm=r -F auid>=1000 -F auid!=unset -F key=10.2.3-access-audit-trail"
# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[10]="-a always,exit -F path=/usr/sbin/ausearch -F perm=x -F key=10.2.3-access-audit-trail"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[11]="-a always,exit -F path=/usr/sbin/aureport -F perm=x -F key=10.2.3-access-audit-trail"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[12]="-a always,exit -F path=/usr/sbin/aulast -F perm=x -F key=10.2.3-access-audit-trail"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[13]="-a always,exit -F path=/usr/sbin/aulastlogin -F perm=x -F key=10.2.3-access-audit-trail"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[14]="-a always,exit -F path=/usr/sbin/auvirt -F perm=x -F key=10.2.3-access-audit-trail"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[15]="-a always,exit -F arch=b64 -S setuid -Fa0=0 -F exe=/usr/bin/su -F key=10.2.5.b-elevated-privs-session"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[16]="-a always,exit -F arch=b32 -S setuid -Fa0=0 -F exe=/usr/bin/su -F key=10.2.5.b-elevated-privs-session"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[17]="-a always,exit -F dir=/var/log/audit/ -F perm=wa -F key=10.5.5-modification-audit"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[18]="-a always,exit -F arch=b32 -S chmod,fchmod,fchmodat -F auid>=1000 -F auid!=4294967295 -F key=perm_mod"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[19]="-a always,exit -F arch=b64 -S chmod,fchmod,fchmodat -F auid>=1000 -F auid!=4294967295 -F key=perm_mod"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[20]="-a always,exit -F arch=b32 -S lchown,fchown,chown,fchownat -F auid>=1000 -F auid!=4294967295 -F key=perm_mod"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[21]="-a always,exit -F arch=b64 -S chown,fchown,lchown,fchownat -F auid>=1000 -F auid!=4294967295 -F key=perm_mod"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[22]="-a always,exit -F arch=b32 -S setxattr,lsetxattr,fsetxattr,removexattr,lremovexattr,fremovexattr -F auid>=1000 -F auid!=4294967295 -F key=perm_mod"

# RHEL7, pci-dss-w31.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[23]="-a always,exit -F arch=b64 -S setxattr,lsetxattr,fsetxattr,removexattr,lremovexattr,fremovexattr -F auid>=1000 -F auid!=4294967295 -F key=perm_mod"

# RHEL9, 30-pci-dss-v31.rule, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[24]="-a always,exit -F path=/etc/sudoers -F perm=wa -F key=10.2.2-priv-config-changes"

# RHEL9, 30-pci-dss-v31.rule, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[25]="-a always,exit -F dir=/etc/sudoers.d/ -F perm=wa -F key=10.2.2-priv-config-changes"

# RHEL9, 30-ospp-v42.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[26]="-a always,exit -F path=/etc/sudoers -F perm=wa -F key=special-config-changes"

# RHEL9, 30-ospp-v42.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[27]="-a always,exit -F dir=/etc/sudoers.d/ -F perm=wa -F key=special-config-changes"

# RHEL9, 44-installers.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[28]="-a always,exit -F perm=x -F path=/usr/bin/dnf -F key=software-installer"
EXCEPTIONS[29]="-a always,exit -F perm=x -F path=/usr/bin/dnf-3 -F key=software-installer"

# RHEL9, 44-installers.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[30]="-a always,exit -F perm=x -F path=/usr/bin/yum -F key=software-installer"

# RHEL9, 44-installers.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[31]="-a always,exit -F perm=x -F path=/usr/bin/pip -F key=software-installer"

# RHEL9, 44-installers.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[32]="-a always,exit -F perm=x -F path=/usr/bin/npm -F key=software-installer"

# RHEL9, 44-installers.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[33]="-a always,exit -F perm=x -F path=/usr/bin/cpan -F key=software-installer"

# RHEL9, 44-installers.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[34]="-a always,exit -F perm=x -F path=/usr/bin/gem -F key=software-installer"

# RHEL9, 44-installers.rules, for -w is -a always,exit added internally, hence no match
EXCEPTIONS[35]="-a always,exit -F perm=x -F path=/usr/bin/luarocks -F key=software-installer"

# RHEL9, [30-ospp-v42.rules, 30-pci-dss-v31.rules, 30-stig.rules], for -w is -a always, exit added internally, hence no match
EXCEPTIONS[36]="-a always,exit -F path=/usr/bin/pkexec -F perm=x -F key=maybe-escalation"

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm $PACKAGE
        rlServiceStop "auditd" && sleep 3
        rlRun "service auditd start && sleep 3" 0 "Starting auditd service"
        rlRun "auditctl -D" 0 "Removing all rules"

        # The list of rules files provided by the package.
        if [ -z "$RULES_FILES" ]; then
            RULES_FILES=$(rpm -ql audit | egrep  "^/usr/.*/.+\.rules$")
            rlLog "There are $(echo $RULES_FILES | tr ' ' '\n' | wc -l) rule files provided"
        fi

    rlPhaseEnd

    # The list of rules files provided by the package.
    # Filesystems rules are ignored since they rely on architecture and kernel.
    if [ -z "$RULES_FILES" ]; then
        RULES_FILES=$(rpm -ql audit | egrep  "^/usr/.*/.+\.rules$" | grep -v "filesystems")
    else
        RULES_FILES=$(ls -1 $RULES_FILES)
    fi

    # All sample rules must be located in:
    #   * /usr/share/doc/audit*/rules in RHEL-8.1 and older;
    #   * /usr/share/audit/sample-rules in RHEL-8.2 and newer.
    rlPhaseStartTest "Rules location"
        if ! rlIsRHEL "<8.2"; then
            exp_loc="/usr/share/audit/sample-rules"
        else
            exp_loc="$(ls -d  /usr/share/doc/audit*/rules)"
        fi

        for R in $RULES_FILES; do 
            act_loc="$(echo $R | sed 's|\(.*\)/.*|\1|')"
            rlAssertEquals "Checking $R" "$act_loc" "$exp_loc"
        done
    rlPhaseEnd

    # Now check every rules file.
    # Filesystems rules are ignored since they rely on architecture and kernel.
    for F in $(echo $RULES_FILES | tr ' ' '\n' | grep -v "filesystems"); do        
        rlPhaseStartTest "$F"

            rlRun "cat $F"

            # Disable audit locking (requires restart to unlock).
            backup_done=0
            if grep "-e 2" $F; then
                backup_done=1
                rlFileBackup $F
                sed -i '/-e 2/d' $F
            fi

            # Load, continue only if succeeded, on s390x there is no accept
            # syscall and hence skipp 71-networking.rules.
            [[ $F =~ 71-networking.rules ]] && [[ $(uname -m) == "s390x" ]] && continue
            rlRun "auditctl -R ${F}" 0 "Loading $F" || continue

            # Verify that all rules are loaded.
            auditctl -l > rules.list

            COUNT_LOADED=$(cat rules.list | grep -v "No rules" | wc -l)
            COUNT_INFILE=$(cat ${F} | egrep "^\-[aw]" | wc -l)
            rlAssertEquals "Checking number of rules loaded" $COUNT_LOADED $COUNT_INFILE

            # Check that each rule is loaded correctly.
            while read -r RULE; do

                # Ignore blank line and comments.
                echo $RULE | egrep -q "^\-" || continue

                # Ignore rules changing state.
                echo $RULE | egrep -q "^\-[Dbfci]" && continue
                echo $RULE | egrep -q "^--loginuid-immutable" && continue
                echo $RULE | egrep -q "^--backlog_wait_time" && continue

                # Get a list of rule components.
                IFS="[ ,]" read -ra OPTS <<<$RULE
                echo "Inspected rule: $RULE"
                echo -n "Components: "
                for OPT in ${OPTS[@]}; do echo -n "($OPT) "; done;
                echo ""

                # Check that there is a loaded rule for each component.
                rule_found=0
                while read -r LOADED_RULE; do
                    match=1
                    for OPT in ${OPTS[@]}; do
                        [[ "$OPT" =~ ^- ]] && OPT="\\$OPT";
                        [[ "$OPT" =~ 4294967295 ]] && OPT=$(echo $OPT | sed "s/4294967295/-1/g")
                        [[ "$OPT" =~ unset ]] && OPT=$(echo $OPT | sed "s/unset/-1/g")
                        [[ "$OPT" =~ a1 ]] && continue
                        [[ "$OPT" =~ a2 ]] && continue

                        # The RHEL9 package ships some rules using "-F uid=<user>", while
                        # previous versions shipped them using "-Fuid=<user>", so let's
                        # parse both alternatives, since for audit both work and generate
                        # a correct rule.
                        # For reference, this is the upstream commit that "fixed" the typo:
                        # https://github.com/linux-audit/audit-userspace/commit/aedb3bfc
                        if [[ "$OPT" =~ Fuid ]] || [[ "$OPT" =~ ^uid ]]; then
                            opt_login=$(echo "$OPT" | awk -F '=' '{print $2}');
                            opt_uid=$(getent passwd | grep $opt_login | awk -F ':' '{print $3}')
                            [[ "$OPT" =~ Fuid ]] && OPT="\\-F uid=$opt_uid";
                            [[ "$OPT" =~ ^uid ]] && OPT="uid=$opt_uid"

                        fi
                        # [[ "$OPT" =~ perm ]] && {
                        #     opt_acl=$(echo "$OPT" | awk -F '=' '{print $2}');
                        #     OPT="\\-p $opt_acl";
                        #     echo "OPT=$OPT";
                        # }                        
                        echo $LOADED_RULE | grep -q "$OPT" || { \
                            match=0
                            # echo $LOADED_RULE
                            # echo "$OPT not found"
                            break
                        }
                    done
                    if [ $match -eq 1 ]; then
                        echo -e "Matched loaded rule: $LOADED_RULE\n"
                        rule_found=1
                        break;
                    fi
                done <rules.list

                # If some component is missing, check exceptions.
                if [ $rule_found -eq 0 ]; then
                    exception_found=0
                    for i in $( seq 0 $[${#EXCEPTIONS[@]}-1] ); do
                        if [ "$RULE" == "${EXCEPTIONS[$i]}" ]; then
                            rlLog "Rule \"$RULE\" is not loaded - an exception"
                            exception_found=1
                            break
                        fi
                    done
                    [ $exception_found -eq 0 ] && rlFail "Rule \"$RULE\" is not loaded"
                fi

            done <${F}
            
            # Clean-up.
            rlRun "auditctl -D" 0 "Removing all rules"
            if [ "$backup_done" ==  "1" ]; then
                rlFileRestore
            fi
        rlPhaseEnd
    done

    rlPhaseStartCleanup

        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
