#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/permissions
#   Description: Test for permissions of files related to audit
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2010 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

function check() {
    local object=$1
    local user=$2
    local group=$3
    local perms=$4
    local context=$5
    local rc=0
    
    if ! test -e $object; then
        rlLogError "Object ($object) does not exist!"
        return 255
    fi
    
    o_user=$( stat -c "%U" $object )
    o_group=$( stat -c "%G" $object )
    o_perms=$( stat -c "%a" $object )
    o_context=$( stat -c "%C" $object )

    if [ "$user" != "$o_user" ]; then
        rlLog "$object owner should be $owner ($o_owner)!"
        rc=1
    fi
    if [ "$group" != "$o_group" ]; then
        rlLog "$object group should be $group ($o_group)!"
        rc=1
    fi
    if [ "$perms" != "$o_perms" ]; then
        rlLog "$object permissions should be $perms ($o_perms)!"
        rc=1
    fi
    if [ "$context" != "$o_context" ]; then
        rlLog "$object context should be $context ($o_context)!"
        rc=1
    fi

    return $rc    
}

rlJournalStart

    rlPhaseStartSetup

        rlAssertRpm $PACKAGE
        rlFileBackup --clean /etc/audit/auditd.conf        
        rlFileBackup --clean /var/log/audit/ 
        rlRun "rm -f /var/log/audit/audit.log.*" 0-255 "Removing rotate audit logs"
        rlServiceStart "auditd"
        
    rlPhaseEnd

    rlPhaseStartTest "Default setting"

        # Setup.
        rlRun "sed -i '/log_group/d' /etc/audit/auditd.conf" 0 "Removing log_group setting"
        rlRun "service auditd restart && sleep 5" 0 "Restarting auditd"
        rlRun "kill -s SIGUSR1 $( pidof auditd )" 0 "Rotating audit logs "

        # Directories.
        rlRun "check /etc/audit/ root root 750 system_u:object_r:auditd_etc_t:s0" 0
        rlRun "check /etc/audit/rules.d/ root root 750 system_u:object_r:auditd_etc_t:s0" 0
        rlRun "check /var/log/audit/ root root 700 system_u:object_r:auditd_log_t:s0" 0

        # Files.
        rlRun "check /etc/audit/auditd.conf root root 640 system_u:object_r:auditd_etc_t:s0" 0
        rlRun "check /etc/audit/auditd.conf root root 640 system_u:object_r:auditd_etc_t:s0" 0
        rlRun "check /etc/audit/audit.rules root root 640 system_u:object_r:auditd_etc_t:s0" 0
        rlRun "check /var/log/audit/audit.log root root 600 system_u:object_r:auditd_log_t:s0" 0
        rlRun "check /var/log/audit/audit.log.1 root root 400 system_u:object_r:auditd_log_t:s0" 0

        # # Special case - stopped auditd.
        # rlRun "service auditd stop && sleep 5" 0 "Stopping auditd"
        # rlRun "check /var/log/audit/audit.log.1 root root 400 system_u:object_r:auditd_log_t:s0" 0
        # # rlRun "check /var/log/audit/audit.log root root 400 system_u:object_r:auditd_log_t:s0" 0
        # rlRun "service auditd start && sleep 5" 0 "Starting auditd"
        
    rlPhaseEnd

    rlPhaseStartTest "Customized log_group setting"

        # Setup.
        rlRun "groupadd audit_test_group" 0 "Creating testing group"
        rlRun "echo 'log_group = audit_test_group' >>/etc/audit/auditd.conf" 0 "Setting log_group"
        rlRun "service auditd restart && sleep 5" 0 "Restarting auditd"
        rlRun "kill -s SIGUSR1 $( pidof auditd )" 0 "Rotating audit logs "

        # Directories.
        rlRun "check /var/log/audit/ root audit_test_group 750 system_u:object_r:auditd_log_t:s0" 0

        # Files.
        rlRun "check /var/log/audit/audit.log root audit_test_group 640 system_u:object_r:auditd_log_t:s0" 0
        rlRun "check /var/log/audit/audit.log.1 root audit_test_group 440 system_u:object_r:auditd_log_t:s0" 0

        # # Special case - stopped auditd.
        # rlRun "service auditd stop && sleep 5" 0 "Stopping auditd"
        # # rlRun "check /var/log/audit/audit.log root audit_test_group 640 system_u:object_r:auditd_log_t:s0" 0
        # rlRun "check /var/log/audit/audit.log.1 root root 440 system_u:object_r:auditd_log_t:s0" 0
        # rlRun "service auditd start && sleep 5" 0 "Starting auditd"

        # Cleanup.
        rlRun "groupdel audit_test_group" 0 "Removing testing group"
    
    rlPhaseEnd        

    rlPhaseStartCleanup

        rlFileRestore
        rlServiceRestore "auditd"

    rlPhaseEnd

rlJournalPrintText

rlJournalEnd
