#!/bin/bash
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Sanity/syscalls-smoke
#   Description: Sanity test for auditing specific syscalls
#   Author: Ondrej Moris <omoris@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2013 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include rhts environment
. /usr/share/beakerlib/beakerlib.sh

PACKAGE="audit"

case $(uname -i) in
    "x86_64" | "ppc64" | "s390x" | "ppc64le" | "aarch64" ) 
        ARCH="b64"
        ;;
    *)
        ARCH="b32"
        ;;
esac

rlJournalStart

  rlPhaseStartSetup
    rlCheckRpm "audit" || rlDie    
    rlRun "rlImport audit/testing" || rlDie
    rlRun "gcc -g -W -Wall -Wundef -D_GNU_SOURCE do_mq_open.c -o do_mq_open -lrt"
    rlAssertExists "do_mq_open" || rlDie

    rlFileBackup --clean "/var/log/audit/" 
    rlFileBackup --clean "$auditDaemonConf" 
    rlServiceStop "auditd" && sleep 3
    rlRun "auditConfig 'freq' '1'" 0
    rlRun "auditConfig 'flush' 'INCREMENTAL'" 0
    rlRun "service auditd start && sleep 3" 0
    rlRun "echo '' > $auditLog"
    rlRun "auditctl -D" 0
    rlRun "auditctl -a exit,always -F arch=$ARCH -S mq_open" 0
    rlRun "gcc -g -W -Wall -Wundef -D_GNU_SOURCE do_mq_open.c -o do_mq_open -lrt" 0 "Build do_mq_open"
  rlPhaseEnd

  rlPhaseStartTest "mq_open"
    ts=$(date "+%T")
    rlRun "./do_mq_open /test create:rdwr" 0
    rlAssertExists "/dev/mqueue/test" 
    rlRun "auditWaitForEvent $ts 'MQ_OPEN' '' '30' test.msg" 0
    rlAssertEquals "Checking MQ_OPEN event" $(grep -c MQ_OPEN test.msg) 1
    rlAssertEquals "Checking SYSCALL event" $(grep -c SYSCALL test.msg) 1
  rlPhaseEnd

  rlPhaseStartCleanup
    rlRun "auditctl -D" 0
    rlRun "rm -f /dev/mqueue/test log do_mq_open" 0
    rlRun "service auditd stop && sleep 3" 0
    rlFileRestore
    rlServiceRestore "auditd"
  rlPhaseEnd

rlJournalEnd

rlJournalPrintText
