#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   runtest.sh of /CoreOS/audit/Regression/bz869555-ausearch-i-does-not-interpret-execve-arguments
#   Description: Test for BZ#869555 (ausearch -i does not interpret execve arguments)
#   Author: Martin Zember <mzember@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2014 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Include Beaker environment
. /usr/share/beakerlib/beakerlib.sh || exit 1

PACKAGE="audit"

rlJournalStart
    rlPhaseStartSetup
        rlAssertRpm $PACKAGE
        rlRun "TmpDir=\$(mktemp -d)" 0 "Creating tmp directory"
        rlRun "pushd $TmpDir"
        # restart the service because upgrade/downgrade does not start the new binary
        rlServiceStop auditd && sleep 3
        rlRun "service auditd start && sleep 3" 0
	rlRun "auditctl -D" 
        # TODO RHEL 7 should be handled by beakerlib implementation and to hide implementation details
        # add audit filter for 32-bit
        rlRun "auditctl -a exit,always -F arch=b32 -S execve" 0,255 "Add audit filter for 32-bit, it will not harm if not applicable"
        export RET_32=$?
        # add audit filter for 64-bit, it will not harm if not applicable
        rlRun "auditctl -a exit,always -F arch=b64 -S execve" 0,255 "Add audit filter for 64-bit, it will not harm if not applicable"
        export RET_64=$?
        rlRun "test $RET_32 = 0 || test $RET_64 = 0" 0 "At least one filter was added successfully"
    rlPhaseEnd

    rlPhaseStartTest
        # check if any audit messages are logged
        rlRun "ausearch -i -if /var/log/audit/audit.log >audit.log"
        # this is not enough, no EXECVE record: rlRun "echo check if any audit messages are logged"
        rlRun "man non-existing-cmd" 0-255
        sleep 1
        rlRun "ausearch -i -if /var/log/audit/audit.log >auditnew.log"
        rlRun "diff audit{,new}.log" 0,1
        rlRun "diff audit{,new}.log | grep 'EXECVE.*=man '" 0 "Check if any audit messages with EXECVE are logged"

        # is it splitting?
        rlRun "ausearch -i -if /var/log/audit/audit.log >audit.log"
        rlRun "man `perl -e "print 'b'x20000"`" 0-255 # we make one long argument. If we provide a big number of arguments, we split the record, but we do not get any aN_len=...
        sleep 1
        rlRun "ausearch -i -if /var/log/audit/audit.log >auditnew.log"
        rlRun "diff audit{,new}.log" 0,1
        rlRun "diff audit{,new}.log | grep 'EXECVE.*_len='" 0 "An audit record is splitted"
        # is it interpreting numbers?
        rlRun "diff audit{,new}.log | grep 'EXECVE.*_len=[0-9]'" 0 "The argument length is interpreted as a number"

    rlPhaseEnd

    rlPhaseStartCleanup
        # remove audit filter for 32-bit
        rlRun "auditctl -d exit,always -F arch=b32 -S execve" 0,255 "Remove audit filter for 32-bit, it will not harm if not applicable"
        # remove audit filter for 64-bit
        rlRun "auditctl -d exit,always -F arch=b64 -S execve" 0,255 "Remove audit filter for 64-bit, it will not harm if not applicable"
        rlRun "popd"
        rlRun "rm -r $TmpDir" 0 "Removing tmp directory"
    rlPhaseEnd
rlJournalPrintText
rlJournalEnd
